/*=============================================================================

                  ***  Bag of Incomplete Futures Class  ***

 ============================================================================*/

#include "circlist.C"  // DSC Circular List Methods

class BIF              // BIF Task Scheduling Base Class
{ 
 //
 // BIF Data
 //
  private:
    int  NumberofTasks;     // Number of subproblems
    int  SuccessPercentage; // Successful percent completion
    char *TaskSourceFile;   // Subproblem source file name
    FILE *scr;              // Parent task scratch file name
    int  CPU_Estimate;      // Estimated CPU usage (lo, med, hi)
    int  Memory_Estimate;   // Estimated memory usage (in megs)
    int  Coroutines;        // Flag indicating BIF contains coroutines
    circlist *TaskList;     // Pointer to list of subtasks

  public:          

 //
 // Constructors
 //
    //
    // This constructor will create a dummy cell in a list of heterogeneous
    //   tasks to be built by AddTask().
    //
    BIF (int sp, FILE *sf);

    //
    // This constructor will build a list of homogeneous tasks based on
    //   the parameter list.  Heterogeneous tasks may be later added with
    //   AddTask().
    //
    BIF (int nt, int sp, char *tsf, FILE *sf, int cpu, int mem);

 //
 // BIF Methods
 //
    //
    // Method to declare BIF as a bag of coroutines
    // 
    void MakeCoroutines (void);

    //
    // Method to build a list of HOMOGENEOUS subtasks
    //
    void BuildTaskList (char *, int, int);

    //
    // Method to run a percentage of tasks in a BIF.  The last parameter is
    //   a flag, which is set to 1 for coroutines so that RunTasks can skip
    //   its submit step while monitoring sleeping coroutines.
    //
    int RunTasks (int, int = 0);

    //
    // Method to return data file from a successfully completed task
    //
    char *GetDataFile (void);

    //
    // Method to submit tasks to DSC
    //
    int SubmitTasks (void);

    //
    // Add a task to a HOMOGENEOUS BIF.
    //
    void AddTask (void);

    //
    // Add a task to a HETEROGENEOUS BIF.  The last paramenter is a flag,
    //   which defaults to "heterogeneous == true".
    //
    void AddTask (int, char *, char *, char *, int, int, int = 1);

    //
    // Remove a task from a HOMOGENEOUS BIF.
    //
    void RemoveTask (void);

    //
    // Remove a task from a HETEROGENEOUS BIF.
    //
    void RemoveTask (int);

    //
    // Mark a task complete.
    //
    void MarkComplete (long, int);

    //
    // Mark all tasks complete.
    //
    void MarkComplete (void);

    //
    // Wait for a percentage of subtasks to complete.
    //
    int Wait (double, int, int);

    //
    // Kill incomplete subtasks.  The default parameter, if set to 1,
    //   will tell the routine to issue a dscpr_kill() to all subtasks.
    //
    int Kill (int = 0);

    //
    // Resume sleeping coroutines.
    //
    int ResumeTasks (int);

    //
    // Create fid, build and exec strings to be passed to dscpr_sub().
    //
    void BuildStrings (int, char *, char *, char *, char *);

    //
    // Refresh BIF by deleting all tasks from task list.
    //
    void Empty (void);
};
