/*****************************************************************************/
/*                                                                           */
/*                                circlist.C                                 */
/*                                                                           */
/*****************************************************************************/

#include "circlist.h"            // circular list class definitions

/*****************************************************************************/
/*                                                                           */
/* method:  cell::cell (void)                                                */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Default constructor.                                             */
/*                                                                           */
/*****************************************************************************/
cell::cell (FILE *scratch)
{
     scr = scratch;

#ifdef LISTDEBUG
fprintf (scr, "circlist : cell::cell()     : creating cell\n");
fflush (scr);
#endif

     // Point cell at itself.

     prev = this;
     next = this;

     // Initialize dummy cell's information.

     info = new INFO;
     info -> index = -1;
     for (int i = 0 ; i < DSC_STRINGZ ; i++) {
          info -> fid[i] = '\0';
          info -> build[i] = '\0';
          info -> exec[i] = '\0';
     }
     info -> cpu = -1;
     info -> mem = -1;
     info -> complete = 0;
     info -> success = 0;
}

/*****************************************************************************/
/*                                                                           */
/* method:  cell::cell (cell *, cell *, FILE *)                              */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose:                                                                  */
/*                                                                           */
/*****************************************************************************/
cell::cell (cell *p, cell *n, FILE *scratch)
{ 
     scr = scratch;

#ifdef LISTDEBUG
fprintf (scr, "circlist : cell::cell()     : creating cell\n");
fflush (scr);
#endif

     info = new INFO;  prev = p;  next = n; 
} 

/*****************************************************************************/
/*                                                                           */
/* method:  cell *cell::advance (char *node)                                 */
/*                                                                           */
/* inputs:  string indicating direction in which to move                     */
/*                                                                           */
/* outputs: none                                                             */
/*                                                                           */
/* return:  pointer to next cell                                             */
/*                                                                           */
/* purpose: Move forward or back in list.                                    */
/*                                                                           */
/*****************************************************************************/
cell *cell::advance (char *node)
{
     if (!strcmp (node, "next")) 
          return next;
     else
          return prev;
}

/*****************************************************************************/
/*                                                                           */
/* method:  circlist::circlist()                                             */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Circular list constructor                                        */
/*                                                                           */
/*****************************************************************************/
circlist::circlist (FILE *scratch)
{ 
     scr = scratch;
     rear = new cell (scr);
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose:                                                                  */
/*                                                                           */
/*****************************************************************************/
int circlist::empty (void)
{
     return (rear == rear -> next);
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Add an item to rear of list.                                     */
/*                                                                           */
/*****************************************************************************/
cell *circlist::add (INFO *x)
{
#ifdef LISTDEBUG
fprintf (scr, "circlist : circlist::add()  : adding cell\n");
fflush (scr);
#endif

     cell *temp = rear -> next;   // local declaration

     rear -> info = x;
     rear = rear -> next = new cell (rear, rear -> next, scr);
     temp -> prev = rear;

     return rear -> prev;         // return cell just added
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Pop an item from head of list.                                   */
/*                                                                           */
/*****************************************************************************/
INFO *circlist::pop (void)
{

     if (empty ()) return 0;

#ifdef LISTDEBUG
fprintf (scr, "circlist : circlist::pop()  : deleting cell\n");
fflush (scr);
#endif

     cell *front = head ();
     rear -> next = front -> next;
     front -> next -> prev = rear;
     INFO *x = front -> info;
     delete front;
     return x;
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Delete an item from middle of list.                              */
/*                                                                           */
/*****************************************************************************/
void circlist::remove (cell *x)
{
#ifdef LISTDEBUG
fprintf (scr, "circlist : circlist::delete()  : deleting cell\n");
fflush (scr);
#endif
     x -> prev -> next = x -> next;
     x -> next -> prev = x -> prev;
     delete x;
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Add an item to rear of list.                                     */
/*                                                                           */
/*****************************************************************************/
INFO *circlist::data (cell *c)
{
     return c -> info;
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Return pointer to head of list.                                  */
/*                                                                           */
/*****************************************************************************/
cell *circlist::head (void)
{
     return rear -> next;
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Return pointer to tail of list.                                  */
/*                                                                           */
/*****************************************************************************/
cell *circlist::tail (void)
{
     return rear -> prev;
}

/*****************************************************************************/
/*                                                                           */
/* method:                                                                   */
/*                                                                           */
/* inputs:                                                                   */
/*                                                                           */
/* outputs:                                                                  */
/*                                                                           */
/* return:                                                                   */
/*                                                                           */
/* purpose: Return 1 if at end of list, 0 if not.                            */
/*                                                                           */
/*****************************************************************************/
int circlist::atEnd (cell *x)
{
     return x == rear;
}
