/*=============================================================================

                     *** DSC Task Circular List Class ***       

 ============================================================================*/

#include "dscpr.C"     // DSC C++ User Library

#define LISTDEBUG 1

//------------------------
// INFORMATION DATA TYPE:
//------------------------
struct INFO
{
    long index;
    char fid[DSC_STRINGZ];
    char build[DSC_STRINGZ];
    char exec[DSC_STRINGZ];
    int  cpu;
    int  mem;
    int  complete;
    int  success;
};

//-------------------------
// LINKED LIST CELL CLASS:
//-------------------------
class cell
{
 //
 // Private Data
 //
  private:
    INFO *info;
    cell *prev,
         *next;
    FILE *scr;

 //
 // Constructors
 //
    //
    // Create a cell that points to itself.
    //
    cell (FILE *scratch);

    //
    // Create a cell in list.
    //
    cell (cell *p, cell *n, FILE *scratch);

 //
 // Class That Can Use Private Data
 //
    friend class circlist;

 //
 // Public Methods
 //
  public:
    cell *advance (char *node);      // Move forward or back in list.
};

//----------------------
// CIRCULAR LIST CLASS:
//----------------------
class circlist
{
 //
 // Private Data
 //
  private:
    cell *prev;
    cell *rear;
    FILE *scr;

 //
 // Constructors
 //
  public:
    circlist (FILE *scratch);

 //
 // Public Methods
 //
    cell *add   (INFO *);      // Add an item to rear of list.
    INFO *pop   (void);        // Pop an item from head of list.
    void remove (cell *);      // Delete an item from within list.
    cell *head  (void);        // Return pointer to head of list.
    cell *tail  (void);        // Return pointer to tail of list.
    int  atEnd  (cell *);      // Return 1 if at end of list, 0 if not.
    int  empty  (void);        // Return 1 if list is empty.
    INFO *data  (cell *);      // Return information in specified cell.
};
