;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;; Package : dscpr_lisp.lsp                                                ;;;
;;;                                                                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;; Functions                                                               ;;;
;;;     dscpr_sub     - for problems to start subproblems                   ;;;
;;;     dscpr_wait    - wait for one or more subproblems to complete        ;;;
;;;     dscpr_kill    - kill specified subproblem                           ;;;
;;;     dscpr_next    - wait for next subproblem to complete                ;;;
;;;     dscdbg_start  - debuggin functions                                  ;;;
;;;                                                                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;;       Link functions to the LISP environment        ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(si:faslink "dsc_lisp_dbg.o" "-lc")
(si:faslink "dsc_lisp_wait.o" "-lc")
(si:faslink "dsc_lisp_sub.o" "-lc")
(si:faslink "dsc_lisp_kill.o" "-lc")
(si:faslink "dsc_lisp_next.o" "-lc")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;;           Define Estimated MEM Use constants        ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq high_cpu 240)
(setq med_cpu 160)
(setq low_cpu 80)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : string-listofint                         ;;;
;;;                                                     ;;;
;;;   Input : string                                    ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : List of character codes                  ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun string-listofint (instring)
 (let ( (len (length instring))
        (count 0)
        (listofint '())
       )
   (loop
    (cond ( (not (= len count))
            (setq listofint (append listofint 
                             (list (char-int (aref instring count)))
                            )
            )
            (setq count (+ 1 count))
          )
          ( (= len count) (return) )
    ) ; End of cond
   ) ; End of loop
 (setq listofint listofint)
 ) ; End of let
) ; End of string-listofchar

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : fill                                     ;;;
;;;                                                     ;;; 
;;;  Set the elements of inlist equal to number         ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun fill (inlist number)
 (cond ( (null inlist) nil)
       ( t (eval (list 'setq (car inlist) number))
           (fill (cdr inlist) number)
       )
  ) ; End of cond
) ; End of fill

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : dsc_copysub < subproblem                 ;;;
;;;                              listofsubproblems >    ;;;
;;;                                                     ;;;
;;; subproblem : "source file name or path of file"     ;;;
;;;                                                     ;;;
;;; listofsubproblems : ("target1" ... "targetn")       ;;;
;;;                                                     ;;;
;;;                                                     ;;;
;;;  Copy the file subproblem to the file names in the  ;;;
;;;  list of subproblems.                               ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

(defun dsc_copysub (subproblem listofsubproblems)
 (copysub_sub subproblem listofsubproblems)
) ; End of dsc_copysub

(defun copysub_sub (subprob listofsubprob)
 (cond ( (null listofsubprob) nil )
       ( t 
           (setq source_file 
                 (open subprob :direction
                               :input)
           )
           (setq outfile (car listofsubprob) )
           (setq outpntr 
                 (open outfile :direction
                               :output
                               :if-exists :overwrite
                               :if-does-not-exist :create)
           )
           (setq in (read source_file nil))
           (loop
            (cond ( (equal in nil) (return) )
                  ( t (print in outpntr)
                      (setq in (read source_file nil))
                  )
            ) ; End cond
           ) ; End of loop
           (close outpntr)
           (close source_file)
           (copysub_sub subprob (cdr listofsubprob)) 
       )
 ) ; End of cond
) ; End of copysub_sub         

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dscpr_sub                                ;;;
;;;                                                     ;;;
;;;   Input : fid - file id of problem file             ;;;
;;;           (Max Len ... 48)                          ;;;
;;;           types - types of input files (c,l,o,i)    ;;;
;;;           (Max Len ... 3)                           ;;;
;;;           build - build command                     ;;;
;;;           (Max Len ... 48)                          ;;;
;;;           exec - execute command                    ;;;
;;;           (Max len ... 150)                         ;;;
;;;           cpu - expected cpu use                    ;;;
;;;           mem - extpected mem use                   ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : index to new subproblem in DSC server    ;;;
;;;            database                                 ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dscpr_sub (fid types build exec cpu mem)
 (let ( 
        (flen 0)
        (tlen 0)
        (blen 0)
        (elen 0)
        (flist '(f1 f2 f3 f4 f5 f6 f7 f8 f9 f10 f11 f12 f13 f14 f15 f16 
               f17 f18 f19 f20 f21 f22 f23 f24 f25 f26 f27 f28 f29 f30 f31 f32
               f33 f34 f35 f36 f37 f38 f39 f40 f41 f42 f43 f44 f45 f46 f47 f48))
        (tlist '(t1 t2 t3))
        (blist '(b1 b2 b3 b4 b5 b6 b7 b8 b9 b10 b11 b12 
                 b13 b14 b15 b16 b17 b18 b19 b20 b21 b22 b23 b24
                 b25 b26 b27 b28 b29 b30 b31 b32 b33 b34 b35 b36
                 b37 b38 b39 b40 b41 b42 b43 b44 b45 b46 b47 b48
                )
        )
        (elist '(e1 e2 e3 e4 e5 e6 e7 e8 e9 e10 e11 e12
               e13 e14 e15 e16 e17 e18 e19 e20 e21 e22 e23 e24
               e25 e26 e27 e28 e29 e30 e31 e32 e33 e34 e35 e36
               e37 e38 e39 e40 e41 e42 e43 e44 e45 e46 e47 e48
       ))
        (count 1)
       )
;;;
;;; Check for correct inputs 
;;;
 (cond( ( > (length fid) 48) (format t "File ID too long. (MAX 48 char)~%") )
      ( ( > (length types) 3) (format t "File type too long. (MAX 3 char)~%") )
      ( ( > (length build) 48) (format t "Build too long. (MAX 48 char)~%") )
      ( ( > (length exec) 48) (format t "Exec too long. (MAX 48 char)~%") )
      ( t 
;;;
;;; Set the elements of the C arguments to their default value 
;;; 
   (fill flist 32)
   (fill tlist 32)
   (fill blist 32) 
   (fill elist 32)
;;;
;;; Convert input strings to a list of charcater codes
;;;
   (setq charcodesfid (string-listofint fid))
   (setq charcodestypes (string-listofint types))
   (setq charcodesbuild (string-listofint build))
   (setq charcodesexec (string-listofint exec))
;;;
;;; Set lengths of strings
;;;
   (setq flen (length charcodesfid))
   (setq tlen (length charcodestypes))
   (setq blen (length charcodesbuild))
   (setq elen (length charcodesexec))
;;;
;;; Fill file id of the problem file
;;;
   (setq count 1)
   (loop
    (cond ( (<= count flen) (eval (list 'setq (car flist) (car charcodesfid)))
                            (setq flist (cdr flist))
                            (setq charcodesfid (cdr charcodesfid))
                            (setq count (+ 1 count))
          )
          ( (>= count flen) (return) )
    ) ; End of cond
   ) ; End of loop
;;;
;;; Fill types of input files  
;;;
   (setq count 1)
   (loop
    (cond ( (<= count tlen) (eval (list 'setq (car tlist) (car charcodestypes)))
                            (setq tlist (cdr tlist))
                            (setq charcodestypes (cdr charcodestypes))
                            (setq count (+ 1 count))
          )
          ( (>= count tlen) (return) )
    ) ; End of cond
   ) ; End of loop
;;;
;;; Fill build command
;;;
   (setq count 1)
   (loop
    (cond ( (<= count blen) (eval (list 'setq (car blist) (car charcodesbuild)))
                            (setq blist (cdr blist))
                            (setq charcodesbuild (cdr charcodesbuild))
                            (setq count (+ 1 count))
          )
          ( (>= count blen) (return) )
    ) ; End of cond
   ) ; End of loop
;;;
;;; Fill execute command
;;;
   (setq count 1)
   (loop
    (cond ( (<= count elen) (eval (list 'setq (car elist) (car charcodesexec)))
                            (setq elist (cdr elist))
                            (setq charcodesexec (cdr charcodesexec))
                            (setq count (+ 1 count))
          )
          ( (>= count elen) (return) )
    ) ; End of cond
   ) ; End of loop
;;;
;;; Call C routine
;;;

(dscpr_sub_sub f1  f2  f3  f4  f5  f6  f7  f8  f9  f10 f11 f12
               f13 f14 f15 f16 f17 f18 f19 f20 f21 f22 f23 f24
               f25 f26 f27 f28 f29 f30 f31 f32 f33 f34 f35 f36
               f37 f38 f39 f40 f41 f42 f43 f44 f45 f46 f47 f48
               flen
               t1  t2  t3
               tlen
               b1  b2  b3  b4  b5  b6  b7  b8  b9  b10 b11 b12
               b13 b14 b15 b16 b17 b18 b19 b20 b21 b22 b23 b24
               b25 b26 b27 b28 b29 b30 b31 b32 b33 b34 b35 b36
               b37 b38 b39 b40 b41 b42 b43 b44 b45 b46 b47 b48
               blen
               e1  e2  e3  e4  e5  e6  e7  e8  e9  e10 e11 e12
               e13 e14 e15 e16 e17 e18 e19 e20 e21 e22 e23 e24
               e25 e26 e27 e28 e29 e30 e31 e32 e33 e34 e35 e36
               e37 e38 e39 e40 e41 e42 e43 e44 e45 e46 e47 e48
               elen
               cpu mem
)
) ; End of PE pair
) ; End of cond
 ) ; End of let
) ; End of dscpr_sub


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dscpr_wait                               ;;;
;;;                                                     ;;;
;;;   Input : index of subproblem to wait on            ;;;
;;;           (-1 --> wait on all)                      ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : Status of Command                        ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dscpr_wait (index)
 (dscpr_wait_sub index)
) ; End of dscpr_wait

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dscpr_kill                               ;;;
;;;                                                     ;;;
;;;   Input : index of subproblem to kill               ;;;
;;;           (Max index is 125 * 125)                  ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : Status of Command                        ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dscpr_kill (index)
 (let ( (sign 1) )
  (if (minusp index) (setq sign -1) )
  (setq index (abs index))
  (cond ( (> index (* 125 125)) (format t "Index to dsc_kill is to large.") )
        (t  
           (setq quo (floor (/ index 125)))
           (setq remainder (rem index 125))
           (dscpr_kill_sub (* quo sign) (* remainder sign))
        )
  ) ; End of cond
 ) ; End of let  
) ; End of dscpr_kill

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dscdbg_start                             ;;;
;;;                                                     ;;;
;;;   Input : none                                      ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : Index to new sub problems in DSC server  ;;;
;;;            database.                                ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dscdbg_start ()
 (dscdbg_start_sub 1) 
) ; End of dscdbg_start


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dscpr_next                               ;;;
;;;                                                     ;;;
;;; Input : none                                        ;;;
;;;                                                     ;;;
;;; Output : none                                       ;;;
;;;                                                     ;;;
;;; Return : Index of the next subproblem.              ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dscpr_next ()
 (dscpr_next_sub  1  )
) ; End of dscpr_next

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;;  Redefine the universal-error-handler to exit the   ;;;
;;; LISP environment with an error code when an error   ;;;
;;; occurs.                                             ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(in-package "system")
;(defun universal-error-handler (error-name correctable function-name
                                ;continue-format-string error-format-string args)  
; (format t "~%")
 ;(format t "Error: ") 
 ;(format t error-format-string args)
 ;(format t "~%")
 ;(format t "Error: Signaled by ~S ~%" function-name)
 ;(bye 4)
;)
;(in-package "lisp")

;;;
;;; End of dsc_lisp package
;;;
