;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;; Package : dsc_lisp.library.lsp                                          ;;;
;;;                                                                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;; functions                                                               ;;; 
;;;              dsc_string-listofint                                       ;;;
;;;              dsc_copysub                                                ;;;
;;;              dsc_strcat                                                 ;;;
;;;              dsc_int-str                                                ;;;
;;;              dsc_extractname                                            ;;;
;;;              dsc_bif                                                    ;;;
;;;              empty_bif                                                  ;;;
;;;              add_bif                                                    ;;;
;;;              sub_bif                                                    ;;;
;;;              completed_bif                                              ;;;
;;;              leftover_bif                                               ;;;
;;;              errorstat_bif                                              ;;;
;;;                                                                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                           ;;;
;;; TO COMPILE ...                            ;;;
;;;                                           ;;;
;;;  1)(load "loop.o")                        ;;;
;;;  2)(compile-file "dsc_lisp_library.lsp"   ;;;
;;;                                           ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : dsc_string-listofint                     ;;;
;;;                                                     ;;;
;;;   Input : string                                    ;;;
;;;                                                     ;;;
;;;   Output : none                                     ;;;
;;;                                                     ;;;
;;;   Return : List of character codes                  ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dsc_string-listofint (instring)
 (let ( (len (length instring))
        (count 0)
        (listofint '())
       )
   (loop
    (cond ( (not (= len count))
            (setq listofint (append listofint 
                             (list (char-int (aref instring count)))
                            )
            )
            (setq count (+ 1 count))
          )
          ( (= len count) (return) )
    ) ; End of cond
   ) ; End of loop
 (setq listofint listofint)
 ) ; End of let
) ; End of dsc_string-listofint


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : dsc_copysub < subproblem                 ;;;
;;;                              listofsubproblems >    ;;;
;;;                                                     ;;;
;;; subproblem : "source file name or path of file"     ;;;
;;;                                                     ;;;
;;; listofsubproblems : ("target1" ... "targetn")       ;;;
;;;                                                     ;;;
;;;                                                     ;;;
;;;  Copy the file subproblem to the file names in the  ;;;
;;;  list of subproblems.                               ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

(defun dsc_copysub (subproblem listofsubproblems)
 (copysub_sub subproblem listofsubproblems)
) ; End of dsc_copysub

(defun copysub_sub (subprob listofsubprob)
 (cond ( (null listofsubprob) nil )
       ( t 
           (setq source_file 
                 (open subprob :direction
                               :input)
           )
           (setq outfile (car listofsubprob) )
           (setq outpntr 
                 (open outfile :direction
                               :output
                               :if-exists :overwrite
                               :if-does-not-exist :create)
           )
           (setq in (read source_file nil))
           (loop
            (cond ( (equal in nil) (return) )
                  ( t (print in outpntr)
                      (setq in (read source_file nil))
                  )
            ) ; End cond
           ) ; End of loop
           (close outpntr)
           (close source_file)
           (copysub_sub subprob (cdr listofsubprob)) 
       )
 ) ; End of cond
) ; End of copysub_sub         

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dsc_strcat                               ;;;
;;;                                                     ;;;
;;; Input : < "string1" "string2" >                     ;;;
;;;                                                     ;;;
;;; Output : none                                       ;;;
;;;                                                     ;;;
;;; Return : "string1string2"                           ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dsc_strcat ( strg1 strg2 )
 (let* (                         
        (i 0)
        (l1 (- (length strg1) 1))
        (l2 (- (length strg2) 1))
        (newstrg (make-array (+ l1 l2 2)
                             :element-type
                             '(string-char)
                 )
        )
       )
       (loop for i from 0 to l1 by 1
         do
          (setf (aref newstrg i) (aref strg1 i))
       ) 
       (loop for i from 0 to l2 by 1
         do
          (setf (aref newstrg (+ l1 i 1)) (aref strg2 i))
       ) 
       (setq newstrg newstrg)
 ) ; End let*
) ; End dsc_strcat


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dsc_int-str                              ;;;
;;;                                                     ;;;
;;; Input : < int >                                     ;;;
;;;                                                     ;;;
;;; Output : none                                       ;;;
;;;                                                     ;;;
;;; Return : "int"                                      ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;
;;; Count number of elements in the integer
;;;
(defun numint ( int )
 ( let ( (i 0) )
         (loop until ( = 0 (floor (/ int 10)) )
           do
             (setq i (+ i 1))
           do
             (setq int (floor (/ int 10)) )
         ) ; End loop
 (setq i i)
 ) ; End let
) ; End numint

(defun dsc_int-str ( int )  
 (let (
        (i 1)      ;;; i hold the size of the string
        (l 0)      ;;; l is the for loop counter 
        (il 0)     ;;; il is the for loop counter
        (raise 0 ) ;;; raise is the power 10 is raised to 
        (strg '()) ;;; strg in the string version of the number
        (tmp int)  ;;; temporay integer space
        (tmp2 int) ;;; temporay integer space
        (tmp3 int) ;;; temporay integer space
      )
      ;;; 
      ;;; Find the length of the string
      ;;;
      (setq i (numint int))
      (setq strg (make-array  (+ i 1)
                              :element-type
                              '(string-char)
                  )
      )
      (setq tmp int)
      (loop for l from 0 to (- i 1) by 1
       do
        (progn
        (setq tmp2 tmp)
            (loop until ( =  (floor (/ tmp2 10)) 0 )
                 do
                 (setq tmp2  (floor (/ tmp2 10)) )
            ) ; End loop
        (setf (aref strg l) (int-char (+ tmp2 48) ) )
        ;;;
        ;;; generate new number to look for
        ;;; Logic : i = number of digits
        ;;;         tmp = old number
        ;;;         tmp2 = head of old number
        ;;;         10^i  = place of number
        ;;;         tmp - (tmp2*tmp3) = new number
        ;;;
        (setq tmp3 1)
        (setq raise (numint tmp))
        (loop for il from 1 to raise by 1
          do
          (setq tmp3 (* tmp3 10))
        ) ; End of loop
        (setq tmp ( -  tmp (* tmp2 tmp3)) )
        )
      ) ; End of loop
     (setf (aref strg i) (int-char (+ tmp 48) ) )
     (setq strg strg)
 ) ; End of let
) ;  End of dsc_int-str


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dsc_extractname                          ;;;
;;;                                                     ;;;
;;; Input : < "filename.extension" >                    ;;;
;;;                                                     ;;;
;;; Output : none                                       ;;;
;;;                                                     ;;;
;;; Return : "filename"                                 ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun dsc_extractname ( strg )
 (let (
       (i 0)
       (newstrg "")
      )
  (cond ( (> (length strg) 0)
      (loop until ( = (char-int (aref strg i))  (char-int #\.) )
        do
         (setq i ( + i 1))
      ) ; End loop
      (setq newstrg (make-array  i
                                 :element-type
                                 '(string-char)
                    )
      )
      (setq i 0)
      (loop until ( = (char-int (aref strg i)) (char-int #\.) )
        do
         (setf (aref newstrg i) (aref strg i) )
        do
         (setq i (+ i 1))
      ) ; End loop
      (setq newstrg newstrg)
        )
        ( t nil)
  ) ; End of cond
 ) ; End let
) ; End dsc_extractname

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Data Structure : bif (Bag of Incomplete Futures)    ;;;
;;;                                                     ;;;
;;; Functions ...                                       ;;;
;;;              empty_bif                              ;;;
;;;              add_bif                                ;;;
;;;              sub_bif                                ;;;
;;;              completed_bif                          ;;;
;;;              leftover_bif                           ;;;
;;;              errorstat_bif                          ;;;
;;;              dsc_bif                                ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : Empty_bif < name of data structure >     ;;;
;;;                                                     ;;;
;;;                                                     ;;;
;;; Empty the global variable 'name' to be used as      ;;;
;;; the bif abstract data structure.                    ;;;
;;;                                                     ;;;
;;; Example of use ...                                  ;;;
;;;                                                     ;;;
;;; (empty_bif 'bif)                                    ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun empty_bif (name)
 (eval (list 'setq name '()))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : Add_bif < name of data structure         ;;;
;;;                      filename type buil exec >      ;;;
;;;                                                     ;;;
;;; Add elements to the bag of incomplete futures.      ;;;
;;;                                                     ;;;
;;; Example of use ...                                  ;;;
;;;                                                     ;;;
;;; (add_bif 'bif "filename" "type" "build" "exec")     ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun add_bif (bifname filename type build exec)
 (setq temp (gensym) ) 
 (setf (get temp 'name) filename)
 (setf (get temp 'type) type)
 (setf (get temp 'build) build)  
 (setf (get temp 'exec) exec)
 (setf (get temp 'index) 0)
 (setf (get temp 'done) nil)
 (setf (get temp 'estat) nil)
 (setq tempa (append (eval bifname) (list temp)))
 (eval (list 'setq bifname 'tempa))
) ; End of add_bif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;; Function : Sub_bif < name of data structure >       ;;;
;;;                                                     ;;;
;;; Delete an element from the bag. This is deletion    ;;;
;;; will delete a "random" element.                     ;;;
;;;                                                     ;;;
;;; Example of use ...                                  ;;;
;;;                                                     ;;;
;;;  (subb_bif 'bif)                                    ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   

(defun sub_bif (bifname)
 (setq temps (cdr (eval bifname)))
 (eval (list 'setq bifname 'temps))
) ; End of sub_bif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Function : dsc_bif                                  ;;;
;;;                                                     ;;;
;;; Input : <bif (bag of Incomplete Future)             ;;;
;;;                %completion>                         ;;;      
;;;                                                     ;;;
;;;  The elements of the bif should be a property list. ;;;
;;;  A typical structure is as follows ....             ;;;
;;;                                                     ;;;
;;;   For each subproblem 1 to n do  ...                ;;;
;;;                                                     ;;;
;;; BEGIN                                               ;;;
;;;                                                     ;;;
;;;     (setf (get 'sub# 'name) "filename")             ;;;
;;;     (setf (get 'sub# 'type) "file type")            ;;;
;;;     (setf (get 'sub# 'build) "build command")       ;;;  
;;;     (setf (get 'sub# 'exec) "exec command")         ;;;
;;;     (setf (get 'sub# 'index) 0)                     ;;;
;;;     (setf (get 'sub# 'done) nil)                    ;;;
;;;     (setf (get 'sub# 'etsat) nil)                   ;;;
;;;                                                     ;;;
;;;  END                                                ;;;
;;;                                                     ;;;
;;;  The percent of completions refers to the number of ;;;
;;; subproblems in the bag that must be completed for   ;;;
;;; a succesful future. This routine will wait on this  ;;;
;;; number and restart subproblems if necessary.        ;;;
;;;                                                     ;;;
;;; Where # ranges from 1 to n.                         ;;;
;;;                                                     ;;;
;;; Output : Error and restart messages                 ;;;
;;;                                                     ;;;
;;; Return : nil                                        ;;;
;;;                                                     ;;;
;;; Example of use ...                                  ;;;
;;;                                                     ;;;
;;; (dsc_bif bif 100)                                   ;;;
;;;                                                     ;;;
;;;  Spawn problems from the bag "bif" and wait for     ;;;
;;;  total execution.                                   ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
(defun dsc_bif (lst percent)
 (let* ( (count 1)
         (numofsubprob (length lst))
         (limit (floor (* (/ percent 100) numofsubprob)))
       )
       (setq numberdone 0)
;;;
;;; Spawn first batch of subproblems
;;;
       (spawn lst)
;;;
;;; Wait on subproblems and restart or kill subproblems when needed
;;;
       (modiwait lst limit) 
 ) ; End of let
) ; End of dsc_bif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;;Call dscpr_sub with the elements of listofsubproblems;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun spawn (lst)
 (cond ( (null lst) nil)
       ( (null (get (car lst) 'done) ) 
               (format t "Spawing ~s ~%" (get (car lst) 'name))
         (setf (get (car lst) 'index)
               (dscpr_sub (get (car lst) 'name) (get (car lst) 'type)
                          (get (car lst) 'build) (get (car lst) 'exec)
               )
         )
                (format t "Index is ~s ~%" (get (car lst) 'index) )
         (spawn (cdr lst))
       )
       ( (not (null (get (car lst) 'done)))
         (spawn (cdr lst))
       )
 ) ; End of cond
) ; End of spawn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; 1) Wait on the necessary amout of subproblems for a ;;;
;;;    succesful future.                                ;;;
;;;                                                     ;;; 
;;; 2)Count the number of completed subproblems.        ;;;
;;;                                                     ;;;
;;; 3)If the number of completed subproblems is less    ;;;
;;;   than the limit restart subproblems and goto 1.    ;;;
;;;	                                                ;;;
;;; 4)Once the limit has been reached kill the remaining;;;
;;;   subproblems.                                      ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          
(defun modiwait (lst limit)
  (waitonsub lst limit)
  (setq numberdone (countdone lst) )
 (format t "NUMBERDONE ~s ~%" numberdone)
  (cond 
         (  (< (- (length lst) numberdone) (- limit numberdone))
            (format t
            "Error: NOT ENOUGH CORRECT SUBPROBLEMS TO SUCCESSFULY COMPLETE~%")
            (kill lst)
         )
         (
           ( and (< numberdone limit) (>= (- (length lst) numberdone) limit) )
           (format t "NUMBERDONE ~s LIMIT ~S" numberdone limit)
           (kill lst)
           (dscpr_sleep 50)
           (spawn lst)
           (modiwait lst limit )
        )
        (t
         (kill lst))
  ) ; End of cond
) ; End of modiwait

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Call dscpr_kill with the elements of the list of    ;;;
;;; subproblems that have not yet ended.                ;;;
;;; (The subproblems that have 'done set to nil are the ;;;
;;;  killed ones)                                       ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun kill (lst)
 (cond ( (null lst) nil)
       (  (null (get (car lst) 'done))
          (format t "KILLING ~s ~%" (car lst))
          (dscpr_kill (get (car lst) 'index))
          (kill (cdr lst))
       )
       (t (kill (cdr lst)))
 ) ; End cond 
) ; End kill

  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;;      Count the number of complete subproblems       ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun countdone (lst)
 (setq nd 0)
 (countdone_sub lst)
 (setq nd nd)
)

(defun countdone_sub (lst)
 (cond ( (null lst) nil)
       ( (and (not (null (get (car lst) 'done))) (null (get (car lst) 'estat)) )
         (setq nd (+ 1 nd))
         (countdone_sub (cdr lst))
       )
       ( t (countdone_sub (cdr lst)) )
  ) ; End of cond
) ; End of countdone_sub

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;; Wait on the necessary amout of subproblems for a    ;;;
;;; succesful future.                                   ;;;
;;;                                                     ;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun waitonsub (lst limit)
 (let ( (count 1)
         (next 0)
         (rcode 0)
      )
  (loop
   (cond 
;;;
;;; Wait on next subproblem
;;;
         ( (<= count limit) 
                            (setq next (dscpr_next))
                            (format t "return code from next is ~s ~%" next)
;;;
;;; Check for system limitations 
;;;
                (cond  
                      ( (> next 10001)
                        (format t "~%Error: Index too large!~%")
                        (bye 4)
                      )
                      ( (and (< next -20000) (not (= next -40000)) ) 
                        (format t "~%Error: Index too large!~%")
                        (bye 4)
                      )
;;;
;;; Set the flag(s) in the property list of the ended subproblem
;;;

;;;
;;; A problem ended correctly
;;; (index 1..10000)
;;; Set the done flag to t
;;; The estat flag remain at nil
;;;
                      (
                       (and (<= count limit) (and (<= next 10000) (>= next 0))) 

                       (format t "Problem ended corectly ~s ~%" next)
                       (setdoneflag lst next)
                      ) 
;;;
;;; No more subproblems to account for
;;;
                      (
                       (and (<= count limit) (= next -40000) )
                       (format t "No more subproblems ~%")
                       (setq count limit)
                      )
;;;
;;; An uncorrectable error has occured
;;; Set done flag to t (So that the problem does not get restarted)
;;; set estat to t 
;;;
                      (
                      (and (<= count limit) (and (<= next -1) (>= next -10000)))
                       (format t
                        "Subproblem ~s has a uncorrectable internal error ~%" 
                                 (abs next) )
                       (setdoneflag lst (abs next) )
                       (setestatflag lst (abs next) )
                      )
;;;
;;; An correctable error has occured
;;; Do not set the done flag (So that this problem is restarted)
;;; Do not set the estat flag (The error is correctable)
;;;
                     (
                      (and (<= count limit) (and (<= next -10000) (>= next -20000)))
                       (format t
                        "Subproblem ~s has a correctable internal error~%" 
                                 (abs next) )
                     )
                ) ; End of cond
                (setq count (+ 1 count))
         )
;;;
;;; Check for the incomplete future
;;;
         ( (>= count limit)
           (format t "leaving wait loop limit is ~s ~%" limit)
           (return) )
   ) ; End of cond
  ) ; End of loop
 ) ; End of let
) ; End of waitonsub

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;;  Set the 'done' flag of the completed subproblems   ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun setdoneflag (lst index)
 (cond ( (null lst) nil)
       ( (= (get (car lst) 'index) index)
         (setf (get (car lst) 'done) t)
         (setdoneflag (cdr lst) index)
       )
       ( t (setdoneflag (cdr lst) index) )
 ) ; End cond
) ; End setdoneflag

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;; 
;;;  Set the 'estat' flag of the completed subproblems  ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun setestatflag (lst index)
 (cond ( (null lst) nil)
       ( (= (get (car lst) 'index) index)
         (setf (get (car lst) 'estat) t)
         (setestatflag (cdr lst) index)
       )
       ( t (setestatflag (cdr lst) index) )
 ) ; End cond
) ; End setestatflag

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                     ;;;
;;;  Redefine the universal-error-handler to exit the   ;;;
;;; LISP environment with an error code when an error   ;;;
;;; occurs.                                             ;;;
;;;                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(in-package "system")
;(defun universal-error-handler (error-name correctable function-name
                                ;continue-format-string error-format-string args)  
; (format t "~%")
 ;(format t "Error: ") 
 ;(format t error-format-string args)
 ;(format t "~%")
 ;(format t "Error: Signaled by ~S ~%" function-name)
 ;(bye 4)
;)
;(in-package "lisp")

;;;
;;; End of dsc_lisp package
;;;
