/*****************************************************************************/
/*                                                                           */
/*                                 dsc_defs.h                                */
/*                                                                           */
/* Change History:                                                           */
/* ~~~~~~~~~~~~~~~                                                           */
/* (AAP) - Added "#define DSC_CPU DSC_CPU_UNKNOWN" as a default.             */
/*                                                                           */
/* (AAP) - Added C++ defines and variables:                                  */
/*                                                                           */
/*         #define DSC_DB_CPP "dsc_C.build"                                  */
/*         #define DSC_CPP    'C'                                            */
/*         int Cff             (struct nr)                                   */
/*         unsigned char Ctyp  (stuct dbr)                                   */
/*         unsigned char Ctyp  (struct strt)                                 */
/*                                                                           */
/* (AAP) - Added DSC_DIRZ and DSC_STRINGZ.                                   */
/* (AAP) - Added #defines for dscpr_next() return codes.                     */
/*                                                                           */
/*****************************************************************************/

#include "dsc_debug.h"    /* include debugging hooks for each DSC function   */

/*****************************************************************************/
/*                                                                           */
/*                                   macros                                  */
/*                                                                           */
/*****************************************************************************/

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif
#ifndef MIN
#define MIN(x,y) ((x<y)?x:y)
#endif
#ifndef MAX
#define MAX(x,y) ((x>y)?x:y)
#endif

/*****************************************************************************/
/*                                                                           */
/*                            general status flags                           */
/*                                                                           */
/*****************************************************************************/

#define DSC_SUCCESS 0
#define DSC_FAILURE 1
#define DSC_WORKING 2

/*****************************************************************************/
/*                                                                           */
/*                                copy flags                                 */
/*                                                                           */
/*****************************************************************************/

#define DSC_SEND 0		
#define DSC_RECV 1
#define DSC_EOF -1

/*****************************************************************************/
/*                                                                           */
/*                              string lengths                               */
/*                                                                           */
/*****************************************************************************/

#define DSC_NAMEZ   32                  /* max name length                   */
#define DSC_NODEZ   80                  /* max length of node name           */
#define DSC_FILEZ   80                  /* max file name length              */
#define DSC_DIRZ    100                 /* max length of a directory name    */
#define DSC_STRINGZ 132                 /* max length of generic work string */
#define DSC_CSHZ    256                 /* command for csh                   */
#define DSC_BUFFZ   1024                /* message buffer size               */
#define DSC_SETUPFZ 10000               /* max size of dsc_setup.csh file    */

/*****************************************************************************/
/*                                                                           */
/*                            limiting parameters                            */
/*                                                                           */
/*****************************************************************************/

#define DSC_CPU_LIMIT 5                 /* active job limit                  */
#define DSC_NODEX  100                  /* MAX node for database             */
#define DSC_PROBX  2000                 /* MAX # (sub)problems for this node */
#define MAXMSG     1400                 /* MAX socket message size           */
#define MAX_RETRY  3                    /* max copy message retry            */

/*****************************************************************************/
/*                                                                           */
/*                           string constants                                */
/*                                                                           */
/*****************************************************************************/

#define DSC_ING_NAME     "dscrp"          /* longernet service name for RP   */
#define DSC_SETUP_NAME   "dsc_setup.csh"  /* Dsc setup file name             */
#define DSC_SERVCSH_NAME "dsc_serv.csh"   /* Server start up script          */
#define DSCRP_NETFILE    "./dscrp_net.lis"/* DSCRP network file              */
#define DSCRP_ADDFILE    "./dscrp_net.add"/* DSCRP node address file         */
#define DSCPSPROC        "./dsc_ps"       /* DSC PS deamon program           */
#define DSCPSPASS        "DSCPS"          /* DSC PS filler pasword           */
#define DSCTEXTPASS      "DSC_PASSWORD"   /* DSC password text variable      */
#define DSCTEXTPORT      "DSC_PORT"       /* DSC port text variable          */
#define DSC_DB_C       "dsc_c.build"      /* dummy C build shell             */
#define DSC_DB_CPP     "dsc_C.build"      /* dummy C++ build shell (AAP)     */
#define DSC_DB_L       "dsc_lisp.build"   /* dummy LISP build shell          */
#define DSC_DB_P       "dsc_maple.build"  /* dummy Maple build shell         */
#define DSC_DB_M       "dsc_math.build"   /* dummy Mathematica build shell   */

/*****************************************************************************/
/*                                                                           */
/*                      dynamic scheduling constants                         */
/*                                                                           */
/*****************************************************************************/

#define DSCQUEUED -2                     /* No suitable node found           */
#define SWAPFACTOR -5                    /* Swap Factor                      */
#define MINMEM 5                         /* minimum avaliable memory needed  */
#define LOW_CPU 80                       /* Low CPU usage                    */
#define MED_CPU 160                      /* Medium CPU usage                 */
#define HIGH_CPU 240                     /* High CPU usage                   */

/*****************************************************************************/
/*                                                                           */
/*                             DSC file types                                */
/*                                                                           */
/*****************************************************************************/

#define DSC_C     'c'                    /* C file type                      */
#define DSC_CPP   'C'                    /* C++ file type (AAP)              */
#define DSC_LISP  'l'                    /* lisp file type                   */
#define DSC_P     'p'                    /* Maple file type                  */
#define DSC_M     'm'                    /* Mathematica file type            */
#define DSC_O     'o'                    /* object file type                 */
#define DSC_D     'd'                    /* data/output file type            */
#define DSC_I     'i'                    /* input file type                  */

/*****************************************************************************/
/*                                                                           */
/*                                CPU types                                  */
/*                                                                           */
/*****************************************************************************/

#define DSC_CPU_UNKNOWN  0
#define DSC_CPU_DECMIPS  1
#define DSC_CPU_DECVAX   2
#define DSC_CPU_SUNSPARC 3
#define DSC_CPU_SUNMC    4
/* #define DSC_CPU DSC_CPU_UNKNOWN */
#if sun
#ifdef sparc
#define DSC_CPU DSC_CPU_SUNSPARC
#else
#define DSC_CPU DSC_CPU_SUNMC
#endif
#endif
#ifdef vax
#define DSC_CPU DSC_CPU_DECVAX
#endif
#ifdef mips
#define DSC_CPU DSC_CPU_DECMIPS
#endif

/*****************************************************************************/
/*                                                                           */
/*                               (sub)problem errors                         */
/*                                                                           */
/*****************************************************************************/

#define DSCERR_P_GENERAL 0x01
#define DSCERR_P_NOSOURCE 0x02
#define DSCERR_P_BUILD 0x04
#define DSCERR_P_EXEC 0x08
#define DSCERR_P_DBFULL 0X10
#define DSCERR_P_STATUS 0X20
#define DSCERR_P_RETCODE 0X40
#define DSCERR_P_COREDUMP 0X80
#define DSCERR_P_KILLPAR 0X100
#define DSCERR_P_KILLCTRL 0X200
#define DSCERR_P_NOPAR 0X400
#define DSCERR_P_FORKING 0X800
#define DSCERR_P_DUPNAME 0X1000
#define DSCERR_P_COMM 0X2000
#define DSCERR_P_BADIDX 0X4000
#define DSCERR_P_NODEHALT 0X8000
#define DSCERR_P_ORPHANED 0X10000
#define DSCERR_S_NODEHALT 0X20000000
#define DSCERR_S_NODESTRT 0X40000000
#define DSCERR_S_GENERAL 0X80000000
#define DSCERR_MASK 0XFFFFFFCFF

/*****************************************************************************/
/*                                                                           */
/*                            DSC Return Codes                               */
/*                                                                           */
/*****************************************************************************/
#define ALL      	 	1
#define SUCCESS			1
#define FAILURE			0
#define SUCCESS_LO		1
#define SUCCESS_HI		10000
#define NO_MORE_SUBTASKS   	-40000
#define CORRECTABLE_LO          -20000
#define CORRECTABLE_HI          -10001
#define UNCORRECTABLE_LO        -10000
#define UNCORRECTABLE_HI        -1
#define COROUTINE     		1

/*****************************************************************************/
/*                                                                           */
/*                            DSC Message types                              */
/*                                                                           */
/*****************************************************************************/

#define MSG_UNKNOWN     10       /* unknown message                          */
#define MSG_CHECKIN     11       /* server checkin message                   */
#define MSG_START       12       /* start (sub)problem message               */
#define MSG_SOLVE       13       /* (sub)problem solution message            */
#define MSG_HALT        14       /* halt all problems on local node          */
#define MSG_QSTAT       15       /* request status message                   */
#define MSG_PSTAT       16       /* problem database status                  */
#define MSG_NSTAT       17       /* node database status                     */
#define MSG_SSTAT       18       /* server status                            */
#define MSG_PRBLOCK     19       /* blocking for subproblems to complete     */
#define MSG_COPYCTRL    20       /* copy request/ack message                 */
#define MSG_COPY        21       /* copy request/ack message                 */
#define MSG_BEGIN       22       /* unblock dscpr_sub                        */
#define MSG_KILL        23       /* kill (sub)problem message                */
#define MSG_SLEEPING    24       /* subproblem going to sleep                */
#define MSG_AWAKEN      25       /* (sub)problem requesting an awakening     */
#define MSG_PSSTAT      26       /* Status from local DSC PS deamon          */
#define MSG_PSRSTAT     27       /* Remote node status                       */
#define MSG_MAX         28       /* maximum msg type number                  */

/*****************************************************************************/
/*                                                                           */
/*                               Type Definitions                            */
/*                                                                           */
/*****************************************************************************/

/*                   */
/* node record entry */
/*                   */

typedef struct nr {
   long  index;                  /* longerlock                              */
   unsigned short portg;         /* datagram port                           */
   unsigned long naddr;          /* internet address number                 */
   char nname[DSC_NODEZ];        /* node name                               */
   long  act_prob;               /* count of active problems                */
   long  blk_prob;               /* count of blocked problems               */
   long  err_cnt;                /* error count                             */
   FILE *fp;                     /* file pointer for open file              */
   int cff;                      /* C support field                         */
   int Cff;                      /* C++ support field (AAP)                 */
   int lff;                      /* Lisp support field                      */
   int mff;                      /* Mathematica support field               */
   int pff;                      /* Maple support field                     */
   int cpu;                      /* Most current CPU usage desision value   */ 
   int mem;                      /* Most current MEM usage desision value   */
   int cpurating;                /* Node CPU rating                         */
   int memrating;                /* Node MEM rating                         */
   int numproc;                  /* Number of Processors                    */
   int fsystem;                  /* file system number                      */
   } NODE_REC;

/*                     */
/* node selection list */
/*                     */

typedef struct lst
{
  int   index;                   /* index to node  dbase                    */
  int   m, c;                    /* memory and cpu value                    */
  struct lst    *next;           /* next record                             */
} LST, *LPTR;

/*                     */
/* DSC database record */
/*                     */

typedef struct dbr {
   long  index;                         /* longerlock                       */
   long          depth;                 /* subproblem depth                 */
   char         prob_name[  DSC_NAMEZ ];/* problem name                     */
   long         prob_PID;               /* problem process ID               */
   long         prob_alen;              /* prob unix addr length            */
   long         parent_PID;             /* PID of parent process            */
   long         parent_idx;             /* index of parent in this database */
   unsigned long pnode_addr;            /* 32 bit node address              */
   unsigned long enode_addr;            /* 32 bit node address              */
   unsigned long cpu_type;              /* cpu type                         */
   unsigned char ctyp;                  /* c input                          */
   unsigned char Ctyp;                  /* C++ input (AAP)                  */
   unsigned char ltyp;                  /* lisp input                       */
   unsigned char mtyp;                  /* mathematica input                */
   unsigned char ptyp;                  /* maple input                      */
   unsigned char otyp;                  /* object input                     */
   unsigned char ityp;                  /* input data input                 */
   unsigned char dtyp;                  /* data solution, output            */
   char         src_name[ DSC_FILEZ ];  /* source file name                 */
   char         src_build[ DSC_CSHZ ];  /* command to build source file     */
   char         src_exec[ DSC_CSHZ  ];  /* command to execute image         */
   long         block_sok;              /* socket the problem is blocked on */
   long         block_sub;              /* subproblem blocked on            */
   long         build_flg;              /* build complete flag              */
   long         copy_flg;               /* copy complete flag               */
   long         compl_flg;              /* completion flag                  */
   long         compl_stat;             /* completion status                */
   long         peer_next;              /* peer problem list                */
   long         child_next;             /* subproblem list                  */
   long         num_sub;                /* number of subproblems            */
   long         awk_sub;                /* number of awake subproblems      */
   long         state;                  /* Alert state of the subproblem    */
   int          cpurating;              /* estimated CPU needs              */
   int          memrating;              /* estimated MEM needs              */
} DSCDB, *DSCDB_PTR;

/*                      */
/* node checkin message */
/*                      */

typedef struct ckin {
   long          msg_type;              /* message type id                  */
   long          msg_size;              /* size of message                  */
   long          fnode;                 /* from node                        */
   char          fnname[DSC_NODEZ];     /* from node                        */
   long          act_prob;              /* # active problems                */
   long          blk_prob;              /* # blocked problems               */
   long          flag;                  /* flag                             */
                                        /* 0 = running, 1 = startup         */
                                        /* 2 = halting                      */
   long          reply;                    /* reply needed ?                */
   char          dsc_password[DSC_NAMEZ];  /* message password              */
   long          sanity;                   /* message type id               */
} DSCKIN, *DSCKIN_PTR;

/*                       */
/* DSC PS status message */
/*                       */

typedef struct psstat {
   long          msg_type;                /* message type id                */
   long          msg_size;                /* size of message                */
   long          fnode;                   /* from node                      */
   char          fnname[DSC_NODEZ];       /* from node                      */
   int           cpu;                     /* Statistical CPU use            */
   int           mem;                     /* Statistical MEM use            */
   char          dsc_password[DSC_NAMEZ]; /* message password               */ 
   long          sanity;                  /* message type id                */
} DSCPSSTAT, *DSCPSSTAT_PTR;

/*                     */
/* node status message */
/*                     */

typedef struct psrstat {
   long          msg_type;                /* message type id                */
   long          msg_size;                /* size of message                */
   long          fnode;                   /* from node                      */
   char          fnname[DSC_NODEZ];       /* from node                      */
   int           cpu;                     /* Statistical CPU use            */
   int           mem;                     /* Statistical MEM use            */
   char          dsc_password[DSC_NAMEZ]; /* message password               */
   long          sanity;                  /* message type id                */
} DSCPSRSTAT, *DSCPSRSTAT_PTR;

/*                          */
/* start subproblem message */
/*                          */

typedef struct strt {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   struct sockaddr_in addr;              /* sock addr to send begin msg     */
   long         parent_PID;              /* PID of parent                   */
   long         fnode;                   /* from node                       */
   long         subflg;                  /* 0=problem, 1=subproblem         */
   char         prob_name[ DSC_NAMEZ ];  /* problem name                    */
   unsigned char ctyp;                   /* c input                         */
   unsigned char Ctyp;                   /* C++ input (AAP)                 */
   unsigned char ltyp;                   /* lisp input                      */
   unsigned char mtyp;                   /* mathematica input               */
   unsigned char ptyp;                   /* maple input                     */
   unsigned char otyp;                   /* object input                    */
   unsigned char ityp;                   /* input data input                */
   unsigned char dtyp;                   /* data solution, output           */
   unsigned long cpu_type;               /* cpu type RISC/VAX/SPARC/MC      */
   char         src_name[ DSC_FILEZ ];   /* source file name                */
   char         src_build[ DSC_CSHZ ];   /* command to build src file       */
   char         src_exec[ DSC_CSHZ ];    /* command to execute image        */
   int          cpurating;               /* estimated CPU needs             */
   int          memrating;               /* estimated MEM needs             */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCSTRT, *DSCSTRT_PTR;

/*                                 */
/* put subproblem to sleep message */
/*                                 */

typedef struct slp {
   long         msg_type;                 /* message type id                */
   long         msg_size;                 /* size of message                */
   long         PID;                      /* subproblem PID                 */
   long         fnode;                    /* from node                      */
   long         copyflag;                 /* copy datafile flag             */
                                          /* 0 = do not copy data file      */
                                          /* 1 = copy data file             */
   char         fnname[DSC_NAMEZ];        /* from node name                 */
   char         prob_name[DSC_NAMEZ];     /* problem name                   */
   char         dsc_password[DSC_NAMEZ];  /* message password               */
   long         sanity;                   /* message type id                */
} DSCSLP, *DSCSLP_PTR;

/*                           */
/* subproblem awaken message */
/*                           */

typedef struct awk {
   long         msg_type;                 /* message type id                */
   long         msg_size;                 /* size of message                */
   struct sockaddr_in addr;               /* sock addr to recieve status    */
   long         PID;                      /* (sub)problem PID               */
   long         fnode;                    /* from node                      */
   long         index;                    /* index of problem to awaken     */
   char         filename[DSC_NAMEZ];      /* filename flag                  */
   char         fnname[DSC_NAMEZ];        /* from node name                 */
   char         prob_name[DSC_NAMEZ];     /* problem name                   */
   char         dsc_password[DSC_NAMEZ];  /* message password               */
   long         sanity;                   /* message type id                */
} DSCAWK, *DSCAWK_PTR;

/*                               */
/* (sub)problem solution message */
/*                               */

typedef struct solv {
   long          msg_type;                 /* message type id               */
   long          msg_size;                 /* size of message               */
   char          prob_name[ DSC_NAMEZ ];   /* name of problem               */
   char          src_name[ DSC_FILEZ ];    /* name of source file           */
   long          index;                    /* index complete                */
   long          status;                   /* DSC_SUCCESS/DSC_FAILURE       */
   long          act_prob;                 /* active problems on node       */
   long          blk_prob;                 /* blocked problems on node      */
   char          dsc_password[DSC_NAMEZ];  /* message password              */
   long          sanity;                   /* message type id               */
} DSCSOLV, *DSCSOLV_PTR;

/*                             */
/* halt a (sub)problem message */
/*                             */

typedef struct halt {
   long          msg_type;                /* message type id                */
   long          msg_size;                /* size of message                */
   long          from_PID;                /* PID of sender                  */
   char          from_name[ DSC_NAMEZ ];  /* name of msg sender             */ 
   long          prob_PID;                /* problem PID                    */
   long          mom_PID;                 /* parent PID                     */
   char          dsc_password[DSC_NAMEZ]; /* message password               */
   long          sanity;                  /* message type id                */
} DSCHALT, *DSCHALT_PTR;

/*                        */
/* request status message */
/*                        */

typedef struct qsta {
   long          msg_type;                 /* message type id               */
   long          msg_size;                 /* size of message               */
   struct sockaddr_in addr;                /* special inet socket port      */
   long          flag;                     /* type of status                */
                                           /* flag =  MSG_PSTAT | MSG_NSTAT */
                                           /*         | MSG_SSTAT           */
   char          dsc_password[DSC_NAMEZ];  /* message password              */
   long          sanity;                   /* message type id               */
} DSCQSTA, *DSCQSTA_PTR;

/*                        */
/* problem status message */
/*                        */

typedef struct psta {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   char         more;                    /* more flag                       */
   DSCDB        prob;                    /* problem db record               */
   char         enname[DSC_NODEZ];       /* executing node name             */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCPSTA, *DSCPSTA_PTR;

/*                     */
/* NODE status message */
/*                     */

typedef struct nsta {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   char         more;                    /* more flag                       */
   NODE_REC     node;                    /* node db record                  */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCNSTA, *DSCNSTA_PTR;

/*                       */
/* server status message */
/*                       */

typedef struct ssta {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   char         nname[DSC_NODEZ];        /* node name                       */
   long         naddr;                   /* node address                    */
   long         act_prob;                /* currently active problems       */
   long         blk_prob;                /* blocked problems                */
   long         waiting;                 /* queued/waiting problems         */
   int          cpu;                     /* Current CPU value               */
   int          mem;                     /* Current MEM value               */
   int          cpurating;               /* cpu MIPS rating                 */
   int          memrating;               /* mem MIPS rating                 */
   int          numproc;                 /* Number of Processors            */
   int          fsystem;                 /* file system number              */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCSSTA, *DSCSSTA_PTR;

/*                          */
/* problem blocking message */
/*                          */

typedef struct prblk {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   struct sockaddr_in addr;              /* special inet socket port        */
   long         from_PID;                /* PID of sender                   */
   long         index;                   /* index to wait on                */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCPRBLK, *DSCPRBLK_PTR;

/*                          */
/* copy request/ack message */
/*                          */

typedef struct cpyctrl {
   long          msg_type;
   long          msg_size;
   struct sockaddr_in addr;               /* special inet socket port       */
   long          flag;                    /* request to send or rcv         */
                                          /* 0 = req send, 1 = req recv,    */
                                          /* -1 = EOF                       */
   char          fname[DSC_FILEZ];        /* file name                      */
   long          totz;                    /* only used if EOF               */
   char          dsc_password[DSC_NAMEZ]; /* message password               */
   long          sanity;                  /* message type id                */
 } DSCCPYCTRL, *DSCCPYCTRL_PTR;

typedef struct cpy {
   long          msg_type;
   long          msg_size;
   long          cnt;                     /* # bytes in this msg            */
   char          buffer[DSC_BUFFZ];       /* buffer of file content         */
   char         dsc_password[DSC_NAMEZ];  /* message password               */
   long          sanity;                  /* message type id                */
 } DSCCPY, *DSCCPY_PTR;

typedef struct beg {
   long          msg_type;
   long          msg_size;
   long          index;                   /* started subproblem index       */
   char         dsc_password[DSC_NAMEZ];  /* message password               */
   long          sanity;                  /* message type id                */
 } DSCBEG, *DSCBEG_PTR;

/*                          */
/* problem blocking message */
/*                          */

typedef struct prkil {
   long         msg_type;                /* message type id                 */
   long         msg_size;                /* size of message                 */
   long         from_PID;                /* PID of sender                   */
   long         index;                   /* index to kill                   */
   char         prob_name[DSC_NAMEZ];    /* problem name to kill            */
   char         dsc_password[DSC_NAMEZ]; /* message password                */
   long         sanity;                  /* message type id                 */
} DSCKILL, *DSCKILL_PTR;
