##############################################################################
#                                                                            #
#                              dsc_maple.mpl                                 #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
# functions:                                                                 #
#      dscpr_pwd        - get current working directory                      #
#      dscpr_sub        - for problems to start subproblems                  #
#      dscpr_wait       - wait for one or more subproblems to complete       #
#      dscpr_next       - wait for next subproblem to complete               #
#      dscpr_kill       - kill specified subproblem                          #
#      dscpr_cosetup    - setup when using coroutines                        #
#      dscpr_coresume   - awaken a sleeping subproblem                       #
#      dscpr_cowait     - put a subproblem to sleep                          #
#      dscpr_maple_kill - shut down dsc_maple deamon                         #
#      dscdbg_start     - (sub)problem debugging function                    #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
#                               debugging hooks                              #
#                                                                            #
##############################################################################

DSC_M_PWD_DEBUG      := 'DSC_M_PWD_DEBUG';
DSC_M_SUB_DEBUG      := 'DSC_M_SUB_DEBUG';
DSC_M_WAIT_DEBUG     := 'DSC_M_WAIT_DEBUG';
DSC_M_NEXT_DEBUG     := 'DSC_M_NEXT_DEBUG';
DSC_M_KILL_DEBUG     := 'DSC_M_KILL_DEBUG';
DSC_M_COSETUP_DEBUG  := 'DSC_M_COSETUP_DEBUG';
DSC_M_CORESUME_DEBUG := 'DSC_M_CORESUME_DEBUG';
DSC_M_COWAIT_DEBUG   := 'DSC_M_COWAIT_DEBUG';
DSC_M_MAPLEK_DEBUG   := 'DSC_M_MAPLEK_DEBUG';
DSC_M_DBG_DEBUG      := 'DSC_M_DBG_DEBUG';   

DSC_M_PWD_DEBUG      := 1;
DSC_M_SUB_DEBUG      := 1;
DSC_M_WAIT_DEBUG     := 1;
DSC_M_NEXT_DEBUG     := 1;
DSC_M_KILL_DEBUG     := 1;
DSC_M_COSETUP_DEBUG  := 1;
DSC_M_CORESUME_DEBUG := 1;
DSC_M_COWAIT_DEBUG   := 1;
DSC_M_MAPLEK_DEBUG   := 1;
DSC_M_DBG_DEBUG      := 1;

##############################################################################
#                                                                            #
#                                   globals                                  #
#                                                                            #
##############################################################################

# DSC_M_PWD      - global variable to store full path name
# DSC_M_INDEX    - global variable to store the subproblem index number
# DSC_M_STATUS   - global variable to store the return status
# DSC_M_ANSWER   - global name of next completed subproblem
# DSC_M_ERROR    - global variable to store the error value

##############################################################################
#                                                                            #
#                              load  help text                               #
#                                                                            #
##############################################################################

read `dsc_maple.hlp`;

##############################################################################
#                                                                            #
# subroutine: dscpr_pwd                                                      #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_PWD   - global variable to store full path name             # 
#                                                                            #
# outputs: none                                                              #
#                                                                            # 
# return:  current workind directory                                         #
#                                                                            #
# purpose: maple procedure to get the full path of the maple shell           #
#                                                                            #
##############################################################################

dscpr_pwd := proc(commandfile,messagepid,dscmaplepid,answerfile)
  local callC,           # shell command string
        filestr,         # file name string
        DSC_M_PWD_LOCAL; # local copy of the current working directory

  if DSC_M_PWD_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_pwd        : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_pwd        : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_pwd        : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_pwd        : answer file %s\n`,answerfile);
  fi;
  
  #                                                                   #
  # set up shell dsc_maple_message execution call                     #
  # dsc_maple_message commandfile messagepid dscmaplepid 0 answerfile #
  #                                                                   #
  callC:= `dsc_maple_message `;      
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 0 `);
  callC:= cat(callC,answerfile);

  if DSC_M_PWD_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_pwd        : system call %s\n`,callC);
  fi; 

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);		

  filestr:= answerfile;         
  DSC_M_PWD:=` `;		
  
  #                    #
  # read response file #
  #                    #
  read(filestr);		

  if DSC_M_PWD = ` ` then
    #                                 #
    # log error is dsc_maple.log file #
    #                                 #
    system(`echo "dscm.mpl : dscpr_pwd        : can not read answerfile" >> dsc_maple.log`);
    ERROR(`dscm.mpl : dscpr_pwd        : can not read file`,filestr);
  fi;

  if DSC_M_PWD_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_pwd        : DSC_M_PWD %s\n`,DSC_M_PWD);
  fi;

  #                         #
  # remove unecessary files #
  #                         #
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,filestr);
  system(filestr);		       # remove response file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile); 
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);  
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application #

  DSC_M_PWD_LOCAL:= DSC_M_PWD;     
  DSC_M_PWD:= 'DSC_M_PWD';	  

  if DSC_M_PWD_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_pwd        : returning %s\n`,DSC_M_PWD_LOCAL);
  fi;

  RETURN(DSC_M_PWD_LOCAL);	    # return DSC_M_PWD_LOCAL to user
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_sub                                                      #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          file        - file id of problem file                             #
#          types       - types of input files (c,l,o,i,p)                    #
#          build       - build command                                       #
#          exec        - execute command                                     #
#          cpu         - estimated cpu usage (LOW_CPU,MED_CPU,HIGH_CPU)      #
#          mem         - estimated mem usage in megs                         #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_INDEX - global variable to store the subproblem index number#
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return: index to new subproblem in DSC Server database                     #
#                                                                            #
# purpose: maple procedure to spawn subproblems                              #
#                                                                            #
##############################################################################

dscpr_sub := proc(commandfile,messagepid,dscmaplepid,
                  DSC_M_PWD,file,types,build,exec,cpu,mem,
                  answerfile)
  local callC,             # shell command string
        filestr,           # file name string
        msgstr,            # log file message string
        DSC_M_INDEX_LOCAL; # local copy of the subproblem index

  if DSC_M_SUB_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_sub        : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_sub        : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_sub        : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_sub        : DSC_M_PWD %s\n`,DSC_M_PWD);
  printf(`dscm.mpl : dscpr_sub        : file %s\n`,file);
  printf(`dscm.mpl : dscpr_sub        : types %s\n`,types);
  printf(`dscm.mpl : dscpr_sub        : build %s\n`,build);
  printf(`dscm.mpl : dscpr_sub        : exec %s\n`,exec);
  printf(`dscm.mpl : dscpr_sub        : cpu %d\n`,cpu);
  printf(`dscm.mpl : dscpr_sub        : mem %d\n`,mem);
  printf(`dscm.mpl : dscpr_sub        : answer file %s\n`,answerfile);
  fi;
 
  #                     #
  # default error value #
  #                     #
  DSC_M_INDEX:= -99;		

  #                                                                   #
  # set up shell dsc_maple_message execution call                     #
  # dsc_message commandfile messagepid dscmaplepid 1 file types build #
  #             exec cpu mem answerfile                               #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 1 `);
  callC:= cat(callC,` "`,file,`" `);
  callC:= cat(callC,` "`,types,`" `);
  callC:= cat(callC,` "`,build,`" `);
  callC:= cat(callC,` "`,exec,`" `);
  callC:= cat(callC,` `,cpu,` `,mem,` `);
  callC:= cat(callC,answerfile);

  if DSC_M_SUB_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_sub        : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);		

  if DSC_M_INDEX = -99 then
        msgstr := cat(`echo "dscm.mpl : dscpr_sub        :cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
	system(msgstr);

   if DSC_M_SUB_DEBUG = 1 then
        msgstr := cat(`dscm.mpl : dscpr_sub        : `,filestr);
        msgstr := cat(msgstr,` for reading`);
	ERROR(msgstr);
   fi; 	
  fi;

   if DSC_M_SUB_DEBUG = 1 then
    printf(`dscm.mpl : dscpr_sub        : DSC_M_INDEX = %d\n`,DSC_M_INDEX);
   fi;

  #                         #
  # remove unecessary files #
  #                         #

  filestr:= cat(`rm -f `,filestr);  # remove response file
  system(filestr);
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_INDEX_LOCAL := DSC_M_INDEX; 
  DSC_M_INDEX:= 'DSC_M_INDEX';	    

   if DSC_M_SUB_DEBUG = 1 then
    printf(`dscm.mpl : dscpr_sub        : returning %d\n`,DSC_M_INDEX_LOCAL);
   fi;

  RETURN(DSC_M_INDEX_LOCAL);	    # return DSC_M_INDEX_LOCAL to user
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_wait                                                     #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          index       - index of subproblem to wait on (-1 == wait on all)  #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_STATUS - global variable to store the return status         #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  wait status                                                       # 
#                                                                            #
# purpose: maple procedure to block on subproblems                           #
#                                                                            #
##############################################################################

dscpr_wait := proc(commandfile,messagepid,dscmaplepid,
                   DSC_M_PWD,index,
                   answerfile)
  local callC,               # shell command string
        filestr,             # file name string
        msgstr,              # log file message string
        DSC_M_STATUS_LOCAL;  # local copy of the dscpr_wait termination status

  if DSC_M_WAIT_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_wait       : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_wait       : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_wait       : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_wait       : DSC_M_PWD %s\n`,DSC_M_PWD);
  printf(`dscm.mpl : dscpr_wait       : index %d\n`,index);
  printf(`dscm.mpl : dscpr_wait       : answer file %s\n`,answerfile);
  fi;

  #                     #
  # default error value #
  #                     #
  DSC_M_STATUS:= -99;		

  #                                                                   #
  # set up shell dsc_maple_message execution call                     #
  # dsc_message commandfile messagepid dscmaplepid 2 index answerfile #
  #                                                                   #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 2 `);
  callC:= cat(callC,index);
  callC:= cat(callC,` `,answerfile);

  if DSC_M_WAIT_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_wait       : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);		

  if DSC_M_STATUS = -99 then
        msgstr := cat(`echo "dscm.mpl : dscpr_wait       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
        system(msgstr);

      if DSC_M_WAIT_DEBUG = 1 then
        msgstr := cat(`dscm.mpl : dscpr_wait       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading`);
        ERROR(msgstr);
      fi;
  fi;

  #                         #
  # remove unecessary files #
  #                         #

  filestr:= cat(`rm -f `,filestr);
  system(filestr);		       # remove response file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_STATUS_LOCAL:= DSC_M_STATUS; 
  DSC_M_STATUS:= 'DSC_M_STATUS';     

  if DSC_M_WAIT_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_wait       : returning %d\n`,DSC_M_STATUS_LOCAL);
  fi;

  RETURN(DSC_M_STATUS_LOCAL);	     # return DSC_M_STATUS_LOCAL to user
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_next                                                     #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          answer      - name of solution datafile                           #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_STATUS - global variable to store the return status         #
#          DSC_M_ANSWER - global name of next completed subproblem           #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  wait status                                                       #
#                                                                            #
# purpose: maple procedure to block on next subproblem                       #
#                                                                            #
##############################################################################

dscpr_next := proc(commandfile,messagepid,dscmaplepid,
                   DSC_M_PWD,answer,
                   answerfile)
 local callC,              # shell command string
       filestr,            # file name string
       msgstr,             # log file message string
       DSC_M_STATUS_LOCAL; # local copy of the dscpr_next termination status

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_next       : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_next       : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_next       : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_next       : DSC_M_PWD %s\n`,DSC_M_PWD);
  printf(`dscm.mpl : dscpr_next       : answer %s\n`,answer);
  printf(`dscm.mpl : dscpr_next       : answer file %s\n`,answerfile);
  fi;

  #                     #
  # default error value #
  #                     #
  DSC_M_STATUS:= -99;

  #                                                                   #
  # set up shell dsc_maple_message execution call                     #
  # dsc_message commandfile messagepid dscmaplepid 4 afile answerfile #
  #                                                                   #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 4 `);
  callC:= cat(callC,answer,` `);
  callC:= cat(callC,answerfile);

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_next       : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);

  if DSC_M_STATUS = -99 then
        msgstr := cat(`echo "dscm.mpl : dscpr_next       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
        system(msgstr);

      if DSC_M_NEXT__DEBUG = 1 then
        msgstr := cat(`dscm.mpl : dscpr_next       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading`);
        ERROR(msgstr);
      fi;
  fi;

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_next       : DSC_M_STATUS = %d\n`,DSC_M_STATUS); 
  fi;

  #                         #
  # remove unecessary files #
  #                         #

  filestr:= cat(`rm -f `,filestr);     # remove response file
  system(filestr);
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_STATUS_LOCAL := DSC_M_STATUS;
  DSC_M_STATUS := 'DSC_M_STATUS';
  answer := DSC_M_ANSWER;
  DSC_M_ANSWER := `DSC_M_ANSWER`;

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_next       : returning %d\n`,DSC_M_STATUS_LOCAL);
  fi;

  RETURN(DSC_M_STATUS_LOCAL);
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_kill                                                     #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          index       - index of subproblem to kill                         #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_ERROR - global variable to store the error value            #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  function status                                                   #
#                                                                            #
# purpose: maple procedure to kill a specific subproblem                     #
#                                                                            #
##############################################################################

dscpr_kill := proc(commandfile,messagepid,dscmaplepid,
                  DSC_M_PWD,index,
                  answerfile)
 local callC,              # shell command string
       filestr,            # file name string
       msgstr,             # log file message string
       DSC_M_ERROR_LOCAL;  # local copy of the dscpr_kill termination status

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_kill       : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_kill       : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_kill       : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_kill       : DSC_M_PWD %s\n`,DSC_M_PWD);
  printf(`dscm.mpl : dscpr_kill       : index %d\n`,index);
  printf(`dscm.mpl : dscpr_kill       : answer file %s\n`,answerfile);
  fi;

  #                     #
  # default error value #
  #                     #
  DSC_M_ERROR := -99;

  #                                                                   #
  # set up shell dsc_maple_message execution call                     #
  # dsc_message commandfile messagepid dscmaplepid 3 index answerfile #
  #                                                                   #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 3 `);
  callC:= cat(callC,index,` `);
  callC:= cat(callC,answerfile);

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_kill       : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);

  if DSC_M_ERROR = -99 then
        msgstr := cat(`echo "dscm.mpl : dscpr_kill       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
        system(msgstr);

      if DSC_M_NEXT__DEBUG = 1 then
        msgstr := cat(`dscm.mpl : dscpr_kill       : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading`);
        ERROR(msgstr);
      fi;
  fi;

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_kill       : DSC_M_ERROR = %d\n`,DSC_M_ERROR);
  fi;

  filestr:= cat(`rm -f `,filestr);     # remove response file
  system(filestr);
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_ERROR_LOCAL := DSC_M_ERROR;
  DSC_M_ERROR := 'DSC_M_ERROR';

  if DSC_M_NEXT__DEBUG = 1 then
  printf(`dscm.mpl : dscpr_kill       : returning %d\n`,DSC_M_ERROR_LOCAL);
  fi;

  RETURN(DSC_M_ERROR_LOCAL);  
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_cosetup                                                  #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  none                                                              #
#                                                                            #
# purpose: setup when using coroutines                                       #
#                                                                            #
##############################################################################

dscpr_cosetup := proc(commandfile,messagepid,dscmaplepid,DSC_M_PWD)
 local callC,   # shell command string
       filestr; # file name strin

  if DSC_M_COSETUP_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_cosetup    : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_cosetup    : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_cosetup    : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_cosetup    : DSC_M_PWD %s\n`,DSC_M_PWD);
  fi;

  #                                                  #
  # set up shell dsc_maple_message execution call    # 
  # dsc_message commandfile messagepid dscmaplepid 5 #
  #                                                  #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 5 `);

  if DSC_M_COSETUP_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_cosetup    : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  if DSC_M_COSETUP_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_cosetup    : returning void\n`);
  fi;

end;

##############################################################################
#                                                                            #
# subroutine: dscpr_coresume                                                 #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          index       - index of problem to awaken                          #
#          com         - copy input file flag                                #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_INDEX - global variable to store the return index or error  #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  subproblem index on success, -1 on failure                        #
#                                                                            #
# purpose: awaken a sleeping subproblem                                      #
#                                                                            #
##############################################################################

dscpr_coresume := proc(commandfile,messagepid,dscmaplepid,DSC_M_PWD,
                       index,command,
                       answerfile)
 local callC,             # shell command string
       filestr,           # file name string
       msgstr,            # log file message string
       DSC_M_INDEX_LOCAL; # local copy of the subproblem index                      
 if DSC_M_CORESUME_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_coresume   : command file %s\n`,commandfile);
  printf(`dscm.mpl : dscpr_coresume   : message pid file %s\n`,messagepid);
  printf(`dscm.mpl : dscpr_coresume   : dsc_maple pid file %s\n`,dscmaplepid);
  printf(`dscm.mpl : dscpr_coresume   : DSC_M_PWD %s\n`,DSC_M_PWD);
  printf(`dscm.mpl : dscpr_coresume   : index %d\n`,index);
  printf(`dscm.mpl : dscpr_coresume   : command %s\n`,command);
  printf(`dscm.mpl : dscpr_coresume   : answer file %s\n`,answerfile);
 fi;

  #                     #
  # default error value #
  #                     #
  DSC_M_INDEX:= -99;

  #                                                                          #
  # set up shell dsc_maple_message execution call                            #
  # dsc_message commandfile messagepid dscmaplepid 7 index command answefile #
  #                                                                          #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 7 `);
  callC:= cat(callC,index,` `);
  callC:= cat(callC,command,` `);
  callC:= cat(callC,answerfile);

 if DSC_M_CORESUME_DEBUG = 1 then
  printf(`dscm.mpl : dscpr_coresume   : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);

  if DSC_M_INDEX = -99 then
       msgstr := cat(`echo "dscm.mpl : dscpr_coresume   : cannot open file `
                     ,filestr);
       msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
       system(msgstr);

     if DSC_M_CORESUME_DEBUG = 1 then
       msgstr := cat(`dscm.mpl : dscpr_coresume   : cannot open file `
                     ,filestr);
       msgstr := cat(msgstr,` for reading`);
       ERROR(msgstr);
     fi;
  fi;

  if DSC_M_CORESUME_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_coresume   : DSC_M_INDEX = %d\n`,DSC_M_INDEX);
  fi;

  #                         #
  # remove unecessary files #
  #                         #

  filestr:= cat(`rm -f `,filestr);     # remove response file
  system(filestr);
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_INDEX_LOCAL := DSC_M_INDEX;
  DSC_M_INDEX := 'DSC_M_INDEX';

  if DSC_M_CORESUME_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_coresume   : returning %d\n`,DSC_M_INDEX_LOCAL);
  fi;

  RETURN(DSC_M_INDEX_LOCAL);
end;

##############################################################################
#                                                                            #
# subroutine: dscpr_cowait                                                   #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          flag        - copy data file flag ( 1 = copy, 0 = do not copy)    #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  none                                                              #
#                                                                            #
# purpose: enter a sleep state                                               #
#                                                                            #
##############################################################################

dscpr_cowait := proc(commandfile,messagepid,dscmaplepid,DSC_M_PWD,
                     flag)
  local callC,   # shell command string
        filestr; # file name string

  if DSC_M_COWAIT_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_cowait     : command file %s\n`,commandfile);
   printf(`dscm.mpl : dscpr_cowait     : message pid file %s\n`,messagepid);
   printf(`dscm.mpl : dscpr_cowait     : dsc_maple pid file %s\n`,dscmaplepid);
   printf(`dscm.mpl : dscpr_cowait     : DSC_M_PWD %s\n`,DSC_M_PWD);
   printf(`dscm.mpl : dscpr_cowait     : flag %d\n`,flag);
  fi;
  
  #                                                       #
  # dsc_message commandfile messagepid dscmaplepid 6 flag #
  #                                                       #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 6 `);
  callC:= cat(callC,flag);

  if DSC_M_COWAIT_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_cowait     : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  if DSC_M_COWAIT_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_cowait     : returning void\n`);
  fi;

end;

##############################################################################
#                                                                            #
# subroutine: dscpr_maple_kill                                               #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  none                                                              #
#                                                                            #
# purpose: kill dsc_maple deamon                                             #
#                                                                            #
##############################################################################

dscpr_maple_kill := proc(commandfile,messagepid,dscmaplepid,DSC_M_PWD)
  local callC,   # shell command string
        filestr; # file name string

  if DSC_M_MAPLEK_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_maple_kill : command file %s\n`,commandfile);
   printf(`dscm.mpl : dscpr_maple_kill : message pid file %s\n`,messagepid);
   printf(`dscm.mpl : dscpr_maple_kill : dsc_maple pid file %s\n`,dscmaplepid);
   printf(`dscm.mpl : dscpr_maple_kill : DSC_M_PWD %s\n`,DSC_M_PWD);
  fi;

  #                                                   #
  # dsc_message commandfile messagepid dscmaplepid -1 #
  #                                                   #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` -1 `);

  if DSC_M_MAPLEK_DEBUG = 1 then
   printf(`dscm.mpl : dscpr_maple_kill : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  #                    #
  # wait for shut down #
  #                    #
  system(`sleep 10`);
end;

##############################################################################
#                                                                            #
# subroutine: dscdbg_start                                                   #
#                                                                            #
# inputs:  commandfile - dsc_maple command request file                      #
#          messagepid  - file to contain pid of dsc_maple_message deamon     #
#          dscmaplepid - file containing pid of dsc_maple deamon             #
#          DSC_M_PWD   - current working directory                           #
#          answerfile  - file containing result of dsc_maple request         #
#                                                                            #
# globals: DSC_M_INDEX - global variable to store the debugging entry index  #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  debugging entry index                                             #
#                                                                            #
# purpose: (sub)problem debugging function                                   #
#                                                                            #
##############################################################################

dscdbg_start := proc(commandfile,messagepid,dscmaplepid,DSC_M_PWD,answerfile)
  local callC,             # shell command string
        filestr,           # file name string
        msgstr,            # log file message string
        DSC_M_INDEX_LOCAL; # local copy of the subproblem index 

  if DSC_M_DBG_DEBUG = 1 then
    printf(`dscm.mpl : dscdbg_start     : command file %s\n`,commandfile);
    printf(`dscm.mpl : dscdbg_start     : message pid file %s\n`,messagepid);
    printf(`dscm.mpl : dscdbg_start     : dsc_maple pid file %s\n`,dscmaplepid);
    printf(`dscm.mpl : dscdbg_start     : DSC_M_PWD %s\n`,DSC_M_PWD);
    printf(`dscm.mpl : dscdbg_start     : answer file %s\n`,answerfile);
  fi;
 
  #                     #
  # default error value #
  #                     #
  DSC_M_INDEX:= -99;

  #                                                             #
  # set up shell dsc_maple_message execution call               #
  # dsc_message commandfile messagepid dscmaplepid 8 answerfile #
  #                                                             #
  callC:= cat(DSC_M_PWD,`/dsc_maple_message `);
  callC:= cat(callC,commandfile,` `);
  callC:= cat(callC,messagepid,` `);
  callC:= cat(callC,dscmaplepid,` 8 `);
  callC:= cat(callC,answerfile);

  if DSC_M_DBG_DEBUG = 1 then
    printf(`dscm.mpl : dscdbg_start     : system call %s\n`,callC);
  fi;

  #                                      #
  # execute the dsc_maple_message deamon #
  #                                      #
  system(callC);

  filestr:= cat(DSC_M_PWD,`/`,answerfile);

  #                    #
  # read response file #
  #                    #
  read(filestr);

  if DSC_M_INDEX = -99 then
        msgstr := cat(`echo "dscm.mpl : dscdbg_start     : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading" >> dsc_maple.log`);
        system(msgstr);

      if DSC_M_DBG_DEBUG = 1 then
        msgstr := cat(`dscm.mpl : dscdbg_start     : cannot open file `
                      ,filestr);
        msgstr := cat(msgstr,` for reading`);
        ERROR(msgstr);
      fi;
  fi;

  if DSC_M_DBG_DEBUG = 1 then
    printf(`dscm.mpl : dscdbg_start     : DSC_M_INDEX = %d\n`,DSC_M_INDEX);
   fi;

  #                         #
  # remove unecessary files #
  #                         #

  filestr:= cat(`rm -f `,filestr);
  system(filestr);		       # remove response file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,commandfile);
  system(filestr);                     # remove command file
  filestr:= cat(`rm -f `,DSC_M_PWD,`/`,messagepid);
  system(filestr);                     # remove message pid file
  # do not remove maple pid file , this is constant throughout application

  DSC_M_INDEX_LOCAL := DSC_M_INDEX; 
  DSC_M_INDEX:= 'DSC_M_INDEX';  

  if DSC_M_DBG_DEBUG = 1 then
    printf(`dscm.mpl : dscdbg_start     : returning %d\n`,DSC_M_INDEX_LOCAL);
   fi;

  RETURN(DSC_M_INDEX_LOCAL);	# return DSC_M_INDEX_LOCAL to user
end;
