/*****************************************************************************/
/*                                                                           */
/*                              dsc_maple_message.c                          */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*      main                   - handle DSC server requests                  */
/*      maple_getpid()         - get and save local pid                      */
/*      dsc_maple_setup()      - setup coroutine like behavior for deamon    */
/*      dsc_maple_handle()     - handle SIGUSER2                             */
/*                                                                           */
/*****************************************************************************/

#include "dscpr.c"        /* include DSC C user interface                    */

/*****************************************************************************/
/*                                                                           */
/*                                 definitions                               */
/*                                                                           */
/*****************************************************************************/

#define MSG_MPLKILL      -1 /* Exit dsc_maple deamon                         */
#define MSG_DSCPR_PWD     0 /* Get PWD                                       */
#define MSG_DSCPRSTART    1 /* For problems to start subproblems             */
#define MSG_DSCPRWAIT     2 /* wait for one or more subproblems to complete  */
#define MSG_DSCPRKILL     3 /* kill specified subproblem                     */
#define MSG_DSCPRNEXT     4 /* Wait for next subproblem to complete          */
#define MSG_DSCPRCOSETUP  5 /* Setup when using coroutines                   */
#define MSG_DSCPRCOWAIT   6 /* Put a subproblem to sleep                     */
#define MSG_DSCPRCORESUME 7 /* Awaken a sleeping subproblem                  */
#define MSG_DSCDBGSTART   8 /* DEBUGGING FUNCTION                            */
#define DEBUG 1             /* For debugging purposes                        */

/*****************************************************************************/
/*                                                                           */
/*                           module level variables                          */
/*                                                                           */
/*****************************************************************************/

int dsc_maple_flag = FALSE;
int mypid;

/*****************************************************************************/
/*                                                                           */
/* subroutine: main (argc, argv)                                             */
/*                                                                           */
/* inputs:    argc    - number of command line arguments                     */
/*            argv[1] - (command file)                                       */
/*                      dsc C user functions that are to be invoked          */
/*            argv[2] - (awaken_pid_file)                                    */
/*                      local file containing mypid for dsc_maple awaken     */
/*            argv[3] - (dsc_maple_pid_file)                                 */
/*                      file to contain dsc_maple deamon pid                 */
/*            argv[4] ... argv[n] intances arg1 ... argm                     */
/*                                                                           */
/* outputs:  file updates, (sub)problems, ...                                */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  daemon program to dispatch distributing commands to the         */
/*           dsc_maple deamon                                                */
/*                                                                           */
/*****************************************************************************/

main(argc,argv)
int argc;
char **argv;

{
int pid;      /* pid of the dsc_maple deamon                                 */
FILE *cfile;  /* command file pointer                                        */
void dsc_maple_setup();

#ifdef DEBUG
fprintf(stderr,"mes_maple: main             : starting dsc_maple_message \n");
fflush(stderr);
#endif

      dsc_maple_setup(argv[2]);
      pid=maple_getpid(argv[3]);

#ifdef DEBUG
fprintf(stderr,"mes_maple: main             : command file %s\n",argv[1]);
fprintf(stderr,"mes_maple: main             : local PID file %s\n",argv[2]);
fprintf(stderr,"mes_maple: main             : dsc_maple PID file %s\n",argv[3]);
fprintf(stderr,"mes_maple: main             : dsc_maple PID %d\n",pid);
fprintf(stderr,"mes_maple: main             : DSC C user function number %d\n"
        ,atoi(argv[4]));
fflush(stderr);
#endif

 unlink(argv[1]);
 cfile = fopen(argv[1],"w");
 if ( cfile == NULL )
  {
fprintf(stderr,"mes_maple: main             : could not open the file %s \n"
              ,argv[1]);
fprintf(stderr,"mes_maple: main             : errno %d\n",errno);
fprintf(stderr,"mes_maple: main             : forced halt!\n");
fflush(stderr);
exit(errno);
  }

      switch(atoi(argv[4])) {
      case MSG_MPLKILL:
      fprintf(stderr,"mes_maple: main             : exiting\n");
      fflush(stderr);
      fprintf(cfile,"%d\n",atoi(argv[4]));
      fclose(cfile);
       if ( kill(pid,SIGUSR2) == -1 )
  {
fprintf(stderr,"mes_maple: main             : could not awaken %d\n",pid);
   fflush(stderr);
   exit(0);
  }

       exit(0);
       break;
      case MSG_DSCPR_PWD:
       fprintf(cfile,"%d %s",atoi(argv[4]),argv[5]);
       break;
      case MSG_DSCPRSTART:
       fprintf(cfile,"%d \"%s\" \"%s\" \"%s\" \"%s\" %d %d %s\n",
               atoi(argv[4]),argv[5],argv[6],argv[7],argv[8],atoi(argv[9]),
               atoi(argv[10]),argv[11]);
       break;
      case MSG_DSCPRWAIT:
       fprintf(cfile,"%d %d %s",atoi(argv[4]),atoi(argv[5]),argv[6]);
       break;
      case MSG_DSCPRKILL:
       fprintf(cfile,"%d %d %s",atoi(argv[4]),atoi(argv[5]),argv[6]);
       break;
      case MSG_DSCPRNEXT:
       fprintf(cfile,"%d %s %s",atoi(argv[4]),argv[5],argv[6]);
       break;
      case MSG_DSCPRCOSETUP:
       fprintf(cfile,"%d",atoi(argv[4]));
       break;
      case MSG_DSCPRCOWAIT:
       fprintf(cfile,"%d %s",atoi(argv[4]),argv[5]);
       break;
      case MSG_DSCPRCORESUME:
       fprintf(cfile,"%d %d %s %s",atoi(argv[4]),atoi(argv[5]),argv[6],argv[7]);
       break;
      case MSG_DSCDBGSTART:
       fprintf(cfile,"%d %s",atoi(argv[4]),argv[5]);
       break;
      default:
       fprintf(stderr,"mes_maple: main             : invalid command in invokation\n");
       fflush(stderr);
       exit(1);
       break;
      }

 fclose(cfile);
#ifdef DEBUG
fprintf(stderr,"mes_maple: main             : awakening dsc_maple %d\n",pid);
fflush(stderr);
#endif

 if ( kill(pid,SIGUSR2) == -1 )
  {
fprintf(stderr,"mes_maple: main             : could not awaken %d\n",pid);
fflush(stderr);
exit(0);
  }
 dsc_maple_flag = FALSE;           /* wait in loop again                  */
 while ( !(dsc_maple_flag) )
 sleep(1);                         /* So that it is not a BUSY wait       */
 exit(0);
} 

/*****************************************************************************/
/*                                                                           */
/* subroutine: maple_getpid(name)                                            */
/*                                                                           */
/* inputs:   name - name of file to read for PID                             */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   integer pid found in the file "name"                            */
/*                                                                           */
/* purpose:  read pid found in the file "name"                               */
/*                                                                           */
/*****************************************************************************/

int maple_getpid(name)
char *name;
{
int pid;                   /*  DSC maple interface command instance          */
FILE *pfile;               /* command file pointer                           */

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_getpid     : name %s\n",name);
fflush(stderr);
#endif

pfile = fopen(name,"r");
if ( pfile == NULL )
 {
fprintf(stderr,"mes_maple: maple_getpid     : could not open %s\n",name);
fprintf(stderr,"mes_maple: maple_getpid     : errno %d\n",errno);
fprintf(stderr,"mes_maple: maple_getpid     : forced halt!\n");
fflush(stderr);
exit(errno);
 }
else
 {
  if ( fscanf(pfile,"%d",&pid) > 0 )
   {

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_getpid     : returning %d\n",pid);
fflush(stderr);
#endif

    fclose(pfile);
    return(pid);
   }
  else
   {

fprintf(stderr,"mes_maple: maple_getpid     : problem reading PID \n");
fprintf(stderr,"mes_maple: maple_getpid     : errno %d\n",errno);
fprintf(stderr,"mes_maple: maple_getpid     : forced halt!\n");
fflush(stderr);
exit(errno);
   }
 }

 fclose(pfile);

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_getpid     : returning void\n");
fflush(stderr);
#endif

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_maple_setup(name)                                         */
/*                                                                           */
/* inputs:   name - file to contain mypid                                    */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  set up for catch for SIGUSR2 and set the wait loop flag to false*/
/*                                                                           */
/*****************************************************************************/

void dsc_maple_setup(name)
char *name;
{
 FILE *mfile;               /* mypid file pointer                            */
 void dsc_maple_handle();   /* signal handler                                */

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_setup      : name %s\n",name);
fflush(stderr);
#endif

 dsc_maple_flag = FALSE;
 mypid = getpid();

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_setup      : dsc_maple_flag set to FALSE\n");
fprintf(stderr,"mes_maple: maple_setup      : mypid %d\n",mypid);
fflush(stderr);
#endif

 mfile = fopen(name,"w");
 if ( mfile == NULL )
  {
fprintf(stderr,"mes_maple: maple_setup      : could not open %s\n",name);
fprintf(stderr,"mes_maple: maple_setup      : errno %d\n",errno);
fprintf(stderr,"mes_maple: maple_setup      : forced halt!\n");
fflush(stderr);
exit(errno);
  }
 else
  {
   fprintf(mfile,"%d",mypid);
  }
 fclose(mfile);
 signal( SIGUSR2, dsc_maple_handle);

#ifdef DEBUG
fprintf(stderr,"mes_maple: maple_setup      : returning void\n");
fflush(stderr);
#endif

}

/*****************************************************************************/
/*                                                                           */
/* signal handler: dsc_maple_handle()                                        */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  reset the wait loop flag to true                                */
/*                                                                           */
/*****************************************************************************/

void dsc_maple_handle()
{

#ifdef DEBUG
fprintf(stderr,"mess_maple: maple_handle     : recieved SIGUSR2\n");
fflush(stderr);
#endif

dsc_maple_flag = TRUE;

}
