/*****************************************************************************/
/*                                                                           */
/*                                 dsc_blok.c                                */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_block( msg, s )                                          */
/*                                                                           */
/* inputs:   msg - msg requesting a block                                    */
/*           *s - socket descriptor                                          */
/*                                                                           */
/* outputs:  None                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:   mark sending problem as "blocked" in databse.                  */
/*            save socked descriptor for later "unblocking".                 */
/*            check for completion of all subproblems                        */
/*                                                                           */
/*****************************************************************************/

int dscd_block( msg, s)
DSCPRBLK *msg;
int s;
{
   int stat=DSC_WORKING;  /* assume subproblem(s) not done                   */
   DSCDB *pp;             /* problem database entry of parent problem        */
   DSCDB *pc;             /* problem database entry of child problem         */
   DSCDB *dscp_find();    /* function to find a problem in the database      */


#ifdef DBLKDEBUG
fprintf(stderr,"dsc_serv : dscd_block       : DSCPRBLK *msg, int s \n");
fprintf(stderr,"dsc_serv : dscd_block       : msg_type %ld msg_size %ld \n",
        ntohl(msg->msg_type),ntohl(msg->msg_size));
fprintf(stderr,"dsc_serv : dscd_block       : recived index to wait on %d \n"
        ,ntohl(msg->index));
fprintf(stderr,"dsc_serv : dscd_block       : PID of sender %ld \n"
        ,ntohl(msg->from_PID));
fflush(stderr);
#endif

   /*                                     */
   /* locate probelms in problem database */
   /*                                     */
   pp = dscp_find(0, ntohl(msg->from_PID));
   if (ntohl(msg->index) >= 0) {
      pc = dscp_find(4, ntohl(msg->index));
      /*                                                      */
      /* if (sub)problem is blocking on a subproblem which is */
      /* not its child, then unblock immediately.             */
      /*                                                      */
      if (pc->parent_PID != pp->prob_PID)
         pp = (DSCDB *) EOF;
    } /* end if waiting on a specific subproblem */
   /*                                                 */
   /* save information needed to unblock the sender   */
   /* when resume condition (specified by msg->index) */
   /* is met.                                         */
   /*                                                 */
   if (pp != (DSCDB *)EOF){

#ifdef DBLKDEBUG
fprintf(stderr,"dsc_serv : dscd_block       : saving information needed to \n");
fprintf(stderr,"dsc_serv : dscd_block       : unblock the sender\n");
fflush(stderr);
#endif

      pp->block_sok = s;   	          /* problem is blocked              */
      pp->block_sub = ntohl(msg->index);  /* resume condition                */
      blk_prob++;                         /* increase number of blocked prob */
      stat = sub_check( pp );             /* check if problem waited on is   */
                                          /* done                            */
      }
   /*                                                             */
   /* on error, (sub)problem is blocking on a subproblem which is */
   /* not its child, unblock sender immediately                   */
   /*                                                             */
   else {

#ifdef DBLKDEBUG
fprintf(stderr,"dsc_serv : dscd_block       : E : (sub)problem is blocking \n");
fprintf(stderr,"dsc_serv : dscd_block       : E : on a subproblem which is \n");
fprintf(stderr,"dsc_serv : dscd_block       : E : not its child, unblocking \n"
       );
fflush(stderr);
#endif

      solvmsg.src_name[0] = 0;
      solvmsg.prob_name[0] = 0;
      solvmsg.status = htonl( DSCERR_P_BADIDX );
      solvmsg.index = htonl(msg->index);
      solvmsg.act_prob = htonl(act_prob);
      solvmsg.blk_prob = htonl(blk_prob);
      write( s, &solvmsg, sizeof(DSCSOLV));

      /*                                                 */
      /* output error message to parent probelm log file */
      /*                                                 */
      sprintf( logstr, "%s: (waiting) block on invalid index %d\n",
        pp->prob_name, ntohl(msg->index));
      dsc_log( pp, logstr );

     /*                                              */
     /* blocking read waiting for socket termination */
     /*                                              */
      read( s, msg, 1 );
      close(s);                           /* on error, return socket         */
      }

#ifdef DBLKDEBUG
fprintf(stderr,"dsc_serv : dscd_block       : returning (%d) \n",stat);
fflush(stderr);
#endif

   return( stat );
}
