/*****************************************************************************/
/*                                                                           */
/*                              dsc_chkin.c                                  */
/*                                                                           */
/* Change History:                                                           */
/* ~~~~~~~~~~~~~~~                                                           */
/* (AAP) - Added Ctyp flag where needed for C++ file recognition.            */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_chkin( msg );                                            */
/*                                                                           */
/* inputs:   msg - pointer to message buffer received                        */ 
/*                                                                           */
/* outputs:  (update node database)                                          */
/*           (return checkin message to sender if needed)                    */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  process checkin message form remote node                        */
/*           record # of jobs, zero communication error count.               */
/*           if problems are declared dead as a result of                    */
/*             node state change, then restart the problems                  */
/*             (possibly on different node).                                 */
/*           if requested, return status of this DSC Server                  */
/*           send status to sender                                           */
/*                                                                           */
/*****************************************************************************/

int dscd_chkin( msg )
DSCKIN *msg;
{
   int rstat = DSC_SUCCESS; /* assume no problems with subroutine            */
   int p;                   /* loop variable                                 */
   NODE_REC *np;            /* node record of checking in node               */ 
   NODE_REC *dscn_find();   /* functin to find record of checking in node    */
   DSCDB *pp;               /* problem database entry                        */
   DSCDB *pa;               /* problem database entry of parent problem      */
   DSCDB *dscp_find();      /* function to find a problem in the database    */
   DSCPSRSTAT mmsg;         /* status message sent to sender                 */

   /*                                                    */
   /* locate the sender of this message in node database */
   /*                                                    */
   np = dscn_find( 0, msg->fnode,0);
   if (np==(NODE_REC *)EOF) goto XDONE;


#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : DSCKIN *msg \n");
fprintf(stderr,"dsc_serv : dscd_chkin       : msg_type %ld msg_size %ld \n",
        ntohl(msg->msg_type),ntohl(msg->msg_size));
fprintf(stderr,"dsc_serv : dscd_chkin       : remote node %s # %d \n",
        msg->fnname, np->index);
fprintf(stderr,"dsc_serv : dscd_chkin       : checking in with %ld act probs\n",
        ntohl(msg->act_prob));
fprintf(stderr,
        "dsc_serv : dscd_chkin       : checking in with %ld blocked probs\n",
        ntohl(msg->blk_prob));
if ( ntohl(msg->flag) == 0 ) 
 fprintf(stderr,"dsc_serv : dscd_chkin       : running \n");
if ( ntohl(msg->flag) == 1 )
 fprintf(stderr,"dsc_serv : dscd_chkin       : startup \n");
if ( ntohl(msg->flag) == 2 )
 fprintf(stderr,"dsc_serv : dscd_chkin       : halting \n");
fprintf(stderr,"dsc_serv : dscd_chkin       : reply %ld \n",ntohl(msg->reply));
fflush( stderr );
#endif

   /*                          */
   /* update the node database */
   /*                          */
   np->act_prob = ntohl(msg->act_prob); /* update number of active problems  */
   np->blk_prob = ntohl(msg->blk_prob); /* update number of blocked problems */
   np->err_cnt = 0;                     /* set remote node error count to 0  */

   /*                                         */
   /* if node is terminating or starting up,  */
   /* any problems we thought were executing  */
   /* are now lost.                           */
   /* Hopefully chkin_loop() caught this, but */
   /* if not, then take care of it here.      */
   /*                                         */ 
   if ((ntohl( msg->flag) == 2) || (ntohl(msg->flag) == 1)) {
     for( p=0; (pp=dscp_find(2, &p))!=(DSCDB *)EOF; ){
        /*                                          */ 
        /* if this problem was lost in the shuffle, */
        /* restart it again                         */
        /*                                          */
        if (pp->pnode_addr == np->naddr) {

          DSCSTRT msg2;

#ifdef DCHKINDEBUG
if ( ntohl( msg->flag) == 2 )
{
 fprintf(stderr,"dsc_serv : dscd_chkin       : rescheduling %s due to node \n"
         ,pp->prob_name);
 fprintf(stderr,"dsc_serv : dscd_chkin       : %s halting \n",np->nname);
}
if ( ntohl( msg->flag) == 1 )
{
 fprintf(stderr,"dsc_serv : dscd_chkin       : rescheduling %s due to node \n"
         ,pp->prob_name);
 fprintf(stderr,"dsc_serv : dscd_chkin       : %s starting up \n",np->nname);
}
fflush( stderr );
#endif

          msg2.msg_type = msg2.sanity = htonl( MSG_START );
          strcpy(msg2.dsc_password , password ); 
          msg2.msg_size = htonl(sizeof(DSCSTRT));
          msg2.subflg = htonl(pp->depth);
          strcpy( msg2.prob_name, pp->prob_name);
          msg2.parent_PID = htonl(pp->parent_PID);
          msg2.fnode = pp->pnode_addr;
          msg2.ctyp = htonl(pp->ctyp);
          msg2.Ctyp = htonl(pp->Ctyp);                /* (AAP) */
          msg2.ltyp = htonl(pp->ltyp);
          msg2.mtyp = htonl(pp->mtyp);
          msg2.ptyp = htonl(pp->ptyp);
          msg2.otyp = htonl(pp->otyp);
          msg2.ityp = htonl(pp->ityp);
          msg2.dtyp = htonl(pp->dtyp);
          msg2.cpu_type = htonl(pp->cpu_type);
          msg2.cpurating = htonl(pp->cpurating);
          msg2.memrating = htonl(pp->memrating);
          strcpy( msg2.src_name, pp->src_name);
          strcpy( msg2.src_build, pp->src_build);
          strcpy( msg2.src_exec, pp->src_exec);
          if ( pssched == TRUE ) 
          {
#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : searching for node using dynamic scheduling \n");
fflush( stderr );
#endif
           /* find node using dynamic scheduling */
           np = dscn_find( 4 , pp->cpurating, pp->memrating);

#ifdef CHKINLDEBUG
if ( np == (NODE_REC *) (EOF) )
 fprintf(stderr,"dsc_serv : dscd_chkin       : local noded chosen %s\n",
         my_node);
else
if  (np == (NODE_REC *) (DSCQUEUED) )
 fprintf(stderr,
 "dsc_serv : dscd_chkin       : could not find a suitable node must queue\n");
else
 fprintf(stderr,"dsc_serv : dscd_chkin       : %s chosen \n",np->nname);
fflush( stderr );
#endif

          }
          else 
          {

#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : searching for node using static scheduling \n");
fflush( stderr );
#endif

           /* find node using static scheduling */
           np = dscn_find( 1, 9999,0); /* force export */

#ifdef CHKINLDEBUG
if ( np != (NODE_REC*)(EOF) )
 fprintf(stderr,"dsc_serv : dscd_chkin       : %s chosen \n",np->nname);
else
 fprintf(stderr,"dsc_serv : dscd_chkin       : local nodes chosen %s\n",
         my_node);
fflush( stderr );
#endif

          }
       /*                                                         */
       /* when using dynamic scheduling and there is no available */
       /* processor                                               */
       /*                                                         */
       if ( (pssched == TRUE ) && ( np == (NODE_REC *) (DSCQUEUED) ) )
        {
        pp->prob_PID = 0;
        waiting_on_sub++;

#ifdef CHKINLDEBUG
fprintf(stderr,
        "dsc_serv : dscd_chkin       : no room at the inn %s is waiting \n"
        ,pp->prob_name);
fflush( stderr );
#endif

         goto XDONE;
        }
      /*                        */
      /* if not running locally */
      /*                        */
      if (np != (NODE_REC *) EOF) {
       pp->enode_addr = np->naddr;
       rstat = dscp_export( pp, dscp_find(0,pp->parent_PID), &np, &msg2);
       if (rstat != DSC_SUCCESS)  np = (NODE_REC *) EOF;
      } /* End of not running locally */
      /*                    */
      /* if running locally */
      /*                    */
      if (np == (NODE_REC *) EOF)
      {
       sprintf( logstr,
             "dsc_serv : dscd_chkin       : %s: running (sub)problem locally\n"
             , pp->prob_name);
       dsc_log( pp, logstr);
       pp->enode_addr = my_addr;
       if (strlen( msg2.src_build ))
        {
         rstat = dscp_build( pp );
         if (rstat != DSC_SUCCESS)
           {
            pa = dscp_find(0, ntohl(msg2.parent_PID));
            sprintf( logstr,
                  "dsc_serv : dscd_chkin       : %s: build failed: %s\n",
                   pp->prob_name, msg2.src_build);
            dsc_log( pp, logstr);
            fprintf( stderr, logstr);
            dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
                       DSCERR_P_BUILD|rstat);
            goto XDONE;
           } /* if build failed */
        } /* if build command exists */
      } /* End if running locally */
     } /* end if this problem is now dead */
    }  /* end for all problems in database */
   } /* end if node is now gone */
   
   /*                                         */
   /* if requested, return a NODE checkin msg */
   /*                                         */ 
   if (ntohl(msg->reply)) {

      DSCKIN  msg2;

#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : sending check in reply to \n");
fprintf(stderr,"dsc_serv : dscd_chkin       : %s \n",np->nname);
fflush( stderr );
#endif

      msg2.msg_type = msg2.sanity = htonl( MSG_CHECKIN );
      strcpy( msg2.dsc_password , password);
      msg2.msg_size = htonl( sizeof( DSCKIN ));
      msg2.fnode = my_addr;
      strcpy( msg2.fnname, my_node);
      msg2.act_prob = htonl(act_prob);
      msg2.blk_prob = htonl(blk_prob);
      msg2.flag = 0;
      msg2.reply = htonl(FALSE);

      /*                          */
      /* return a checkin message */
      /*                          */
      rstat = dsc_dg_send( &msg2, (int) sizeof( DSCKIN ),
         sockdg, np->portg, np->naddr);
      }

      /*                             */
      /* Send current CPU/MEM status */
      /* to checking in node         */
      /*                             */

#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : sending ps stats to %s\n"
        ,np->nname);
fflush( stderr );
#endif

      sleep(1);
      mmsg.msg_type = mmsg.sanity = htonl(MSG_PSRSTAT);
      strcpy(mmsg.dsc_password , password );
      mmsg.msg_size = htonl(sizeof(MSG_PSRSTAT));
      mmsg.fnode = my_addr;
      strcpy( mmsg.fnname, my_node);
      mmsg.cpu = cpu;
      mmsg.mem = mem;
      rstat = dsc_dg_send( &mmsg, (int) sizeof(DSCPSRSTAT), sockdg,
                      np->portg, np->naddr);
XDONE:

#ifdef DCHKINDEBUG
fprintf(stderr,"dsc_serv : dscd_chkin       : returning (%d) \n",rstat);
fflush( stderr );
#endif

   
   return( rstat );
}
