/*****************************************************************************/
/*                                                                           */
/*                               dsc_kill.c                                  */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_kill( msg )                                               */
/*                                                                           */
/* inputs:   msg - ptr to message requesting a kill                          */
/*                                                                           */
/* outputs:  (subproblem is killed)                                          */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  find requested problem                                          */
/*           set problem status and kill it in orderly fassion.              */
/*           remove killed problem from database                             */
/*                                                                           */
/*****************************************************************************/

int dscd_kill( msg )
DSCKILL *msg;
{
   int stat;                 /* problem termination status                   */
   DSCDB *pc;                /* pointer to problem data base entry           */
   DSCDB *dscp_find();       /* find problem in data base                    */
   NODE_REC *dscn_find();    /* find node in data base                       */

#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : DSCKILL *msg\n");
fprintf(stderr,"dsc_serv : dscd_kill()      : msg->prob_name %s\n",
                msg->prob_name);
fprintf(stderr,"dsc_serv : dscd_kill()      : msg->index %d\n",
                ntohl(msg->index));
fprintf(stderr,"dsc_serv : dscd_kill()      : msg->from_PID %d\n",
                htonl(msg->from_PID));
fflush(stderr);
#endif

   /*                      */
   /* find problem to kill */
   /*                      */
   if (htonl(msg->from_PID) == (-2)) 
        pc = dscp_find( 1, msg->prob_name);
   else pc = dscp_find( 4, ntohl(msg->index));

   if (pc == (DSCDB *)EOF)  /* validate existance */
    {
#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : problem not found\n");
fprintf(stderr,"dsc_serv : dscd_kill()      : returning 0\n");
fflush(stderr);
#endif 
       return(0);             
    }
#ifdef DSDKILL 
   else
   {
fprintf(stderr,"dsc_serv : dscd_kill()      : problem found\n");
fprintf(stderr,"dsc_serv : dscd_kill()      : pc->src_name %s\n",
        pc->src_name);
   }
#endif

   if (pc->prob_PID == ntohl(msg->from_PID))      /* can't kill self   */ 
    {
#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : can't kill self\n");
fprintf(stderr,"dsc_serv : dscd_kill()      : returning 0\n");
fflush(stderr);
#endif
      return(0);
    }

   if (pc->compl_flg)                             /* already complete        */
    {
#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : already complete\n");
fprintf(stderr,"dsc_serv : dscd_kill()      : returning 0\n");
fflush(stderr);
#endif
      return(0);                
    }

   if (pc->parent_PID == ntohl(msg->from_PID))    /* can only kill children  */
      stat = DSCERR_P_KILLPAR;
   else if (ntohl(msg->from_PID) == -1)           /* DSCCTRL is all powerful */
      stat = DSCERR_P_KILLCTRL;
   else if (ntohl(msg->from_PID) == -2)           /* RP, parent termination  */
      stat = DSCERR_P_ORPHANED;
   else
      return(0);

   if (pc->enode_addr == my_addr) act_prob--;


   /*                             */
   /* mark problem as completed   */
   /* note that call to dscn_find */
   /* may have a problem          */ 
   /*                             */

#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : marking as completed\n");
fflush(stderr);
#endif

   dscp_compl( pc,
	      dscp_find( 4, pc->parent_idx),
	      dscn_find( 1, pc->enode_addr,0),
	      pc->prob_name,
	      stat );

   /*                                 */
   /* remove this entry from database */
   /*                                 */

#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : removing from datavase\n");
fflush(stderr);
#endif

   dscp_del( pc );

#ifdef DSDKILL
fprintf(stderr,"dsc_serv : dscd_kill()      : returning 0\n");
fflush(stderr);
#endif

   return(0);
}

