/*****************************************************************************/
/*                                                                           */
/*                                 dsc_log.c                                 */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*     dsc_log() create/append log file and insert messages                  */
/*     dsc_err() create/append log file and insert deciphered error messages */
/*                                                                           */
/*****************************************************************************/

char logstr[256];

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_log( pp, sptr )                                           */
/*                                                                           */
/* inputs:   pp - pointer to problem record                                  */
/*           sptr - pointer to string to log                                 */
/*                                                                           */
/* outputs:  (log file has messages added to it)                             */
/*                                                                           */
/* return:   0                                                               */
/*                                                                           */
/* purpose:  construct log file name.                                        */
/*           open/create log file for problem (append mode) and write        */
/*            string into it.                                                */
/*           close log file.                                                 */
/*                                                                           */
/*****************************************************************************/

int dsc_log( pp, sptr)
DSCDB *pp;
char *sptr;
{
   char fid[DSC_FILEZ+1];     /* log file id                                 */
   FILE *fp;                  /* log file to open/create                     */
   char *dscd_getword();      /* get name w/o path information function      */
   char fname[DSC_FILEZ+8];   /* fid name w/o path information               */
   char fpath[DSC_FILEZ+8];   /* fid name w/ path information                */

#ifdef DLOG
fprintf(stderr,"dsc_serv : dsc_log()        : DSCDB *pp\n");
fprintf(stderr,"dsc_serv : dsc_log()        : pp->src_name %s\n",
              pp->src_name);
fprintf(stderr,"dsc_serv : dsc_log()        : char *sptr %s\n",
              sptr);
fflush(stderr);
#endif

   /*                         */
   /* construct log file name */
   /*                         */
   strcpy(fpath,pp->src_name);
   strcpy(fname,dscd_getword(fpath));
   sprintf( fid, "%s.log", fname);

   /*                                 */
   /* open/create file in append mode */
   /*                                 */

   fp = fopen( fid, "a");   /* open for append at EOF */
   if (fp == NULL) {
     perror( "opening log file");
fprintf(stderr,"dsc_serv : dsc_log()        : error opening log file\n");
fprintf(stderr,"dsc_serv : dsc_log()        : %s.d\n", pp->prob_name);    
fprintf(stderr,"dsc_serv : dsc_log()        : errno %d\n",errno);
fprintf(stderr,"dsc_serv : dsc_log()        : fid %s\n",fid);
fflush(stderr);
   }
   else {

#ifdef DLOG
fprintf(stderr,"dsc_serv : dsc_log()        : writing to log file %s\n",
               fid);
fflush(stderr);
#endif

     fputs( sptr, fp);
     fclose (fp);
   }

#ifdef DLOG
fprintf(stderr,"dsc_serv : dsc_log()        : returning 0\n");
fflush(stderr);
#endif

 return(0);

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_err(pp, pstat)                                            */
/*                                                                           */
/* inputs:   pp - pointer to problem record                                  */
/*           pstat - problem status mask                                     */
/*                                                                           */
/* outputs:  (error messages written to log file)                            */
/*                                                                           */
/* return:   0                                                               */
/*                                                                           */
/* purpose:  open/create log file for problem (append mode)                  */
/*           write error message for each bit which is set                   */
/*           close log file                                                  */
/*                                                                           */
/*****************************************************************************/

int dsc_err( pp, pstat)
DSCDB *pp;
long pstat;
{
   char fid[DSC_FILEZ+1];     /* log file id                                 */
   char fname[DSC_FILEZ+8];   /* fid name w/o path information               */
   char fpath[DSC_FILEZ+8];   /* fid name w/ path information                */
   char *cptr;
   FILE *fp;                  /* log file to open/create                     */
   char *dscd_getword();      /* get name w/o path information function      */

#ifdef DERR
fprintf(stderr,"dsc_serv : dsc_err()        : DSCDB *pp\n");
fprintf(stderr,"dsc_serv : dsc_err()        : pp->src_name %s\n",
              pp->src_name);
fprintf(stderr,"dsc_serv : dsc_err()        : pstat %ld\n",pstat);
fflush(stderr);
#endif

   cptr = strrchr( pp->src_name);
   if (cptr == 0) cptr = pp->src_name;

   /*                         */
   /* construct log file name */
   /*                         */

   strcpy(fpath,pp->src_name);
   strcpy(fname,dscd_getword(fpath));
   sprintf( fid, "%s.log", fname);

   fp = fopen( fid, "a");   /* open for append at EOF */
   if (fp == NULL) {
     perror( "opening log file");
fprintf(stderr,"dsc_serv : dsc_err()        : error opening log file\n");
fprintf(stderr,"dsc_serv : dsc_err()        : %s.d\n", pp->prob_name);
fprintf(stderr,"dsc_serv : dsc_err()        : errno %d\n",errno);
fprintf(stderr,"dsc_serv : dsc_err()        : fid %s\n",fid);
fflush(stderr);
   }
   else {

#ifdef DERR
fprintf(stderr,"dsc_serv : dsc_err()        : writing to log file %s\n",
               fid);
#endif


     if (pstat & DSCERR_P_GENERAL)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; problem failure\n",
	       cptr);
     if (pstat & DSCERR_P_NOSOURCE)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; source file not found\n",
	       cptr);
     if (pstat & DSCERR_P_BUILD)
       fprintf( fp,
               "\tTERMINATION CONDITION: %s; termination during build\n",
	       cptr);
     if (pstat & DSCERR_P_EXEC)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; termination during execution\n",
	       cptr);
     if (pstat & DSCERR_P_DBFULL)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; DATABASE full\n",
	       cptr);
     if (pstat & DSCERR_P_STATUS)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; non-zero status at termination\n",
	       cptr);
     if (pstat & DSCERR_P_RETCODE)
       fprintf( fp,
	      "\tTERMINATION CONDITION: %s; non-zero retcode at termination\n",
	       cptr);
     if (pstat & DSCERR_P_COREDUMP)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; termination with coredump\n",
	       cptr);
     if (pstat & DSCERR_P_KILLPAR)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; killed by parent\n",
	       cptr);
     if (pstat & DSCERR_P_KILLCTRL)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; killed by DSC Control\n",
	       cptr);
     if (pstat & DSCERR_P_NOPAR)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; parent problem not found\n",
	       cptr);
     if (pstat & DSCERR_P_FORKING)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; FORKING\n",
	       cptr);
     if (pstat & DSCERR_P_DUPNAME)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; duplicate problem name\n",
	       cptr);
     if (pstat & DSCERR_P_COMM)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; communicatin error\n",
	       cptr);
     if (pstat & DSCERR_P_BADIDX)
       fprintf( fp,
            "\tTERMINATION CONDITION: %s; bad index specified during wait\n",
	       cptr);
     if (pstat & DSCERR_P_NODEHALT)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; local node halt\n",
	       cptr);
     if (pstat & DSCERR_P_ORPHANED)
       fprintf( fp,
    "\tTERMINATION CONDITION: %s; parent problem terminated or non-existant\n",
	       cptr);
     if (pstat & DSCERR_S_NODEHALT)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; subproblem node halt\n",
	       cptr);
     if (pstat & DSCERR_S_NODESTRT)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; node restart, subproblem lost\n",
	       cptr);
     if (pstat & DSCERR_S_GENERAL)
       fprintf( fp,
	       "\tTERMINATION CONDITION: %s; subproblem failure\n",
	       cptr);

     fclose (fp);
   }

#ifdef DERR
fprintf(stderr,"dsc_serv : dsc_err()        : returning 0\n");
fflush(stderr);
#endif
 
 return(0);

}
