/*****************************************************************************/
/*                                                                           */
/*                                 dsc_misc.c                                */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*      dsc_read()       - read designed to handle multipart messages        */
/*      dsc_dg_send()    - send message to destination                       */
/*      dsc_exec()       - execute a command string                          */
/*      sub_check()      - check if problem waited on is done                */
/*      subproc_signal() - deal with subprocess termination                  */
/*      subproc_body()   - process terminated subprocess                     */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* subroutine: int dsc_read( d, buf, nbytes)                                 */
/*                                                                           */
/* inputs:   d      - descriptor                                             */
/*           buf    - message buffer                                         */
/*           nbytes - number of bytes to read                                */
/*                                                                           */
/* outputs:  (receive a file)                                                */
/*                                                                           */
/* return:   length of message to read                                       */
/*                                                                           */
/* purpose: this function will piece together a whole message when it        */
/*          arrives in more than one message. ( this has been observed to    */
/*          happen sometimes with messages over 1024  bytes in length,       */
/*          It may happen other times too....)                               */
/*                                                                           */
/*****************************************************************************/

int dsc_read( d, buf, nbytes)
int d;
DSCKIN_PTR buf;
int  nbytes;
{
  int cc;               /* number of bytes actually  read on success         */
  int len;              /* size of message                                   */
  int remaining;        /* number of remaining bytes to be read              */
  char *xbuf;           /* pointer to message buffer                         */

#ifdef DREAD
fprintf(stderr,"dsc_serv : dsc_read         : int d %d\n",d);
fprintf(stderr,"dsc_serv : dsc_read         : DSCKIN_PTR buf\n");
fprintf(stderr,"dsc_serv : dsc_read         : int nbytes %d\n",nbytes);
#endif

  cc = read( d, buf, nbytes );
  if (cc == 0) return(0);
  else if (cc > 8) len = ntohl(buf->msg_size);
  else {
fprintf(stderr,"dsc_serv : dsc_read         : less than 8 bytes received\n");
fprintf(stderr,"dsc_serv : dsc_read         : %d bytes recieved\n",cc);
fprintf(stderr,"dsc_serv : dsc_read         : errno %d\n",errno);
fprintf(stderr,"dsc_serv : dsc_read         : fatal error !!!\n");
    exit(errno);
    }


  xbuf = (char *) buf;
  remaining = len - cc;
  while (remaining > 0) {

fprintf(stderr,"dsc_serv : dsc_read         : another read required\n");
fprintf(stderr,"dsc_serv : dsc_read         : for %d bytes\n", remaining);

    /*                                    */
    /* calc pointer for begging next read */
    /*                                    */
     xbuf += cc;

    /*                      */
    /* read rest of message */
    /*                      */
     cc = read( d, xbuf, remaining );

    /*                                                        */
    /* determine if we are done yet, done when remaining == 0 */
    /*                                                        */
     remaining -= cc;
  }

#ifdef DREAD
fprintf(stderr,"dsc_serv : dsc_read         : returning (%d)\n",len);
#endif

  return( len );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_dg_send( msg, len, sok, dstport, dstaddr )                */
/*                                                                           */
/* inputs:   char *msg         - message to send out datagram socket         */
/*           int len           - length of message                           */
/*           int sok           - datagram socket                             */
/*           unsigned long dstport  - destination port address               */
/*           unsigned long dstaddr  - destination node address               */
/*                                                                           */
/* outputs:  (message sent to remote node/process)                           */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  build message destination description                           */
/*           send message to destination                                     */
/*                                                                           */
/*****************************************************************************/

int dsc_dg_send( msg, len, sok, dstport, dstaddr )
DSCKIN *msg;
int len;
int sok;
unsigned short dstport;
unsigned long dstaddr;
{
   int rstat=DSC_SUCCESS;
   struct sockaddr_in server;

#ifdef DDGSEND
fprintf(stderr,"dsc_serv : dsc_dg_send()    : DSCKIN *msg\n");
fprintf(stderr,"dsc_serv : dsc_dg_send()    : len %d\n",len);
fprintf(stderr,"dsc_serv : dsc_dg_send()    : sok %d\n",sok);
fprintf(stderr,"dsc_serv : dsc_dg_send()    : ushort dstport %d\n",dstport);
fprintf(stderr,"dsc_serv : dsc_dg_send()    : ulong dstaddr\n");
fprintf(stderr,"dsc_serv : dsc_dg_send()    : %lu.%lu.%lu.%lu\n",
              (dstaddr&0xff000000) >> 24,
              (dstaddr&0x00ff0000) >> 16,
              (dstaddr&0x0000ff00) >>  8,
              (dstaddr&0x000000ff) );
fflush(stderr); 
#endif

   /*                              */
   /* build destination descriptor */
   /*                              */
   if (dstaddr == 0) dstaddr = my_addr;
   if (dstport == 0) dstport = htons( portnm );
   server.sin_family = AF_INET;
   server.sin_port = dstport;
   server.sin_addr.s_addr = dstaddr;

   /*                         */
   /* send the message        */
   /* handle signal interupts */
   /*                         */

#ifdef DDGSEND
fprintf(stderr,"dsc_serv : dsc_dg_send()    : sending the message \n");
fflush(stderr);
#endif

   do {
      errno = 0;
      sendto( sok, msg, len, 0, &server, sizeof( server ));
      if (errno == ENOBUFS) sleep(2);
      else if ((errno != EINTR) && (errno != 0)) {
         perror( "sending datagram");
fprintf(stderr,"dsc_serv : dsc_dg_send()    : error sending datagram\n");
fprintf(stderr,"dsc_serv : dsc_dg_send()    : errno %d\n",errno);
         rstat = errno;
       }
   } while ((errno == EINTR) || (errno == ENOBUFS));

#ifdef DDGSEND
fprintf(stderr,"dsc_serv : dsc_dg_send()    : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_exec( str )                                               */
/*                                                                           */
/* inputs:   char *str - command to execute                                  */
/*                                                                           */
/* outputs:  (new image is executed)                                         */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  nice our priority to the configured level                       */
/*           execute a command string - this function does not return        */
/*                 unless the command could not be fulfilled.                */
/*                                                                           */
/*****************************************************************************/

static char *argv[100];

int dsc_exec( str )
char *str;
{
   char *cptr,*name,*sptr;
   int i;

#ifdef DEXEC
fprintf(stderr,"dsc_serv : dsc_exec()       : char *str %s\n",str);
fflush( stderr );
#endif

      /*                                           */
      /* problems run at lower priority since they */
      /* tend to be very CPU bound                 */
      /* nice priority according to configuration  */
      /*                                           */
      nice( nice2 );


      /*                                                     */
      /* build argument list for program we are about to run */
      /*                                                     */

      name = str;                         /* name of compiler/program */
      sptr = str;                         /* beginning of command */
      for( i=0; sptr != 0; i++) {	  /* arguments to compiler */
         cptr = strchr( sptr, ' ');
         if (cptr != 0) *cptr++ = 0;
         argv[i] = sptr;
         sptr = cptr;

#ifdef DEXEC
fprintf(stderr,"dsc_serv : dsc_exec()       : argv[%d] %s\n",i,argv[i]);
fflush( stderr );
#endif

         }
      argv[i] = 0;


      i = execvp( name, argv);			/* run program */
      perror( "failure in execvp");
      fprintf(stderr,"dsc_serv : dsc_exec()       : failure in execvp\n");
      fprintf(stderr,"dsc_serv : dsc_exec()       : errno %d\n",errno);
      fflush( stderr );
      
      exit(errno);                              /* clean up I/O and term */
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: sub_check( pp )                                               */
/*                                                                           */
/* inputs:   pp - pointer to problem to be checked                           */
/*                                                                           */
/* outputs:  (unblock parent problem if all conditions are met)              */
/*                                                                           */
/* return:   DSC_SUCCESS = all subproblems complete                          */
/*           DSC_WORKING = some subprobles still working                     */
/*           n           = some other error                                  */
/*                                                                           */
/* purpose:  check if problem waited on is done                              */
/*              check for specified subproblem index                         */
/*              check for "next" completed subproblem                        */
/*              check for all subproblems complete                           */
/*           adjust server counters                                          */
/*           send resume message to blocked problem                          */
/*           close the stream socket                                         */
/*                                                                           */
/*****************************************************************************/

  
int sub_check( pp )
DSCDB *pp;
{
   int rstat=DSC_WORKING;              /* assume resume condition is not met */
   DSCDB *dscp_find();                 /* find problem in data base function */

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : DSCDB *pp\n");
fprintf(stderr,"dsc_serv : sub_check()      : src_name\n",pp->src_name);
fprintf(stderr,"dsc_serv : sub_check()      : pp->block_sok %d\n",
              pp->block_sok);
fprintf(stderr,"dsc_serv : sub_check()      : pp->block_sub %d\n",
              pp->block_sub);
fflush(stderr);
#endif

    /*                              */
    /*  unblock the parent problem  */
    /*                              */
    if (pp->block_sok == EOF) goto XDONE;

/*                                                                           */
/*                                                                           */
/*                if waiting on a particular subproblem                      */
/*                                                                           */
/*                                                                           */
    if (pp->block_sub >= 0) {
      DSCDB *pi;

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : wating on a particular subproblem\n");
fflush(stderr);
#endif

      pi = dscp_find( 4, pp->block_sub);

      if (pi->parent_idx!=pp->index) {           /* invalid index specified */

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : pi->parent_idx %d\n",
               pi->parent_idx);
fprintf(stderr,"dsc_serv : sub_check()      : pp->index %d\n",
               pp->index);
fprintf(stderr,"dsc_serv : sub_check()      : invalid index specified!\n");
fflush(stderr);
#endif

         strcpy( solvmsg.prob_name, pi->prob_name);
         strcpy( solvmsg.src_name, pi->src_name);
         solvmsg.status = htonl(pi->compl_stat);
         solvmsg.index = htonl( DSC_FAILURE );   /* report error             */
         rstat = DSC_SUCCESS;                    /* ok to unblock            */
         }
      else if  (pi->compl_flg) {                 /* subproblem is complete   */

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : subproblem is complete!\n");
fflush(stderr);
#endif

         strcpy( solvmsg.prob_name, pi->prob_name);
         strcpy( solvmsg.src_name, pi->src_name);
         solvmsg.status = htonl(pi->compl_stat);
         solvmsg.index = htonl( pi->index );
         rstat = DSC_SUCCESS;                    /* ok to unblock            */
         }    /* if this subproblem is complete                              */
      }       /* end if waiting on a particular subproblem                   */
/*                                                                           */
/*                                                                           */
/*                        if waiting on next subproblem                      */
/*                                                                           */
/*                                                                           */
    else if (pp->block_sub == (-2)) {
       DSCDB *pi;
       long i;

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : waiting on next subproblem\n");
fflush(stderr);
#endif

      /*                    */
      /* check for children */
      /*                    */
       if (pp->num_sub <= 0) {
          fprintf( stderr,
"dsc_serv : sub_check()      : problem %s waiting for next when no subproblems exist\n",
	    pp->prob_name);
          fflush( stderr);
          strcpy( solvmsg.prob_name, "");
          strcpy( solvmsg.src_name, "");
          solvmsg.status = htonl( DSC_FAILURE );
          solvmsg.index = htonl( -1 );
          rstat = DSC_SUCCESS;
          }      /* erroneous wait */

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : searching for a complete subproblem\n");
fflush(stderr);
#endif

       for (i=pp->child_next; (i != (-1)); i=pi->peer_next) {
          pi = dscp_find(4,i);
          if (pi == (DSCDB *)EOF) break;
          if ((pi->index == i) && (pi->compl_flg)) {

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : found pi->src_name %s\n",
              pi->src_name);
fflush(stderr);
#endif

             strcpy( solvmsg.prob_name, pi->prob_name);
             strcpy( solvmsg.src_name, pi->src_name);
             solvmsg.status = htonl( pi->compl_stat );
             solvmsg.index = htonl( pi->index );
             rstat = DSC_SUCCESS;
             pp->block_sub = (-1);
             /*                                          */
             /* problem must be deleted from database so */
             /* we dont resume on it again next time.    */
             /*                                          */
             dscp_del( pi );
             break;
	     }   /* found a completed subproblem        */
          }      /* searching for a complete subproblem */
     }           /* end if waiting for next             */
/*                                                                           */
/*                                                                           */
/*                       if waiting on all subproblems                       */
/*                                                                           */
/*                                                                           */
    else if (pp->block_sub == (-1)) {

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : waiting on all subproblems\n");
fprintf(stderr,"dsc_serv : sub_check()      : numsub %d awksub %d \n"
              ,pp->num_sub,pp->awk_sub);
fflush(stderr);
#endif

       if ( (pp->num_sub != 0 ) && (pp->awk_sub == 0) )
          {

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : wait for coroutines complete!\n");
fflush(stderr);
#endif

           strcpy( solvmsg.prob_name, pp->prob_name );
           strcpy( solvmsg.src_name, pp->src_name );
           solvmsg.status = htonl(pp->compl_stat);
           solvmsg.index = htonl( -1 );
           write( pp->block_sok, &solvmsg, sizeof( DSCSOLV ));
           /*                                       */
           /* blocking read until socket terminates */
           /*                                       */
           read( pp->block_sok, &solvmsg, 1);
           close( pp->block_sok);
           pp->block_sok = EOF;
           blk_prob--;
           rstat = DSC_SUCCESS;
           goto XXDONE;
          }

       if (pp->num_sub == 0) {       /* if no active subproblems */

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : no active subproblems\n");
fflush(stderr);
#endif

          strcpy( solvmsg.prob_name, pp->prob_name );
          strcpy( solvmsg.src_name, pp->src_name );
          solvmsg.status = htonl(pp->compl_stat);
          solvmsg.index = htonl( -1 );
          rstat = DSC_SUCCESS;       /* ok to resume */
          } /* end if parent needs to be un-blocked */
      else rstat = DSC_WORKING;
      }     /* end if waiting on all */

   if (rstat == DSC_SUCCESS) {

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : rstat == DSC_SUCCESS\n");
fflush(stderr);
#endif

          sprintf( logstr, "%s %s : unblocking flag = %d\n",
               pp->prob_name, pp->src_name,pp->block_sub);
          dsc_log( pp, logstr );

#ifdef SUBCHECK
          fprintf( stderr,"dsc_serv : sub_check()      : %s",logstr);
          fflush(stderr);
#endif

          blk_prob--;
          write( pp->block_sok, &solvmsg, sizeof( DSCSOLV ));
          /*                                       */
          /* blocking read until socket terminates */
          /*                                       */
          read( pp->block_sok, &solvmsg, 1);
          /*                                      */
          /*   set problem as no longer blocked   */
          /*                                      */
          close( pp->block_sok);
          pp->block_sok = EOF;
          rstat = DSC_SUCCESS;
   }
#ifdef SUBCHECK
   else
    {
fprintf(stderr,"dsc_serv : sub_check()      : rstat != DSC_SUCCESS\n");
fflush(stderr);
    }
#endif

XDONE:
   /*                                                   */
   /* when all subproblems are done, clean them from DB */
   /*                                                   */
   if (pp->num_sub <= 0) {
     int i;
     DSCDB *pi;

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : all subproblems are done\n");
fprintf(stderr,"dsc_serv : sub_check()      : cleaning database\n");
fflush(stderr);
#endif

     for (i=pp->child_next; (i!=(-1)); i=pi->peer_next) {
        pi = dscp_find(4,i);
        if (pi != (DSCDB *)(EOF)) {
	  dscp_del(pi);
	  } /* end if database entry is valid */
     }  /* end for all children */
   }  /* end of subproblem cleanup */

XXDONE:

#ifdef SUBCHECK
fprintf(stderr,"dsc_serv : sub_check()      : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* signal handler: subproc_signal()                                          */
/*                                                                           */
/* inputs:    none (called by UNIX asynchronously as a signal)               */
/*                                                                           */
/* outputs:   none                                                           */
/*                                                                           */
/* return:    0                                                              */
/*                                                                           */
/* purpose:  deal with subprocess termination                                */
/*           determine which subprocess terminated (loop up PID in database) */
/*           call subproc_body to process the event                          */
/*           loop for all currently terminated subprocesses                  */
/*                                                                           */
/*****************************************************************************/
extern struct rusage rusage;
extern union wait statusp;

void subproc_signal( )
{
   int pid;
   void subproc_body();

#ifdef SUBPSIGNAL
fprintf(stderr,"dsc_serv : subproc_signal() : void\n");
fflush(stderr);
#endif

   /*                                            */
   /*    DETERMINE WHICH SUBPROCESS TERMINATED   */
   /*                                            */
   do {
      pid = wait3( &statusp, WNOHANG, &rusage );
      if (pid > 0) subproc_body( pid );
    } while( pid > 0);

#ifdef SUBPSIGNAL
fprintf(stderr,"dsc_serv : subproc_signal() : returning  void\n");
fflush(stderr);
#endif

} 

/*****************************************************************************/
/*                                                                           */
/* subroutine: subproc_body( PID )                                           */
/*                                                                           */
/* inputs:     PID - process id of terminated child process                  */
/*                                                                           */
/* outputs:    (datagram messages, log file messages, database updates)      */
/*                                                                           */
/* return:    0                                                              */
/*                                                                           */
/* purpose:  determine completion status                                     */
/*           look up problem in database                                     */
/*           if build complete ...                                           */
/*               if bad status, record problem complete                      */
/*               start the problem (record problem complete if this fails)   */
/*           if problem complete                                             */
/*               log completion                                              */
/*           check to see if there are queued problems to start              */
/*               fork build of (sub)problem                                  */
/*                                                                           */
/*****************************************************************************/

void subproc_body( pid )
int pid;
{
   int i;
   int pstat=0;
   DSCDB *pp, *dscp_find();

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : pid %d\n",pid);
fflush(stderr);
if ((statusp.w_coredump) || (statusp.w_retcode) || (statusp.w_status)) {
fprintf(stderr,"dsc_serv : subproc_body()   : signal cordump %d\n"
              ,statusp.w_coredump);
fprintf(stderr,"dsc_serv : subproc_body()   : signal retcode %d\n"
              ,statusp.w_retcode);
fprintf(stderr,"dsc_serv : subproc_body()   : signal status  %d\n"
              ,statusp.w_status);
}
fflush(stderr);
#endif

   /*                                    */
   /*   DETERMINE STATUS OF SUBPROCESS   */
   /*                                    */
   if (statusp.w_status)
     pstat |= DSCERR_P_STATUS;
   if (statusp.w_coredump) 
     pstat |= DSCERR_P_COREDUMP;
   if (statusp.w_retcode) 
     pstat |= DSCERR_P_RETCODE;

   /*                                 */
   /* find problem in database,       */
   /*                                 */
   if (pid<=0) goto XDONE;
   pp = dscp_find(0,  pid);
   if (pp == (DSCDB *) (EOF))
     {
       if ( pid == pspid )
         {
fprintf(stderr,"dsc_serv : subproc_body()   : PS DEAMON died or could not be started.\n");
fprintf(stderr,"dsc_serv : subproc_body()   : using round robin scheduling!\n");
fflush(stderr);
          pssched = FALSE; 
         }
       goto XDONE;                  /* return, but restore signals */
     }
   pp->prob_PID = -1;             /* this PID is now gone */

   /*                                               */
   /*   IF BUILD JUST FINISHED, START THE PROBLEM   */
   /*                                               */
   if (pp->build_flg == FALSE) {
      sprintf( logstr,
	      "%s: build complete, status %d, retcode %d\n",
              pp->src_name,
	      statusp.w_status,
	      statusp.w_retcode);
      dsc_log( pp, logstr );

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : %s\n",logstr);
fflush(stderr);
#endif

      /*                    */
      /* note build failure */
      /*                    */
      if (pstat) {

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : build failure\n");
fprintf(stderr,"dsc_serv : subproc_body()   : pp->src_name %s\n",
               pp->src_name);
fprintf(stderr,"dsc_serv : subproc_body()   : pp->src_build %s\n",
               pp->src_build);
fflush(stderr);
#endif

        act_prob--;
        dscp_compl( pp, dscp_find( 4, pp->parent_idx),
             pp->pnode_addr, pp->prob_name, pstat|DSCERR_P_BUILD);
        goto XDONE;
        }

      /*                                                                    */
      /* This does not inform the parent (if it is off node) that the build */
      /* was completed successfully. In order to properlly update the       */ 
      /* problem data base of the off node parent a message must be sent to */
      /* the off node parent. This is not yet implemented but will be soon  */
      /*                                                                    */

      pp->build_flg = TRUE;

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : forking exec %s \n",
               pp->src_exec);
fflush(stderr);
#endif

      pp->prob_PID = fork();
      if (pp->prob_PID == 0) {		/* CHILD */
        dsc_exec( pp->src_exec );
        exit(errno);
        }                       /* end if this process is child */
      else {                    /* else PARENT */

         /*                           */
         /* make sure the fork worked */
         /*                           */
         if (pp->prob_PID == (-1)) {
            perror( "forking subproblem");
fprintf(stderr,"dsc_serv : subproc_body()   : error forking subproblem\n");
fprintf(stderr,"dsc_serv : subproc_body()   : exec %s \n",
               pp->src_exec);
fprintf(stderr,"dsc_serv : subproc_body()   : errno %d\n",errno);
fflush(stderr);
            act_prob--;
            dscp_compl( pp, dscp_find( 4, pp->parent_idx),
		       pp->pnode_addr, pp->prob_name,
		       DSCERR_P_FORKING|DSCERR_P_EXEC);
            goto XDONE;
	    }                   /* end if error forking */
         }                      /* end if this process is the parent */
      }                         /* if the child was building the subproblem */
   /*                            */
   /*     SUBPROBLEM COMPLETE    */
   /*                            */
   else {
      if (pstat) pstat |= DSCERR_P_EXEC;
      sprintf( logstr,
	      "%s: exec complete, status %d, retcode %d\n",
	      pp->prob_name,
	      statusp.w_status,
	      statusp.w_retcode);
      dsc_log( pp, logstr );

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : %d\n",logstr);
fflush(stderr);
#endif

      act_prob--;
      dscp_compl( pp, dscp_find(4, pp->parent_idx), 
        pp->pnode_addr, pp->prob_name, pstat);
      }                                 /* end if problem complete */

   /*                                                   */
   /* check to see if we can now start a new subproblem */
   /*                                                   */
   while ((act_prob < DSC_CPU_LIMIT) && (waiting_on_sub > 0) && 
          ( pssched != TRUE ) ) {

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : round robin scheduling\n");
fprintf(stderr,"dsc_serv : subproc_body()   : DSC_CPU_LIMIT %d allows starting new job\n",DSC_CPU_LIMIT);
fflush(stderr);
#endif

      i = 0;
      while( (pp=dscp_find( 2, &i)) != (DSCDB *)(EOF)) {
	if (pp->prob_PID == (0)) {

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : attempting to start %s\n",
          pp->prob_name);
fprintf(stderr,"dsc_serv : subproc_body()   : pp->src_name\n",
          pp->src_name);
fflush(stderr);
#endif

	   if ((pstat=dscp_build( pp )) != DSC_SUCCESS) {
             dscp_compl( pp, dscp_find( 4, pp->parent_idx),
                 pp->pnode_addr, pp->prob_name, pstat|DSCERR_P_BUILD);
 	     }   /* end if build failed */
           break;
	}        /* end if this job needs running */
      }          /* end for all jobs in database */
   }             /* end if new DSC_CPU_LIMIT slot is available */
XDONE:   

#ifdef SUBPBODY
fprintf(stderr,"dsc_serv : subproc_body()   : returning void\n");
fflush(stderr);
#endif

   fflush(stderr );   /* flush regardless of debug switches */

   return;
} 
