/*****************************************************************************/
/*                                                                           */
/*                              dsc_prob.c                                   */
/*                                                                           */
/* Change History:                                                           */
/* ~~~~~~~~~~~~~~~                                                           */
/* (AAP) - Added Ctyp processing for recognition of C++ files.               */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */
#include <sys/time.h>     /* time functions                                  */

/*****************************************************************************/
/*                                                                           */
/* functions :                                                               */
/*     dscd_sleep( msg )          handle subproblem is sleeping message      */
/*     dscd_awaken( msg , s)      handle subproblem awken message            */
/*     dscd_start( msg, s)        start a subproblem                         */
/*     dscd_export( pp, pa, np )  send files and start message to target node*/
/*     dscp_build( )              fork a subprocess to build problem         */
/*     dscp_find( flg, arg )      find problem in problem database           */
/*     dscp_init()                initialize problem database                */
/*     dscp_unlink( pc )          unlink a child problem from a parent       */
/*     dscp_del( pp )             delete a problem and dependants from dbase */
/*     dscp_compl( )              declare a problem as complete              */
/*     dscp_begin()               return the problem index to the parent     */
/*     dscd_getword()             extract name from FID                      */
/*                                                                           */
/*****************************************************************************/

static DSCDB prob_db[ DSC_PROBX ];    /* problem database                    */
static int   pcnt;

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_init()                                                   */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  initialize problem database to all bits set                     */
/*                                                                           */
/*****************************************************************************/

int dscp_init()
{

#ifdef  DSPINIT
fprintf(stderr,"dsc_serv : dscp_init()      : void\n");
#endif

   memset( &prob_db[0], ~0, DSC_PROBX*sizeof( DSCDB ));

#ifdef  DSPINIT
fprintf(stderr,"dsc_serv : dscp_init()      : returning DSC_SUCCESS\n");
#endif

   return( DSC_SUCCESS );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_find( flg, arg )                                         */
/*                                                                           */
/* inputs:   flg  0 = find by PID                                            */
/*                1 = find by prob name                                      */
/*                2 = find next used by offset w/ incriment                  */
/*                3 = next free (unused) element                             */
/*                4 = find by offset                                         */
/*           arg  PID, prob_name, or offset                                  */
/*                                                                           */
/* outputs:  arg  offset of current element +1 (only if flg == 2)            */
/*                                                                           */
/* return:   (DSCDB *) = pointer to database element                         */
/*                 EOF = element not found                                   */
/*                                                                           */
/* purpose: find problem in problem database according to flag               */
/*                                                                           */
/*****************************************************************************/

DSCDB *dscp_find( flg, arg )
int flg;
unsigned long arg;
{
   DSCDB *pp;      /* pointer to problem database record                     */
   int   i;        /* index of found problem                                 */

#ifdef  DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : flg %d\n",flg);
fprintf(stderr,"dsc_serv : dscp_find()      : arg %ld\n",arg);
#endif

   switch( flg ) {
      /*             */
      /* find by PID */
      /*             */
      case 0:

#ifdef DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : find by PID\n");
#endif

           for (i=pcnt; i>=0; i--) {
               if (prob_db[i].index == (EOF)) continue;
               if (prob_db[i].prob_PID == (long) arg) break;
               }             /* end for all elements in databse */ 
           break;
      /*                      */
      /* find by problem name */
      /*                      */
      case 1:

#ifdef DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : find by problem name\n");
#endif

           for (i=pcnt; i>=0; i--) {
               if (prob_db[i].index == (EOF)) continue;
               if (strcmp( prob_db[i].prob_name, (char *)arg) ==0) break;
               }             /* end for all elements in databse */ 
           break;
      /*                          */
      /* find next with incriment */
      /*                          */
      case 2:

#ifdef DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : find next with incriment\n");
#endif

           i = *((int *)arg);
           if (i >= 0) {
              for (; i<pcnt; i++) {
                  if (prob_db[i].index != (EOF)) break;
                  }             /* end for all elements in databse */ 
              if (i >= pcnt) i = (EOF);
              *((int *)arg) = (i+1);
	    }
           break;
      /*                  */
      /* find next unused */
      /*                  */
      case 3:

#ifdef DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : find next unused\n");
#endif

           for (i=0; i<DSC_PROBX; i++) {
               if (prob_db[i].index == (EOF)) break;;
               }             /* end for all elements in databse */ 
           if (i>=DSC_PROBX) i = (EOF);
           else {
             prob_db[i].index = i;
             if (i>=pcnt) pcnt++;
	     }
           break;
      /*                */
      /* find by offset */
      /*                */
      case 4:

#ifdef DSPFIND
fprintf(stderr,"dsc_serv : dscp_find()      : find by offset\n");
#endif

	   i = arg;
	   break;
      default:
fprintf(stderr,"dsc_serv : dscp_find()      : invalid flag for dscp_find()\n");
fflush( stderr );
      }              /* end of switch */

   if (i>=0)
     {
      pp = &prob_db[i];

#ifdef DSPFIND
if ( flg != 3)
{
fprintf(stderr,"dsc_serv : dscp_find()      : returning record for %s\n",
              pp->prob_name);      
fprintf(stderr,"dsc_serv : dscp_find()      : source file name %s\n",
              pp->src_name);
}
#endif

     }
   else 
     {
      pp = (DSCDB *) EOF;

#ifdef DSPFIND      
fprintf(stderr,"dsc_serv : dscp_find()      : returning (DSCDB *) EOF\n");
#endif

     }

   return( pp );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_sleep( msg )                                             */
/*                                                                           */
/* inputs:   msg - message to log problem as sleeping                        */
/*                                                                           */
/* outputs:  (possible data file to remote node)                             */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  if message is from local node                                   */
/*            mark problem as sleeping                                       */
/*            if parent is local decrease number of awk subproblems          */
/*           if message is from remote node                                  */
/*            mark problem as sleeping                                       */
/*            if parent is local decrease number of awk subproblems          */
/*           if parent node is not local                                     */
/*            send possible data file                                        */
/*            send subproblem is sleeping message to parent                  */
/*                                                                           */
/*****************************************************************************/

int dscd_sleep( msg )
DSCSLP *msg;
{
   int rstat=DSC_SUCCESS;
   int rval = DSC_SUCCESS;
   char *cptr;
   char cpyname[DSC_FILEZ+8];  /* name of opt data file to be cpy to rmt node*/
   DSCDB *pp;                  /* pointer to problem in database             */
   DSCDB *pa;                  /* pointer to parent of problem in database   */
   DSCSLP msg2;
   NODE_REC *np;               /* pointer to a node record in the node dbase */
   NODE_REC *dscn_find();      /* find a suitable node from the node dbase   */

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : msg\n");
fflush(stderr);
#endif

  /*                                             */
  /* Check if the message is from the local node */
  /*                                             */
  if ( (msg->fnode) == my_addr )
   {  

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : local node request\n");
#endif

    pp = dscp_find( 0, ntohl(msg->PID));

    if (pp == (DSCDB *)(EOF)) {
    fprintf(stderr,"dsc_serv : dscd_sleep()     : unknown PID %d \n"
                  ,ntohl(msg->PID));
    rstat = DSC_FAILURE;
    pp == NULL;
    goto XDONE;
   }
#ifdef DSDSLEEP
  else
   {
fprintf(stderr,"dsc_serv : dscd_sleep()     : problem found\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : record %s\n",
              pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_sleep()     : source file name %s\n",
              pp->src_name);
fflush(stderr);
   }
#endif
    

   pp->state = 1; /* 0 awake ; 1 sleeping */
   pa = dscp_find( 0, pp->parent_PID);

   if ( pa != (DSCDB *)(EOF) )
    {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : parent found\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : record %s\n",
              pa->prob_name);
fprintf(stderr,"dsc_serv : dscd_sleep()     : source file name %s\n",
              pa->src_name);
fflush(stderr);
#endif

      pa->awk_sub--;

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : # of awake subproblems for parent %d\n",pa->awk_sub);
#endif

      sub_check(pa);
    } 
#ifdef DSDSLEEP
  else 
   {
fprintf(stderr,"dsc_serv : dscd_sleep()     : parent node not found here!\n");
   }
#endif

  }
  /*                                   */
  /* The message is from a remote node */
  /*                                   */
  else
   {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : remote node request\n");
#endif

   pp = dscp_find(1 , msg->prob_name);

   if ( pp == (DSCDB *)(EOF)) {
    fprintf(stderr,"dsc_serv : dscd_sleep()     : unknown problem name %s\n"
                  ,msg->prob_name);
    rstat = DSC_FAILURE;
    goto XDONE;
   }
#ifdef DSDSLEEP
  else
   {
fprintf(stderr,"dsc_serv : dscd_sleep()     : problem found\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : record %s\n",
              pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_sleep()     : source file name %s\n",
              pp->src_name);
fflush(stderr);
   }
#endif

   pp->state = 1; /* 0 awake ; 1 sleeping */
   pa = dscp_find( 0, pp->parent_PID);

   if ( pa != (DSCDB *)(EOF) ) 
    {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : parent found\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : record %s\n",
              pa->prob_name);
fprintf(stderr,"dsc_serv : dscd_sleep()     : source file name %s\n",
              pa->src_name);
fflush(stderr);
#endif

     pa->awk_sub--;

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : # of awake subproblems for parent %d\n",pa->awk_sub);
#endif

     sub_check(pa);
    } 
#ifdef DSDSLEEP
  else
   {
fprintf(stderr,"dsc_serv : dscd_sleep()     : parent node not found here!\n");
   }
#endif

  } /* End from node != my_addr */

 /*                                                     */
 /* If the parent node is not local then copy data file */
 /* and notify the parent of its condition              */
 /*                                                     */

 if  ( ( (msg->fnode) == my_addr ) && ( (pp->pnode_addr) != my_addr ) )
  {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : parent node is not local,\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : notifing parent of condition\n");
fflush(stderr);
#endif

   np = dscn_find(0 , pp->pnode_addr);

   /*                               */
   /* Copy data file to remote node */
   /*                               */
   cptr = strrchr( pp->src_name,'/')+1;
   if (cptr == (char *)1) cptr = pp->src_name;
   sprintf(cpyname,"%s.d",cptr);

   if ( msg->copyflag == 1 )
      {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : copying %s to %s\n"
               ,cpyname,np->nname);
#endif

        rstat = dsc_cpyreq(1,cpyname,*np);
      }
     else
      {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : no copy requested\n");
#endif

        rstat = DSC_SUCCESS;

      }

   if ( rstat != DSC_SUCCESS ) {

    cpytry++;

 /*                                                                          */
 /* due to a potential bottle neck that occurs if both the remote node       */ 
 /* and the local node want to send files to each other the message recieved */
 /* will be sent to the local server again as if it can from the process     */
 /* itself.                                                                  */
 /*                                                                          */

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : dsc_cpyreq failed!\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : resending sleep message\n");
fprintf(stderr,"dsc_serv : dscd_sleep()     : cpytry %d\n",cpytry); 
#endif

    msg2.msg_type = msg2.sanity = htonl(MSG_SLEEPING);
    strcpy(msg2.dsc_password, password );
    msg2.msg_size = htonl(sizeof(DSCSLP));
    msg2.PID = msg->PID;
    msg2.fnode = my_addr;
    strcpy(msg2.prob_name,pp->prob_name);
    
    rval = dsc_dg_send(&msg2,sizeof(DSCSLP), sockdg,portnm,my_addr);

    if ( rval != DSC_SUCCESS ) {

    sprintf(logstr,"dsc_serv : dscd_sleep()     : can not send %s to %s\n"
                  ,cpyname,np->nname);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

sprintf(logstr,"dsc_serv : dscd_sleep()     : error resending sleep message\n");
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

    sprintf(logstr,"dsc_serv : dscd_sleep()     : problem %s forced halt!\n",
                   pp->prob_name);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

    sprintf(logstr,"dsc_serv : dscd_sleep()     : source file %s\n",
                   pp->src_name);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);
    fflush(stderr);

    pa = dscp_find(0,pp->parent_PID);

    dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL);
    cpytry = 0;
    goto XDONE;

    }

   } /* End if rstat != DSC_SUCCESS */

   if ( cpytry >= MAX_RETRY ) {

    sprintf(logstr,"dsc_serv : dscd_sleep()     : can not send %s to %s\n"
                  ,cpyname,np->nname);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

    sprintf(logstr,"dsc_serv : dscd_sleep()     : MAX_RETRY %d exceeded\n",
                   MAX_RETRY);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

    sprintf(logstr,"dsc_serv : dscd_sleep()     : problem %s forced halt!\n",
                   pp->prob_name);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);

    sprintf(logstr,"dsc_serv : dscd_sleep()     : source file %s\n",
                   pp->src_name);
    dsc_log(pp,logstr);
    fprintf(stderr,logstr);
    fflush(stderr);

    pa = dscp_find(0,pp->parent_PID);

    dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL); 
    cpytry = 0;
    goto XDONE;
   }

   if ( rstat == DSC_SUCCESS )
   {

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : copy succeded!\n");
fflush(stderr);
#endif

   cpytry = 0;
   msg2.msg_type = msg2.sanity = htonl(MSG_SLEEPING);
   strcpy(msg2.dsc_password, password );
   msg2.msg_size = htonl(sizeof(DSCSLP)); 
   msg2.PID  = msg->PID;  /* msg->PID is already in network order */
   msg2.fnode = my_addr;
   strcpy(msg2.prob_name,pp->prob_name);

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : sending notification to prnt\n");
fflush(stderr);
#endif

   if (np!=(NODE_REC *)(EOF)) {
     if (np->act_prob != (-1))      /* only if active */
       dsc_dg_send( &msg2, sizeof(DSCSLP), sockdg, np->portg, np->naddr);
   } /* End of node found */
   else
    {
fprintf(stderr,"dsc_serv : dscd_sleep()     : %lu.%lu.%lu.%lu node not found.\n"
             ,(pp->pnode_addr&0xff000000) >> 24,
             (pp->pnode_addr&0x00ff0000) >> 16,
             (pp->pnode_addr&0x0000ff00) >>  8,
             pp->pnode_addr&0x000000ff);
     rstat = DSC_FAILURE;
     np == NULL;
     pa = dscp_find(0,pp->parent_PID);
     dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL); 
     goto XDONE;
    } 
   } /* End if rstat == DSC_SUCCESS */
 }

XDONE:
fflush(stdout);
fflush(stderr);

#ifdef DSDSLEEP
fprintf(stderr,"dsc_serv : dscd_sleep()     : returning rstat %d\n",rstat);
fflush(stderr);
#endif

return(rstat);

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_awaken( msg , sock )                                     */
/*                                                                           */
/* inputs:   msg - message to awaken a problem                               */
/*           sock - socket that will recieve the index                       */
/*                                                                           */
/* outputs:  (possible data file to remote node)                             */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  if the message is from the local node                           */ 
/*            if problem is ready to be awakened                             */
/*             if parent is found increase the number of awake subproblems   */
/*             if executing node is not local                                */
/*               copy input file if necessary                                */
/*               send awake message to remote node                           */
/*             if executing node is local                                    */
/*               copy input file locally if necessary                        */
/*               awaken local problem                                        */
/*               return index to parent                                      */
/*            if problem not asleep error handle                             */
/*           if message is from a remote node                                */
/*            if problem is ready to be awakened                             */
/*             awaken local problem                                          */
/*                                                                           */
/*****************************************************************************/

int dscd_awaken( msg , sock )
DSCAWK *msg;
int sock;
{
   int rstat=DSC_SUCCESS;
   char cpyname[DSC_FILEZ+8]; /* name of input file to be copied             */
   char *cptr;
   DSCDB *pp;                 /* pointer to problem in database              */
   DSCDB *pa;                 /* pointer to parent of problem in database    */
   DSCAWK msg2;
   NODE_REC *np;              /* pointer to a node record in the node dbase  */
   NODE_REC *dscn_find();     /* find a suitable node from the node dbase    */
   void dscp_begin();         /* send return value to calling problem        */
   int dscp_compl();          /* mark (sub)problem as completed              */

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : msg \n");
fprintf(stderr,"dsc_serv : dscd_awaken()    : sock %d\n",sock);
fflush(stderr);
#endif

   /*                                             */
   /* check if the message is from the local node */
   /*                                             */
   if ( (msg->fnode) == my_addr )
    {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : local node request\n");
#endif

      pp = dscp_find(4,ntohl(msg->index));

      if ( pp == (DSCDB *)(EOF)) {
       sprintf(logstr,"dsc_serv : dscd_awaken()    : unknown index %d \n",
                     ntohl(msg->index));
       dsc_log( dscp_find(0,ntohl(msg->PID)), logstr);
       fprintf(stderr,logstr);
       rstat = DSC_FAILURE;
       pp == NULL;
       goto XDONE;
      }
      /*                                                            */
      /* check if the problem has an unsuccessful completion status */
      /*                                                            */
      if  ( pp->compl_stat != DSC_SUCCESS ) 
       {
         sprintf(logstr,"dsc_serv : dscd_awaken()    : %s terminated with\n",
                       pp->prob_name);
         dsc_log( pp , logstr);
         fprintf(stderr,logstr);

         sprintf(logstr,"dsc_serv : dscd_awaken()    : error %d\n",
                        ntohl(pp->compl_stat));
         dsc_log( pp , logstr);
         fprintf(stderr,logstr);

         sprintf(logstr,"dsc_serv : dscd_awaken()    : source file name %s\n",
                       pp->src_name);
         dsc_log( pp , logstr);
         fprintf(stderr,logstr);

       /*                                                          */
       /* return an error index to the probelm issuing the awaken. */
       /*                                                          */
       pp->state = 0;
       pa = dscp_find(0,pp->parent_PID);
       if ( pa != (DSCDB *)(EOF)) pa->awk_sub++;
       dscp_begin( sock, -1);
       goto XDONE;
     }

     /*                                              */
     /* check if the problem is ready to be awakened */
     /*                                              */
    if ( (pp->state == 1) && ( pp->compl_stat == DSC_SUCCESS ) )
     {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : problem is ready to be awakened\n");
#endif

       pp->state = 0;
       pa = dscp_find(0,pp->parent_PID);

       if ( pa != (DSCDB *)(EOF)) 
        { 

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : parent found\n");
fprintf(stderr,"dsc_serv : dscd_awaken()    : record %s\n",
              pa->prob_name);
fprintf(stderr,"dsc_serv : dscd_awaken()    : source file name %s\n",
              pa->src_name);
fflush(stderr);
#endif

         pa->awk_sub++;

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : # of awake subproblems for parent %d\n",pa->awk_sub);
#endif

        }
#ifdef DSDAWAKEN
       else
        {
fprintf(stderr,"dsc_serv : dscd_awaken()    : parent node not found here!\n");
        }
#endif

       /* If the executing node is not local then the input files should be   */
       /* copied to the executing node and an awaken datagram message must be */
       /* sent to the executing node informing it that the subproblem must be */
       /* awakened.                                                           */

    /*                                          */
    /* Check if the executing node is not local */
    /*                                          */
    if ( pp->enode_addr != my_addr)
     {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : executing node is not local\n");
#endif

    np = dscn_find(0,pp->enode_addr);

    /*                                */
    /* Copy input file to remote node */
    /*                                */
    cptr = strrchr( pp->src_name,'/')+1;
    if (cptr ==(char *)1) cptr = pp->src_name;
    sprintf(cpyname,"%s.i",cptr);

    if ( strcmp(msg->filename,"NULL") == 0 ) 
      {
#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : no copy requested\n");
#endif
        rstat = DSC_SUCCESS;
      }
    else
    if ( strcmp(msg->filename,"") == 0 ) 
      {
#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : copying %s to %s\n"
               ,cpyname,np->nname);
#endif

        rstat = dsc_cpyreq(1,cpyname,*np);
      }
    else
    if ( strcmp(msg->filename,"") != 0 )
      {
        strcpy(cpyname,msg->filename);
#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : copying %s to %s\n"
               ,cpyname,np->nname);
#endif
        rstat = dsc_cpyreq(1,cpyname,*np);
      }

    if (rstat != DSC_SUCCESS) {


     sprintf(logstr,"dsc_serv : dscd_awaken()    : can not send %s to %s\n"
                   ,cpyname,np->nname); 
     dsc_log(pp,logstr);
     fprintf(stderr,logstr);

     dscp_begin(sock,-1);
     pa = dscp_find(0,pp->parent_PID);
     dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL); 
     goto XDONE;
    }

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : sending awake request to remote node\n");
#endif

    msg2.msg_type = msg2.sanity = htonl(MSG_AWAKEN);
    strcpy(msg2.dsc_password, password );
    msg2.msg_size= htonl(sizeof(DSCAWK));
    msg2.index = msg->index;
    msg2.PID = msg->PID;
    msg2.fnode = my_addr;
    strcpy(msg2.prob_name,pp->prob_name);
    if ( np != (NODE_REC *)(EOF)) {
     if (np->act_prob != (-1))
       dsc_dg_send(&msg2,sizeof(DSCAWK), sockdg,np->portg,np->naddr);
    } /* End of node found */
    else
     {
fprintf(stderr,"dsc_serv : dscd_awaken()    : %lu.%lu.%lu.%lu node not found.\n"
             ,(pp->pnode_addr&0xff000000) >> 24,
             (pp->pnode_addr&0x00ff0000) >> 16,
             (pp->pnode_addr&0x0000ff00) >>  8,
             pp->pnode_addr&0x000000ff);
      rstat = DSC_FAILURE;
      np = NULL;
      pa = dscp_find(0,pp->parent_PID);
      dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL); 
      dscp_begin(sock,-1);
      goto XDONE;
     }    

    dscp_begin(sock, ntohl(msg->index));
    goto XDONE;
   } /* End if execute node != my_addr */  

  /*                                      */
  /* Check if the executing node is local */
  /*                                      */
  if ( pp->enode_addr  == my_addr )
   {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : executing node is local\n");
#endif


#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : local problem awakening.\n");
#endif

    rstat = kill(pp->prob_PID, SIGUSR1 );
     if ( rstat < 0 ) {

sprintf(logstr,"dsc_serv : dscd_awaken()    : can not awaken subproblem %s .\n",
               pp->prob_name);
      dsc_log(pp,logstr);
      fprintf(stderr,logstr);

sprintf(logstr,"dsc_serv : dscd_awaken()    : source file name %s\n",
                pp->src_name);
      dsc_log(pp,logstr);
      fprintf(stderr,logstr);

      pa = dscp_find(0,ntohl(pp->parent_PID));
      dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name, DSCERR_P_GENERAL);
      dscp_begin(sock,-1);
      goto XDONE;
     } /* End if signal error */

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : returning index %d to parent\n",
               ntohl(msg->index));
#endif

   dscp_begin(sock, ntohl(msg->index));

   goto XDONE;
   } /* End of parent node == my_addr */

 } /* End if state = 1 and status = DSC_SUCCESS */

   /*                                     */
   /* Check if the problem is not assleep */
   /*                                     */
   if ( ( pp->state == 0 ) && ( pp->compl_stat == DSC_SUCCESS ) )  
    {

sprintf(logstr,"dsc_serv : dscd_awaken()    : awaken issued to an awake problem %s \n", pp->prob_name);
      dsc_log(pp,logstr);
      fprintf(stderr,logstr);

 sprintf(logstr,"dsc_serv : dscd_awaken()    : source file name %s\n",
                       pp->src_name);
       dsc_log( pp , logstr);
       fprintf(stderr,logstr);

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : returning index -1 to parent\n");
#endif

       dscp_begin( sock, -1);

       pa = dscp_find(0,pp->parent_PID);
       dscp_compl(pp,pa,pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL); 
       goto XDONE; 
    } /* End if still awake but no errors */

   } /* End if from node == my_addr */

   /*                                   */
   /* The message is from a remote node */
   /*                                   */

   else
    {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : recieved awaken from rnode \n");
fflush(stderr);
#endif

     pp = dscp_find (1, msg->prob_name);
     if ( pp == (DSCDB *)(EOF)) {

     fprintf(stderr,"dsc_serv : dscd_awaken()    : unknown problem name %s \n"
                   ,msg->prob_name);
     fflush(stderr);

      rstat = DSC_FAILURE;
      goto XDONE;
     }

     /*                            */
     /* Check for sleeping problem */
     /*                            */
     if ( ( pp->state == 1 ) && ( pp->compl_stat == 0 ) )
      {

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : problem is ready to be awakened\n"
);
#endif

       rstat = kill(pp->prob_PID, SIGUSR1);
       if ( rstat < 0 ) {

sprintf(logstr,"dsc_serv : dscd_awaken()    : can not awaken subproblem %s .\n",
               pp->prob_name);
      dsc_log(pp,logstr);
      fprintf(stderr,logstr);

sprintf(logstr,"dsc_serv : dscd_awaken()    : source file name %s\n",
                pp->src_name);
      dsc_log(pp,logstr);
      fprintf(stderr,logstr);

      pa = dscp_find(0,pp->parent_PID);
      dscp_compl( pp , pa, pp->pnode_addr,pp->prob_name,DSCERR_P_GENERAL);
      goto XDONE;

      } /* End if signal error */

      pp->state = 0;
    }
   else
fprintf(stderr,"dsc_serv : dscd_awaken()    : invalid awaken mess recieved\n");
   
  } /* End from node != my_addr */
 
XDONE:

   close(sock);
   fflush(stdout);
   fflush(stderr);

#ifdef DSDAWAKEN
fprintf(stderr,"dsc_serv : dscd_awaken()    : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return(rstat);

} /* End of dscd_awaken */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_start( msg, sock )                                       */
/*                                                                           */
/* inputs:   msg - pointer to messages which initiated the request           */
/*           sock - socket that message was received on                      */
/*                                                                           */
/* outputs:  (either send a file or receive a file)                          */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  check for duplicate problem name                                */
/*           build database record                                           */
/*           choose a node to run the problem on                             */
/*           send files to remote computer and issue start message           */
/*           if running locally, fork a build                                */
/*           return problem index to requesting program                      */
/*            (this index may be used later to kill or wait on completion)   */
/*                                                                           */
/*****************************************************************************/

int dscd_start( msg, sock )
DSCSTRT *msg;
int     sock;
{
   int rstat;                          /* return status of dscd_start        */
   NODE_REC *np;                       /* (sub) problem tartget node         */
   DSCDB    *pp;                       /* pointer to problem                 */
   DSCDB    *pa;                       /* pointer to parent problem          */
   struct tm *tmp;                     /* structure returned by  localtime   */
   time_t tt;                          /* value = secs since epoch           */
   void     process_signal();
   NODE_REC *dscn_find();
   void     dscp_begin();

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : DSCSTRT *msg, int sock\n");
fprintf(stderr,"dsc_serv : dscd_start       : msg_type %ld msg_size %ld \n",
        ntohl(msg->msg_type),ntohl(msg->msg_size));
#endif

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : looking for parent by PID %d\n",
        ntohl(msg->parent_PID));
#endif

   pa = dscp_find(0, ntohl(msg->parent_PID));

#ifdef DSDSTART
if ( pa == (DSCDB *) EOF )
fprintf(stderr,"dsc_serv : dscd_start       : parent PID %d not found\n",
        ntohl(msg->parent_PID));
else
{
fprintf(stderr,"dsc_serv : dscd_start       : parent problem %s found \n",
        pa->prob_name);
fprintf(stderr,"dsc_serv : dscd_start       : source file name %s\n",
        pa->src_name);
}
#endif

   /*                                      */
   /*   CHECK FOR DUPLICATE PROBLEM NAME   */
   /*                                      */
   pp = dscp_find( 1, msg->prob_name);

   if (pp != (DSCDB *)(EOF)) {
fprintf(stderr,"dsc_serv : dscd_start       : duplicate problem name: %s\n",
        msg->prob_name);
fprintf(stderr,"dsc_serv : dscd_start       : discarded.\n"); 
      dscp_compl( EOF, pa, msg->fnode, msg->prob_name, DSCERR_P_DUPNAME);
      rstat = DSCERR_P_DUPNAME;
      pp = NULL;
      goto XDONE;
      }
#ifdef DSDSTART
else
fprintf(stderr,"dsc_serv : dscd_start       : unique problem name\n");
#endif

   /*                                        */
   /*   CREATE PROBLEM DATABASE ENTRY        */
   /*                                        */
   pp = dscp_find( 3, 0);

   if (pp == (DSCDB *)(EOF)) {
fprintf(stderr,"dsc_serv : dscd_start       : too many active problems\n");
fprintf(stderr,"dsc_serv : dscd_start       : can not add %s\n", 
         msg->prob_name);
      dscp_compl( EOF, pa, msg->fnode, msg->prob_name,
		 DSCERR_P_DBFULL);
      rstat = DSC_FAILURE;
      pp = (DSCDB *)EOF;
      goto XDONE;
      }               /* end if new entry cannot be built */
#ifdef DSDSTART
else
fprintf(stderr,"dsc_serv : dscd_start       : created problem database entry\n");
#endif

   /*                       */
   /* build database record */
   /*                       */
   pp->depth = ntohl(msg->subflg);
   strcpy( pp->prob_name, msg->prob_name);
   pp->parent_PID = ntohl(msg->parent_PID);
   pp->parent_idx = (pa==(DSCDB *)(EOF))?(EOF):(pa->index);
   pp->pnode_addr = msg->fnode;  /* leave addr in network order */
   pp->ctyp = ntohl(msg->ctyp);
   pp->Ctyp = ntohl(msg->Ctyp);
   pp->ltyp = ntohl(msg->ltyp);
   pp->mtyp = ntohl(msg->mtyp);
   pp->ptyp = ntohl(msg->ptyp);
   pp->otyp = ntohl(msg->otyp);
   pp->ityp = ntohl(msg->ityp);
   pp->dtyp = ntohl(msg->dtyp);
   pp->cpu_type = ntohl(msg->cpu_type);
   strcpy( pp->src_name, msg->src_name);
   strcpy( pp->src_build, msg->src_build);
   strcpy( pp->src_exec, msg->src_exec);
   pp->block_sok = EOF;
   pp->block_sub = (-1);
   pp->build_flg = FALSE;
   pp->copy_flg = FALSE;
   pp->compl_flg = FALSE;
   pp->compl_stat = DSC_SUCCESS;
   pp->peer_next = (EOF);
   pp->child_next = (EOF);
   pp->num_sub = 0;
   pp->awk_sub = 0;
   pp->state = 0;                /* 0 awake ; 1 sleep */
   pp->cpurating = ntohl(msg->cpurating);
   pp->memrating = ntohl(msg->memrating);

   sprintf( logstr, "%s : database entry %d created\n", 
       pp->prob_name, pp->index);
   dsc_log( pp, logstr);

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : %s\n",logstr);
#endif

   /*                                          */
   /* Determine curent time for the Time stamp */
   /*                                          */
   time(&tt);
   tmp = localtime(&tt);
   sprintf( logstr, "%s : %s : start time %s\n",pp->prob_name,pp->src_name,
                  asctime(tmp));
   dsc_log( pp, logstr);

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : %s\n",logstr);
#endif

   /*                                           */
   /*   Link this subproblem into parents list  */
   /*                                           */
   if (pa != (DSCDB *)(EOF)) {
     long nxt;

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : linking problem to parent list\n"
);
#endif

     pa->num_sub++;
     pa->awk_sub++;
     nxt = pa->child_next;
     if (nxt != (EOF)) {
        while (prob_db[nxt].peer_next != (EOF))/* find end of list            */
          nxt = prob_db[nxt].peer_next;
        if (nxt != (EOF))
          prob_db[nxt].peer_next = pp->index;  /* link this (sub) problem     */
        }
     else                                      /*first child for this problem */
       pa->child_next = pp->index;
     }		                               /* if we found the parent      */
#ifdef DSDSTART
else fprintf(stderr,"dsc_serv : dscd_start       : no link necessary\n");
#endif

   /*                                              */
   /* debugging hook to allow interactive problems */
   /*                                              */
   if (msg->sanity != msg->msg_type) {

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : debugging hook\n");
fprintf(stderr,"dsc_serv : dscd_start       : allowing interactive problems\n");
#endif

     pp->build_flg = TRUE;
     pp->prob_PID = ntohl(msg->sanity);
     goto XDONE;
     }

   /*                           */
   /* if parent is local        */
   /* and this not a subproblem */
   /*  PICK A NODE              */
   /*                           */

   /*                      */
   /* ps deamon scheduling */
   /*                      */
   if ( (msg->fnode == my_addr) && 
        (ntohl(msg->subflg) != 0)  &&
        ( pssched == TRUE ) 
      )
      {

#ifdef DSDSTART
fprintf(stderr,"dsc_serv : dscd_start       : searching for a node\n");
fprintf(stderr,"dsc_serv : dscd_start       : using dynamic scheduling\n");
fflush( stderr );
#endif

       /*                                          */
       /*    Must take file type into account!     */
       /*                                          */
       if ( msg->ctyp == 1 ) 
         np =  dscn_find(4 , pp->cpurating, pp->memrating);
       if ( msg->Ctyp == 1 ) 
         np =  dscn_find(4 , pp->cpurating, pp->memrating);
       else if (msg->ltyp == 1 )
         np =  dscn_find(4 , pp->cpurating, pp->memrating);
       else if (msg->mtyp == 1 )
         np =  dscn_find(4 , pp->cpurating, pp->memrating);
       else if (msg->ptyp == 1 )
         np =  dscn_find(4 , pp->cpurating, pp->memrating);
       else
        { 
fprintf(stderr,"dsc_serv : dscd_start       : file type unknown, C assumed\n");
             np =  dscn_find(4 , pp->cpurating, pp->memrating);
        }

#ifdef DSDSTART
   if ( np == (NODE_REC *) (DSCQUEUED) )
fprintf(stderr,"dsc_serv : dscd_start       : could not find suitable node\n");
   else
   if ( np == (NODE_REC *) (EOF) )
fprintf(stderr,"dsc_serv : dscd_start       : running problem localy\n");
   else
fprintf(stderr,"dsc_serv : dscd_start       : distributing to %s \n",np->nname);
fflush( stderr );
#endif

       if ( np == (NODE_REC *) (DSCQUEUED) )
        {
        pp->prob_PID = 0;
        waiting_on_sub++;

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       :  no room at the inn \n");
fprintf( stderr, "dsc_serv : dscd_start       :  %s is waiting\n",
        pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_start       : source file name %s\n",
        pp->src_name);
fflush( stderr );
#endif

         goto XDONE;
        }
      }
   /*                           */
   /* num of problem scheduling */
   /*                           */
   else
   {
   if ((msg->fnode == my_addr) && (ntohl(msg->subflg) != 0)) {

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : searching for a node using round robin\n");
fflush( stderr );
#endif

     if ( msg->ctyp == 1 ) np = dscn_find(1, act_prob, 0); 
     else 
     if ( msg->Ctyp == 1 ) np = dscn_find(1, act_prob, 0);  /* 0=C=C++ (AAP)*/
     else 
     if ( msg->ltyp == 1 ) np = dscn_find(1, act_prob, 1); 
     else
     if ( msg->mtyp == 1 ) np = dscn_find(1, act_prob, 2); 
     else
     if ( msg->ptyp == 1 ) np = dscn_find(1, act_prob, 3);
     else 
      { fprintf( stderr, "Can not distinguish file type, C assumed\n");
        np = dscn_find(1, act_prob, 0);
      }

#ifdef DSDSTART
   if ( np == (NODE_REC *) (DSCQUEUED) ) 
fprintf(stderr,"dsc_serv : dscd_start       : error in sched algorithm!\n");
   else
   if ( np == (NODE_REC *) (EOF) )
fprintf(stderr,"dsc_serv : dscd_start       : running problem localy\n");
   else
fprintf(stderr,"dsc_serv : dscd_start       : distributing to %s \n",np->nname);
fflush( stderr );
#endif

       if ( np == (NODE_REC *) (DSCQUEUED) )
        {
        pp->prob_PID = 0;
        waiting_on_sub++;

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : RR no room at the inn \n");
fprintf( stderr, "dsc_serv : dscd_start       : RR %s is waiting\n",
         pp->prob_name);
fflush( stderr );
#endif

         goto XDONE;
        }
      }                   /* end if source is local main problem */
   else np = (NODE_REC *)EOF;
   }


   /*                                             */
   /*    Move soruce/input files to target node   */
   /*                                             */
   if (np != (NODE_REC *) EOF) {

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : moving files to target node\n");
#endif

      sprintf( logstr, "%s: attempting to run (sub)problem at %s\n",
               pp->prob_name,np->nname);
      dsc_log( pp, logstr);

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : %s\n",logstr);
#endif

      rstat = dscp_export( pp, pa, &np, msg);
      if (rstat != DSC_SUCCESS)  np = (NODE_REC *) EOF;
      }

   /*                       */
   /*   If running LOCAL    */
   /*                       */
   if (np == (NODE_REC *) EOF) {
      sprintf( logstr, "%s: running (sub)problem locally\n", pp->prob_name);
      dsc_log( pp, logstr);

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : %s\n",logstr);
#endif

      pp->enode_addr = my_addr;
      if (strlen( msg->src_build )) {

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : building %s %s\n",
         pp->prob_name, msg->src_build);
#endif

         rstat = dscp_build( pp );

         if (rstat != DSC_SUCCESS) {
            sprintf( logstr, "%s: build failed: %s\n",
              pp->prob_name, msg->src_build);
            dsc_log( pp, logstr);

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : %s\n",logstr);
#endif

            dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
		       DSCERR_P_BUILD|rstat);
            goto XDONE;
            }		/* if build failed */
         }			/* if build command exists */
      }                          /* end else run this program locally */
XDONE:
   if (sock != EOF) {
      if (pp != 0)
       {

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : returning %d index to parent\n"
         ,pp->index);
#endif

         dscp_begin( sock, pp->index );
       }
      else 
       {

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : returning %d index to parent\n"
         ,EOF);
#endif

         dscp_begin( sock, EOF);
       }
      close( sock );
      }
   fflush(stdout);
   fflush(stderr);

#ifdef DSDSTART
fprintf( stderr, "dsc_serv : dscd_start       : returning rstat %d\n",rstat);
#endif

   return(rstat);
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_export( pp, pa, np, msg)                                 */
/*                                                                           */
/* inputs:   pp   - pointer to problem                                       */
/*           pa   - pointer to parent problem                                */
/*           np   - pointer to target node                                   */
/*           msg  - start message                                            */
/*                                                                           */
/* outputs:  (files copied to remote computer)                               */
/*           (start message sent to remote computer)                         */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  check for each source file type                                 */
/*           send each source file to target node                            */
/*           send start message to target node                               */
/*                                                                           */
/*****************************************************************************/

int dscp_export( pp, pa, np, msg )
DSCDB *pp;
DSCDB *pa;
NODE_REC **np;
DSCSTRT *msg;
{
   struct stat fs;
   int rstat;
   char build_name[DSC_FILEZ+8];

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : DSCDB *pp\n");
fprintf( stderr, "dsc_serv : dscp_export      : problem name %s\n",
                 pp->prob_name);
fprintf( stderr, "dsc_serv : dscp_export      : source file name %s\n",
                 pp->src_name);
fprintf( stderr, "dsc_serv : dscp_export      : DSCDB *pa\n");
fprintf( stderr, "dsc_serv : dscp_export      : parent problem name %s\n",
                 pa->prob_name);
fprintf( stderr, "dsc_serv : dscp_export      : prnt source file name %s\n",
                 pa->src_name);
fprintf( stderr, "dsc_serv : dscp_export      : NODE_REC **np\n");
fprintf( stderr, "dsc_serv : dscp_export      : node name %s\n",
                 (*np)->nname);
fprintf( stderr, "dsc_serv : dscp_export      : DSCSTRT *msg\n");
#endif

   /*                  */
   /* check for C file */
   /*                  */

   if (pp->ctyp) {

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : C file\n");
#endif
 
      sprintf(build_name, "%s.c", pp->src_name);

      /*                       */
      /* check for dummy build */
      /*                       */
      if ( strcmp(msg->src_build,DSC_DB_C) != 0 )
        {

         rstat = stat( build_name, &fs);

      if  ( rstat == (EOF) )
        {
sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n",pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
		    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

sprintf( logstr, "dsc_serv : dscp_export      : %s: copying c file %s to %s\n",
                pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif
         
         rstat = dsc_cpyreq( 1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS) 
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF; 
          }
         }
     } /* End of check for Dummy build if */

    } /* End of if pp->ctype */

   /*                           */
   /* Check for C++ file. (AAP) */
   /*                           */

   if (pp->Ctyp) {

#ifdef DSPEXPORT
fprintf (stderr, "dsc_serv : dscp_export      : C++ file\n");
#endif
 
      sprintf (build_name, "%s.C", pp->src_name);

      /*                       */
      /* check for dummy build */
      /*                       */
      if (strcmp (msg->src_build, DSC_DB_CPP) != 0)
        {

         rstat = stat (build_name, &fs);

      if  ( rstat == (EOF) )
        {
sprintf (logstr, 
         "dsc_serv : dscp_export      : %s: source file does not exist: %s\n",
         pp->prob_name, build_name);

         dsc_log (pp, logstr);
         fprintf (stderr, logstr);

         dscp_compl (pp, pa, pp->pnode_addr, pp->prob_name, DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

sprintf (logstr, 
         "dsc_serv : dscp_export      : %s: copying C++ file %s to %s\n",
         pp->prob_name, build_name, (*np)->nname);

         dsc_log (pp, logstr);

#ifdef DSPEXPORT
         fprintf (stderr, logstr);
#endif
         rstat = dsc_cpyreq (1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS) 
          {
sprintf (logstr, "dsc_serv : dscp_export      : resorting to local run\n");

           dsc_log (pp, logstr);
           fprintf (stderr, logstr);
           *np=(NODE_REC *)EOF; 
          }
         }
     } /* End of check for Dummy build if */

    } /* End of if pp->Ctype */

   /*                     */
   /* check for lisp file */
   /*                     */

   if (pp->ltyp) {

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : Lisp file\n");
#endif

      sprintf(build_name, "%s.lsp", pp->src_name);
      if (pp->otyp) { sprintf(build_name, "%s.o", pp->src_name); }

      rstat = stat( build_name, &fs);

      if (rstat == (EOF)) {
         sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n", pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
		    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

sprintf( logstr, "dsc_serv : dscp_export      : %s: copying lisp file %s to %s\n",
                pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

         rstat = dsc_cpyreq( 1, build_name, *np);

         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS)
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF;
          }

         }
   }

   /*                      */
   /* check for Maple file */
   /*                      */

   if (pp->ptyp) {

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : Maple file\n");
#endif

      sprintf(build_name, "%s.mpl", pp->src_name);
      rstat = stat( build_name, &fs);

      if (rstat == (EOF)) {
sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n",pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
                    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

         sprintf( logstr, "dsc_serv : dscp_export      : %s: copying maple file %s to %s\n",pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

         rstat = dsc_cpyreq( 1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS)
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF;
          }
         }
   }

   /*                            */
   /* check for Mathematica file */
   /*                            */

   if (pp->mtyp) {

      sprintf(build_name, "%s.m", pp->src_name);
      rstat = stat( build_name, &fs);

      if (rstat == (EOF)) {
sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n", pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
                    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

sprintf( logstr, "dsc_serv : dscp_export      : %s: copying mathematica file %s to %s\n",pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

         rstat = dsc_cpyreq( 1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS)
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF;
          }
         }
   }

   /*                       */
   /* check for object file */
   /*                       */

   if ( (pp->otyp) && (pp->ltyp != 1) ) {

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : non lisp object file\n");
#endif

      sprintf(build_name, "%s.o", pp->src_name);
      rstat = stat( build_name, &fs);

      if (rstat == (EOF)) {
sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n",pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
		    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {
sprintf( logstr, "dsc_serv : dscp_export      : %s: copying .o file %s to %s\n",
            pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);

         rstat = dsc_cpyreq( 1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS)
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF;
          }
         }
   }

   /*                           */
   /* check for input data file */
   /*                           */

   if (pp->ityp) {

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : input file\n");
#endif

      sprintf(build_name, "%s.i", pp->src_name);
      rstat = stat( build_name, &fs);

      if (rstat == (EOF)) {
sprintf( logstr, "dsc_serv : dscp_export      : %s: source file does not exist: %s\n", pp->prob_name, build_name);
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

         dscp_compl( pp, pa, pp->pnode_addr, pp->prob_name,
		    DSCERR_P_NOSOURCE);
         rstat = DSC_FAILURE;
         pp = NULL;
         goto XDONE;
         }                 /* end if file does exist */
      /*                                                   */
      /* if parent is off-node, we have some files to copy */
      /*                                                   */
      else if (*np != (NODE_REC *) EOF) {

sprintf( logstr, "dsc_serv : dscp_export      : %s: copying input file %s to %s\n",pp->prob_name, build_name, (*np)->nname);
         dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

         rstat = dsc_cpyreq( 1, build_name, *np);
         /*                     */
         /* resort to local run */
         /*                     */
         if (rstat != DSC_SUCCESS)
          {
sprintf( logstr, "dsc_serv : dscp_export      : resorting to local run\n");
         dsc_log( pp, logstr);
         fprintf( stderr, logstr);

           *np=(NODE_REC *)EOF;
          }
         }
   }

   /*                                     */
   /*  TELL REMOTE NODE to START PROBLEM  */
   /*                                     */

   if (*np != (NODE_REC *)(EOF)) {

sprintf( logstr, "dsc_serv : dscp_export      : %s: starting problem on remote node %s\n", pp->prob_name, (*np)->nname);
      dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

      pp->enode_addr = (*np)->naddr;
      rstat = dsc_dg_send(msg, sizeof(DSCSTRT), sockdg,
			  (*np)->portg, (*np)->naddr);

      if (rstat == DSC_SUCCESS) {
	(*np)->act_prob++;

        /*                                                   */
        /* when problems are running remotely, keep checking */
        /* node status, so we don't get abandoned.           */
	if (!loop_flg) {
   	   loop_flg = TRUE;
	   loop_pos = 0;
	   } /* start checkin loop */
        } /* end if remote problem started */
      else {
sprintf(logstr,"dsc_serv : dscp_export      : %s: failed to send start message to node %s, run local\n",pp->prob_name, (*np)->nname);
        dsc_log( pp, logstr);
#ifdef DSPEXPORT
         fprintf( stderr, logstr);
#endif

        *np = (NODE_REC *)EOF;
        } /* end else could not send start message */
      }            /* end if solve problem remotely */

XDONE:

#ifdef DSPEXPORT
fprintf( stderr, "dsc_serv : dscp_export      : returning rstat %d\n",rstat);
#endif

   return( rstat );
}


/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_build( po )                                              */
/*                                                                           */
/* inputs:    po  - offset to problem within problem database                */
/*                                                                           */
/* outputs:  (forked subprocess to build problem)                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  if this node has room to start the problem,                     */
/*              fork a subprocess to build it                                */
/*              log state of subprocess                                      */
/*           else                                                            */
/*              mark database entry as queued                                */
/*                                                                           */
/*****************************************************************************/

int dscp_build( pp )
DSCDB *pp;
{
   int rstat = 0;  /* return status of dscp_build                            */

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : DSCDB *pp\n");
fprintf(stderr,"dsc_serv : dscp_build       : build command %s\n",
                pp->src_build);
fprintf(stderr,"dsc_serv : dscp_build       :  PID %d \n",
                pp->prob_PID);
#endif

   /*                                                        */
   /* check to see if there is room to start this subproblem */
   /*                                                        */
   if (act_prob >= DSC_CPU_LIMIT && pssched != TRUE) {
     pp->prob_PID = 0;
     waiting_on_sub++;

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : RR no room at the inn\n");
fprintf(stderr,"dsc_serv : dscp_build       : RR %s is waiting\n",
        pp->prob_name);
fflush( stderr );
#endif

     goto XDONE;
     }
   else if (pp->prob_PID == 0) {
     waiting_on_sub--;

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : pp->prob_PID == 0\n");
#endif

     }

   /*                      */
   /*   FORK SUBPROCESS    */
   /*                      */

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : forking subprocess\n");
#endif

   pp->prob_PID = fork();
   if (pp->prob_PID == EOF) {
      perror( "forking a build: ");
fprintf(stderr,"dsc_serv : dscp_build       : error forking a build\n");
      rstat = DSCERR_P_FORKING;
      goto XDONE;
      }
   /*                   */
   /* if parent process */
   /*                   */
   else if (pp->prob_PID != 0) {

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : parent process act_prob++\n");
fflush(stderr);
#endif

      act_prob++;                      /* count active problems */
      }			/* end-if parent process */
   /*                  */
   /* if child process */
   /*                  */
   else if (pp->prob_PID == 0)
     {
      char temp[DSC_FILEZ];

      /* change standard out and standard error */
      sprintf( temp, "%s.log", pp->src_name);
      freopen( temp, "w", stdout );
      freopen( temp, "a", stderr );

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : child process executing image\n");
fflush(stderr);
#endif

      dsc_exec( pp->src_build );       /* execute new image */
      _exit(errno);
      }

XDONE:

#ifdef DSPBUILD
fprintf(stderr,"dsc_serv : dscp_build       : returning rstat %d\n",rstat);
#endif

   return( rstat );

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_unlink( pc )                                             */
/*                                                                           */
/* inputs:   pc - pointer to child ;problem                                  */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  unlink a child problem from a parent                            */
/*                                                                           */
/*****************************************************************************/

void dscp_unlink( pc )
DSCDB *pc;
{
   DSCDB *pa, *po, *px, *dscp_find();
   int off;

#ifdef DSPUNLINK 
fprintf(stderr,"dsc_serv : dscp_unlink()    : DSCDB *pc\n");
fprintf(stderr,"dsc_serv : dscp_unlink()    : problem name %s\n",
                 pc->prob_name);
fprintf(stderr,"dsc_serv : dscp_unlink()    : source file name %s\n",
                 pc->src_name);
#endif

   /*             */
   /* find parent */
   /*             */
   pa = dscp_find( 4, pc->parent_idx );
   if (pa == (DSCDB *)EOF)
    {

#ifdef DSPUNLINK 
fprintf(stderr,"dsc_serv : dscp_unlink()    : parent not found\n");
#endif

     goto XDONE;
    }

   /*                                         */
   /* adjust linked list to omit this problem */
   /*                                         */

#ifdef DSPUNLINK 
fprintf(stderr,"dsc_serv : dscp_unlink()    : adjusting linked list to\n");
fprintf(stderr,"dsc_serv : dscp_unlink()    : omit this problem\n");
#endif

   po = (DSCDB *)EOF;
   off = pa->child_next;
   while (off != (EOF)) {
       px = dscp_find( 4, off);
       if (px == pc) {             /* if this is completed problem */
	 if (po != (DSCDB *)EOF)
            po->peer_next = px->peer_next;   /* remove from list */
         else
            pa->child_next = px->peer_next;  /* remove from list */
         px->peer_next = (EOF);     /* erase remnants of list */
         off = (EOF); /* no reason to search further */
         }           /* end if this is completed problem */
       else {
         off = px->peer_next;
	 po = px;
         }           /* end else this is not completed problem */
       }             /* end while not end of child list */

XDONE:

#ifdef DSPUNLINK 
fprintf(stderr,"dsc_serv : dscp_unlink()    : returning void\n");
#endif

 return;

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_del( pp )                                                */
/*                                                                           */
/* inputs:   pp - ptr to problem entry to delete from databse                */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  assume the problem has been unlinked from parent                */
/*           "child-list"                                                    */
/*           mark this problem entry in problem database as available.       */
/*           dec. high water mark in database as problems finish.            */
/*                                                                           */
/*****************************************************************************/

int dscp_del( pp )
DSCDB *pp;
{
   int rstat;

#ifdef DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : DSCDB *pp\n");
fprintf(stderr,"dsc_serv : dscp_del()       : deleting index %d\n",pp->index);
fprintf(stderr,"dsc_serv : dscp_del()       : problem %s\n",pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_del()       : source file name %s\n",
              pp->src_name);
#endif

   /*                */
   /* log any errors */
   /*                */
   dsc_err( pp, pp->compl_stat);

   /*                                                    */
   /* if blocked, unblock so it won't hang on the accept */
   /*                                                    */
   if (pp->block_sok != (long)EOF) {

#ifdef DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : problem block, must unblock\n");
#endif

     solvmsg.status = htonl( pp->compl_stat );
     solvmsg.index = htonl( pp->block_sub );
     blk_prob--;

#ifdef DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : writing message to socket\n");
#endif

     rstat = write( pp->block_sok, &solvmsg, sizeof( DSCSOLV ));

#ifdef DSPDEL
errno = 0;
fprintf(stderr,"dsc_serv : dscp_del()       : wrote %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_del()       : expected %d\n",sizeof(DSCSOLV ));
fprintf(stderr,"dsc_serv : dscp_del()       : errno %d\n",errno);
#endif


    /* wait till msg is read and socket is closed */
    /* this read should always complete with socket broken */

#ifdef DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : reading message from socket\n");
#endif

    rstat = read( pp->block_sok, &solvmsg, sizeof(DSCSOLV));

#ifdef  DSPDEL
errno = 0;
fprintf(stderr,"dsc_serv : dscp_del()       : read %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_del()       : expected %d\n",sizeof(DSCSOLV));
fprintf(stderr,"dsc_serv : dscp_del()       : errno %d\n",errno);
#endif

     close( pp->block_sok );
     }

   /*                       */
   /* kill running problems */
   /*                       */

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : kill running problems \n");
#endif

   if ((!pp->compl_flg) && (pp->prob_PID != (EOF))) {
      errno = 0;
      if (pp->enode_addr == my_addr) {

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : executing local \n");
#endif

         rstat = kill( pp->prob_PID, SIGKILL );

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : killing %s\n",pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_del()       : source file %s\n",pp->src_name);
fprintf(stderr,"dsc_serv : dscp_del()       : rstat %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_del()       : errno %d\n",errno);
fprintf(stderr,"dsc_serv : dscp_del()       : PID %d\n", pp->prob_PID);
#endif

       } /* end if executing local */
      else {
        NODE_REC *np, *dscn_find();

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : remote execution\n");
#endif

	np = dscn_find( 0, pp->enode_addr,0);

        if ((np != (NODE_REC *)EOF) && !pp->copy_flg)  {
   	   strcpy( killmsg.prob_name, pp->prob_name);
	   killmsg.from_PID = htonl(-2);
           rstat = dsc_dg_send( &killmsg, sizeof(DSCKILL), sockdg,
		    np->portg, np->naddr);

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : killing %s\n",pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_del()       : source file %s\n",pp->src_name);
fprintf(stderr,"dsc_serv : dscp_del()       : sent kill to remote node %s\n",
               np->nname);
fprintf(stderr,"dsc_serv : dscp_del()       : rstat %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_del()       : PID %d\n", pp->prob_PID);
#endif

         }/* end if execution node is known */
      } /* end if executing remotely */
      pp->prob_PID = (EOF);           /* the process is gone */
    }

   /*                        */
   /* delete temporary files */
   /*                        */

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : deleting temporary files\n");
#endif

   if ((pp->compl_stat & DSCERR_MASK)==0) {
     char fid[DSC_NAMEZ], *cptr;

     unlink( pp->src_name );      /* delete executable file */

     sprintf( fid, "%s.log", pp->src_name);              /* delete log file */
     unlink( fid );

     /*                                            */
     /* subproblems should have more files deleted */
     /*                                            */
     if (pp->depth != 0) {
        cptr = strrchr( pp->src_name, '/'); /* delete these iff $DSC_PATH */
        if (cptr != 0) cptr++;
	else cptr = pp->src_name;

        if (pp->ctyp) {
          sprintf( fid, "%s.c", cptr);
          unlink( fid );
	  }                                              /* delete c file */
        if (pp->Ctyp) {
          sprintf( fid, "%s.C", cptr);
          unlink( fid );                                           /* (AAP) */
	  }                                              /* delete c++ file */
        if (pp->ityp) {
          sprintf( fid, "%s.i", cptr);
          unlink( fid );
          }                                              /* delete i file */
        if (pp->ltyp) {
          sprintf( fid, "%s.lsp", cptr);
          unlink( fid );
          }                                              /* delete lisp file */
        if (pp->mtyp) {
          sprintf( fid, "%s.m", cptr);
          unlink( fid );
          }                                              /* delete Math file */
        if (pp->ptyp) {
          sprintf( fid, "%s.mpl", cptr);
          unlink( fid );
          }                                              /* delete Maple file*/
        if (pp->otyp) {
          sprintf( fid, "%s.o", cptr);
          unlink( fid );
	}                                                /* delete o file */
      }/* end if subproblem */
     }/* end if successfull completion */

   dscp_unlink( pp );

   pp->depth = (EOF);
   pp->prob_PID = (EOF);
   pp->parent_idx = (EOF);
   pp->index = (EOF);

   while( (prob_db[pcnt-1].depth == (EOF)) && (pcnt > 0)) pcnt--;

#ifdef  DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : returning DSC_SUCCESS\n");
#endif

   return( DSC_SUCCESS );

}

/*****************************************************************************/
/*                                                                           */
/* subroutine:  dscp_compl( pp, pa, pn, name, pstat )                        */
/*                                                                           */
/* inputs:    pp - ptr to problem with error                                 */
/*            pa - ptr to parent problem                                     */
/*            pn - parent node address                                       */
/*            name - name of problem                                         */
/*            pstat - problem completion status                              */
/*                                                                           */
/* outputs:   (database is updated)                                          */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  declare a problem as complete.                                  */
/*           kill it if it is still alive                                    */
/*           check parent block state & unblock if needed                    */
/*                                                                           */
/*****************************************************************************/

int dscp_compl( pp, pa, pn, name, pstat )
DSCDB *pp;        /* problem (if known)                                      */
DSCDB *pa;        /* parent problem                                          */
unsigned long pn;
char *name;       /* source file name                                        */
long pstat;       /* problem status                                          */
{
   int rstat=DSC_SUCCESS;
   DSCDB *prob_find();
   char *dscd_getword();        /* extract name from FID                     */


#ifdef DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : DSCDB *pp\n");
if ( pp != (DSCDB*) EOF )
{
fprintf(stderr,"dsc_serv : dscp_compl()     : problem name %s\n",
                 pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : source file name %s\n",
                 pp->src_name);
}
fprintf(stderr,"dsc_serv : dscp_compl()     : DSCDB *pa\n"); 
if ( pa != (DSCDB*) EOF )
{
fprintf(stderr,"dsc_serv : dscp_compl()     : problem name %s\n",
                 pa->prob_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : source file name %s\n",
                 pa->src_name);
}
fprintf(stderr,"dsc_serv : dscp_compl()     : char *name %s\n",name);
fprintf(stderr,"dsc_serv : dscp_compl()     : long pstat %ld\n",pstat); 
#endif

  errno = 0;

  /*                          */
  /* update completed problem */
  /*                          */
  if (pp!= (DSCDB *)(EOF)) {
   if (!pp->compl_flg) {
    if (pp->enode_addr == my_addr) {

#ifdef  DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : executing local \n");
#endif

      if (pp->prob_PID != (EOF)) {
         rstat = kill( pp->prob_PID, SIGKILL );

#ifdef DSPCOMPL 
fprintf(stderr,"dsc_serv : dscp_compl()     : killing %s\n",pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : source file %s\n",pp->src_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : rstat %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_compl()     : errno %d\n",errno);
fprintf(stderr,"dsc_serv : dscp_compl()     : PID %d\n", pp->prob_PID);
#endif

      } /* end if PID is kown */
    } /* end if executing local */
    else {

      NODE_REC *np, *dscn_find();

#ifdef  DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : remote execution\n");
#endif

      np = dscn_find( 0, pp->enode_addr,0);

      if ((np != (NODE_REC *) EOF) && !pp->copy_flg) {
         strcpy( killmsg.prob_name, pp->prob_name);
         killmsg.from_PID = htonl(-2);
         rstat = dsc_dg_send( &killmsg, sizeof(DSCKILL), sockdg,
		    np->portg, np->naddr);

#ifdef  DSPCOMPL 
fprintf(stderr,"dsc_serv : dscp_compl()     : killing %s\n",pp->prob_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : source file %s\n",pp->src_name);
fprintf(stderr,"dsc_serv : dscp_compl()     : sent kill to remote node %s\n",
               np->nname);
fprintf(stderr,"dsc_serv : dscp_compl()     : rstat %d\n",rstat);
fprintf(stderr,"dsc_serv : dscp_compl()     : PID %d\n", pp->prob_PID);
#endif

       }/* end if node found & copy not already complete */
    } /* end if executing remotely */
    pp->prob_PID = (EOF);           /* the process is gone */
   } /* end if already complete */
   pp->compl_flg = TRUE;           /* must be set before sub_check() */
   pp->compl_stat |= pstat;
   strcpy( solvmsg.src_name, pp->src_name); 
   solvmsg.status = htonl(pp->compl_stat);
   solvmsg.index = htonl(pp->index);

#ifdef  DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_del()       : kill children\n");
#endif
   
   while( pp->child_next != (EOF)) {
       DSCDB *pc;
       pc = dscp_find(4,pp->child_next);
       if (!pc->compl_flg)
         dscp_compl( pc, pp, pc->pnode_addr, pc->prob_name, DSCERR_P_ORPHANED);
       dscp_del(pc);
       }

 } /* end if problem is in database */
 else {

#ifdef DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : problem is not yet in database\n");
#endif

    solvmsg.src_name[0] = 0;
    solvmsg.status = htonl(DSC_FAILURE);
    solvmsg.index = htonl(EOF);
 } /* end if problem is not yet in database */

  /*                       */
  /* update parent problem */
  /*                       */
  if (pa!=(DSCDB *)(EOF)) {

#ifdef  DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : updating parent\n");
#endif

     /*                           */
     /*   update parent status    */
     /*                           */
     if ((pstat & DSCERR_MASK) &&
         (!(pstat & DSCERR_P_ORPHANED)))
       pa->compl_stat |= DSCERR_S_GENERAL;
     if (pp != (DSCDB *)EOF) {
sprintf( logstr, "dsc_serv : dscp_del()       : %s: subproblem %s complete, status %08x\n", pa->prob_name, pp->src_name, pp->compl_stat);
       dsc_log( pa, logstr);
#ifdef DSPDEL
       fprintf( stderr, logstr);
       fflush( stderr );
#endif
       }
     pa->num_sub--;
     pa->awk_sub--;

     /*   UNBLOCK PARENT IF ALL DONE   */
     sub_check( pa );
     }

 /*                                                                    */
 /* clean up this problem if no longer needed, must follow sub_check() */
 /*                                                                    */
  if (pp!=(DSCDB *)(EOF)) {

#ifdef DSPDEL
fprintf(stderr,"dsc_serv : dscp_del()       : cleaning up problem\n");
#endif

   /* delete subproblems whose parents are on another node */
    if (pp->pnode_addr != my_addr) dscp_del( pp );
   /* delete problems which have no parents */
    else if (pp->depth == 0) dscp_del( pp );
    else if (pp->parent_idx == (EOF)) dscp_del( pp );
    }

  /*                      */
  /* prepare solv message */
  /*                      */
  strcpy( solvmsg.prob_name, name);
  solvmsg.act_prob = htonl(act_prob);
  solvmsg.blk_prob = htonl(blk_prob);

  /*                                   */
  /* send solv msg to owner DSC server */
  /*                                   */
  if (pn != my_addr) {
     NODE_REC *np;

#ifdef DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : sending solv msg to owner DSC server\n");
fprintf(stderr,"dsc_serv : dscp_compl()     : RP will copy answer file back\n");
#endif

     /*                                          */
     /*  send solv message                       */
     /*     then RP will copy answer file back   */
     /*                                          */

     np = dscn_find( 0, pn,0);

     if (np!=(NODE_REC *)(EOF)) {
       if (np->act_prob != (-1))     /* only if active */
           dsc_dg_send( &solvmsg,
		       sizeof( DSCSOLV),
		       sockdg,
		       np->portg,
		       np->naddr);
      } /* end if node found */
     }           /* end if parent is remove */
  else {

#ifdef DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : parent node is local\n");
#endif
       }

#ifdef DSPCOMPL
fprintf(stderr,"dsc_serv : dscp_compl()     : returning rstat %d\n",rstat);
#endif

  return( rstat);

} 

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscp_begin( sock, index )                                     */
/*                                                                           */
/* inputs:    sock - socket to send message on                               */
/*            index - index of problem being reported                        */
/*                                                                           */
/* outputs:   none                                                           */
/*                                                                           */
/* return:    none                                                           */
/*                                                                           */
/* purpose:   return the problem index to the parent                         */
/*            with this information the parent can kill a child              */
/*            or wait on a particular child, as opposed to the whole lot.    */
/*                                                                           */
/*****************************************************************************/

void dscp_begin( sock, index )
int sock;
long index;
{
   DSCBEG msg;      
   int rlen;    

#ifdef DSPBEGIN
fprintf(stderr,"dsc_serv : dscp_begin       : int sock %d\n",sock);
fprintf(stderr,"dsc_serv : dscp_begin       : long index %ld\n",index); 
fflush(stderr);
#endif

   msg.msg_type = msg.sanity = htonl(MSG_BEGIN);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl( sizeof( DSCBEG ));
   msg.index = htonl( index );

#ifdef DSPBEGIN
fprintf(stderr,"dsc_serv : dscp_begin       : writing message to socket\n");
#endif

   rlen = write ( sock, &msg, sizeof( DSCBEG ));

#ifdef DSPBEGIN
errno = 0;
fprintf(stderr,"dsc_serv : dscp_begin       : wrote %d\n",rlen);
fprintf(stderr,"dsc_serv : dscp_begin       : expected %d\n",sizeof( DSCBEG ));
fprintf(stderr,"dsc_serv : dscp_begin       : errno %d\n",errno);
#endif

  /* wait till msg is read and socket is closed            */
  /* this read should always complete with socket broken   */
  /* it is now commented out due to the fact that the read */
  /* hangs until the cntrl program is used to rise the     */
  /* server.                                               */
/*
   rlen = read( sock, &msg, sizeof(DSCBEG));
#ifdef DSPBEGIN
errno = 0;
fprintf(stderr,"dsc_serv : dscp_begin       : read %d\n",rlen);
fprintf(stderr,"dsc_serv : dscp_begin       : expected %d\n",sizeof( DSCBEG ));
fprintf(stderr,"dsc_serv : dscp_begin       : errno %d\n",errno);
#endif
*/

#ifdef DSPBEGIN
fprintf(stderr,"dsc_serv : dscp_begin       : returning void\n");
fflush(stderr);
#endif

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_getword( string )                                        */
/*                                                                           */
/* inputs:   string - full FID                                               */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   success = pointer to last entry in FID                          */
/*           failure = '\0'                                                  */
/*                                                                           */
/* purpose:  extract file name from the FID                                  */
/*                                                                           */
/*****************************************************************************/

char *dscd_getword( string )
char *string;
{
   char *wptr;  /* pointer to last entry in FID                              */
   char *cptr;  /* temporary pointer                                         */

#ifdef DSDGETWORD
fprintf(stderr,"dsc_serv : dscd_getword()   : string %s\n", string);
fflush(stderr);
#endif

 for ( wptr = cptr = string; *cptr != '\0' ; cptr++)
  {
   if ( *cptr == '/' ) 
      { cptr++;
        if ( *cptr == '\0' ) { 
fprintf(stderr,"dsc_serv : dscd_getword()   : bad FID %s!\n",string);
fprintf(stderr,"dsc_serv : dscd_getword()   : forced halt!\n");
fflush(stderr);
        dscd_halt();
        }
        wptr = cptr;
      }
  }

#ifdef DSDGETWORD
fprintf(stderr,"dsc_serv : dscd_getword()   : returning wptr %s\n",wptr);
fflush(stderr);
#endif

 return(wptr);

}
