/*****************************************************************************/
/*                                                                           */
/*                                 dsc_psstat.c                              */
/*                                                                           */
/* Change History:                                                           */
/* ~~~~~~~~~~~~~~~                                                           */
/* (AAP) - Added Ctyp flag for C++ file recognition.                         */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*      dsc_psstartup() - startup ps RP deamon                               */
/*      dscd_psstat()   - handle local ps RP deamon message                  */
/*      dscd_psrstat()  - handle remote ps RP deamon message                 */
/*      sendstat_loop() - not used                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_psstartup()                                               */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* ext vars: psid - process id of the ps deamon (-1 denotes startup failure  */
/*           pssched -  dynamic scheduling flag                              */
/*                                                                           */
/* outputs:  started ps RP deamon                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  startup ps RP deamon                                            */
/*                                                                           */
/*****************************************************************************/

int dsc_psstartup()
{
   char temp1[200];   /* local node name                                     */
   char temp2[400];   /* ps RP deamon execute command                        */
   int i;

#ifdef PSSUP
fprintf(stderr,"dsc_serv : dsc_psstartup()  : void\n");
fflush(stderr);
#endif

   pssched = TRUE;

   strcpy(temp1,my_node);
   i = strcspn (temp1, ".");
   temp1[i]= '\0';
   strcat ( temp1 , "_dsc_ps.log");

   sprintf(temp2,"%s %s %d %d",DSCPSPROC,temp1,portnm,numproc);

#ifdef PSSUP
fprintf(stderr,"dsc_serv : dsc_psstartup()  : Starting up DSC PS deamon\n");
fprintf(stderr,"dsc_serv : dsc_psstartup()  : %s \n",temp2);
fflush(stderr);
#endif

   pspid = fork();
   if ( pspid == 0 ) {
   dsc_exec( temp2 );
fprintf(stderr,"dsc_serv : dsc_psstartup()  : error execing %s\n",temp2);
fprintf(stderr,"dsc_serv : dsc_psstartup()  : errno %d\n",errno);
fflush(stderr);
   exit(errno); 
   }
   else 
    if ( pspid == (-1)){
     perror("forking ps deamon");
fprintf(stderr,"dsc_serv : dsc_psstartup()  : error forking ps deamon!\n");
fprintf(stderr,"dsc_serv : dsc_psstartup()  : errno %d\n",errno);
fflush(stderr);
    }

#ifdef PSSUP
fprintf(stderr,"dsc_serv : dsc_psstartup()  : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

  return (DSC_SUCCESS);
 
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_psstat()                                                 */
/*                                                                           */
/* inputs:   msg - pointer to message buffer received                        */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose: handle local ps RP deamon message                                */
/*                                                                           */
/*****************************************************************************/

int dscd_psstat( msg )
DSCPSSTAT *msg;
{
   int rstat = DSC_SUCCESS;
   int pstat = DSC_SUCCESS;
   int tmem;
   int thres1=5;              /* minimum avaliable memory needed in Megs     */
   int thres2=-5;             /* swap Factor in Megs                         */
   int i;
   DSCDB    *pp;              /* pointer to problem in data base             */
   DSCDB *dscp_find();        /* function to find a problem in data base     */

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()    : DSCPSSTAT *msg\n");
fprintf(stderr,"dsc_serv : dscd_psstat()    : local ps RP deamon message\n");
fprintf(stderr,"dsc_serv : dscd_psstat()    : %s\n",my_node);
fprintf(stderr,"dsc_serv : dscd_psstat()    :  cpu %d mem %d \n",
        ntohl(msg->cpu),ntohl(msg->mem));
fflush(stderr);
#endif  

   cpu = ntohl(msg->cpu);
   mem = ntohl(msg->mem);
   tmem = (int)((255-mem)*memrate/255);

   if ( waiting_on_sub > 0 )
    {

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()    : attempting to dequeu!\n");
fflush(stderr);
#endif

   if  (tmem >= thres1)
    {

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()    : able to deque on %s \n",my_node);
fflush(stderr);
#endif

      i = 0;
      while( (pp=dscp_find( 2, &i)) != (DSCDB *)(EOF))
       {
        if ( pp->prob_PID == (0)  && tmem >= thres1 &&
             (tmem - pp->memrating) >= thres2  ) 
         {  

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()    : dequeing problem %s \n"
                 ,pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_psstat()    : src_name %s \n"
                 ,pp->src_name);
fprintf(stderr,"dsc_serv : dscd_psstat()    : src_build %s \n"
                 ,pp->src_build);
fprintf(stderr,"dsc_serv : dscd_psstat()    : PID %d \n"
                 ,pp->prob_PID);
fflush(stderr);
#endif

          pp->enode_addr = my_addr;
          if ((pstat=dscp_build( pp )) != DSC_SUCCESS)
            {
             sprintf( logstr, "%s: build failed (on dequeue) : %s\n",
             pp->prob_name, pp->src_build);
             dsc_log( pp, logstr);

             fprintf( stderr,"dsc_serv : dscd_psstat()    : %s",logstr);
             fflush(stderr);

             dscp_compl( pp, dscp_find( 4, pp->parent_idx),
             pp->pnode_addr, pp->prob_name, pstat|DSCERR_P_BUILD);

                goto XDONE;
            }   /* end if build failed */
#ifdef PSSSTAT
 else
  {
fprintf(stderr,"dsc_serv : dscd_psstat()     : successful build!\n");
fprintf(stderr,"dsc_serv : dscd_psstat()     : PID %d \n",pp->prob_PID);
fflush(stderr);
  }
#endif

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()     : currect mem value %d\n",mem);
fprintf(stderr,"dsc_serv : dscd_psstat()     : currect cpu value %d\n",cpu);
fflush(stderr);
#endif

         /* Update temporary mem and local cpu and memory values             */
         mem = mem + pp->memrating*255/memrate;
         tmem = tmem - pp->memrating; 
         cpu = cpu + pp->cpurating*numproc*255/cpurate;

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()     : updated mem value %d\n",mem);
fprintf(stderr,"dsc_serv : dscd_psstat()     : updated cpu value %d\n",cpu);
fflush(stderr);
#endif

         } /* end if this job needs running */
       } /* end for all jobs in database */
    } /* end if not sufficient memory */   

#ifdef PSSSTAT
 else
  {
fprintf(stderr,"dsc_serv : dscd_psstat()     : can not dequeue!\n");
fprintf(stderr,"dsc_serv : dscd_psstat()     : insufficient memory\n");
fflush(stderr);
  }
#endif

    } /* end if watting on sub */ 

#ifdef PSSSTAT
 else
  {
fprintf(stderr,"dsc_serv : dscd_psstat()     : no queued problems\n");
fflush(stderr);
  }
#endif

XDONE:

   fflush(stdout);
   fflush(stderr);

#ifdef PSSSTAT
fprintf(stderr,"dsc_serv : dscd_psstat()     : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_psrstat()                                                */
/*                                                                           */
/* inputs:   msg - pointer to message buffer received                        */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  handle remote ps RP deamon message                              */
/*                                                                           */
/*****************************************************************************/

int dscd_psrstat( msg )
DSCPSRSTAT *msg;
{
   int rstat = DSC_SUCCESS;
   int tmem;
   int thres1=5;              /* minimum avaliable memory needed in Megs     */
   int thres2=-5;             /* swap Factor in Megs                         */
   int i;
   NODE_REC *np;              /* pointer to node in data base                */
   NODE_REC *dscn_find();     /* function to find a node in data base        */
   DSCDB *pp;                 /* pointer to problem in data base             */
   DSCDB *dscp_find();        /* function to find a problem in data base     */
   DSCDB *pa;                 /* pointer to parent problem in dbase          */
   DSCSTRT msg2;

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : DSCPSRSTAT *msg\n");
fprintf(stderr,"dsc_serv : dscd_psrstat()   : remote ps RP deamon message\n");
fprintf(stderr,"dsc_serv : dscd_psrstat()   : from %s\n",msg->fnname);
fprintf(stderr,"dsc_serv : dscd_psstat()    :  cpu %d mem %d \n",
        ntohl(msg->cpu),ntohl(msg->mem));
fflush(stderr);
#endif 

   /*                                                                    */
   /* This call to dscn_find could cause trouble when a node name is not */
   /* found in the node db and hence is not in the net file. This could  */
   /* inadvertenly happen when a node has two names.                     */
   /*                                                                    */

   np = dscn_find(0,msg->fnode,0); 

   if ( np != ( (NODE_REC *) (EOF) ) )
   { 
    np->cpu = ntohl(msg->cpu);
    np->mem = ntohl(msg->mem);
    tmem = (int)((255-np->mem)*memrate/255);
   if ( waiting_on_sub > 0 )
    {

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : attempting to dequeu!\n");
fflush(stderr);
#endif

      if  (tmem >= thres1)
       {

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : able to deque on %s \n",my_node);
fflush(stderr);
#endif

      i = 0;
      while( (pp=dscp_find( 2, &i)) != (DSCDB *)(EOF))
       {
        if ( pp->prob_PID == (0)  && tmem >= thres1 &&
             (tmem - pp->memrating) >= thres2  )
         {

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : dequeing problem %s \n"
                 ,pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : src_name %s \n"
                 ,pp->src_name);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : src_build %s \n"
                 ,pp->src_build);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : PID %d \n"
                 ,pp->prob_PID);
fflush(stderr);
#endif

          pa = dscp_find(0, ntohl(pp->parent_PID));

          msg2.msg_type = msg2.sanity = htonl(MSG_START); 
          strcpy(msg2.dsc_password , password );
          msg2.msg_size = htonl(sizeof( DSCSTRT ));
          /*                                                   */
          /* msg.addr is the socket to send a begin message to */
          /* this does not apply here                          */
          /* memcpy( &msg2.addr, &paddr, sizeof(paddr));       */
          /*                                                   */
          msg2.parent_PID = htonl(pp->parent_PID); 
          msg2.fnode = htonl(pp->pnode_addr);
          msg2.subflg = htonl(pp->depth); 
          msg2.cpurating = htonl(pp->cpurating);
          msg2.memrating = htonl(pp->memrating);
          strcpy( msg2.prob_name, pp->prob_name);
          strcpy( msg2.src_name, pp->src_name);
          strcpy( msg2.src_build, pp->src_build);
          strcpy( msg2.src_exec, pp->src_exec);
          msg2.ctyp = htonl(pp->ctyp);
          msg2.Ctyp = htonl(pp->Ctyp);     /* (AAP) */
          msg2.ltyp = htonl(pp->ltyp);
          msg2.mtyp = htonl(pp->mtyp);
          msg2.ptyp = htonl(pp->ptyp);
          msg2.otyp = htonl(pp->otyp);
          msg2.ityp = htonl(pp->ityp);
          msg2.dtyp = htonl(pp->dtyp);
          msg2.cpu_type = htonl(pp->cpu_type);
          rstat = dscp_export( pp, pa, &np, msg2);

          if (rstat != DSC_SUCCESS)
          { 
fprintf(stderr,"dsc_serv : dscd_psrstat()   : error in dscp_export\n");
fprintf(stderr,"dsc_serv : dscd_psrstat()   : problem %s\n"
                          ,pp->prob_name);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : problem not discarded\n");
fflush(stderr);
          }
          else
          {

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : successful export!\n");
fprintf(stderr,"dsc_serv : dscd_psrstat()   : currect mem value %d\n",mem);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : currect cpu value %d\n",cpu);
fflush(stderr);
#endif

         /* Update temporary mem and local cpu and memory values             */
         np->mem = np->mem + pp->memrating*255/np->memrating;
         tmem = tmem - pp->memrating;
         np->cpu = np->cpu + pp->cpurating*np->numproc*255/np->cpurating;

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : updated mem value %d\n",mem);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : updated cpu value %d\n",cpu);
fflush(stderr);
#endif

         pp->prob_PID = -1;
         waiting_on_sub--;  
          }
         } /* end if this job needs running */
       } /* end for all jobs in database */
    } /* end if not sufficient memory */

#ifdef PSRSTAT
 else
  {
fprintf(stderr,"dsc_serv : dscd_psrstat()   : can not dequeue!\n");
fprintf(stderr,"dsc_serv : dscd_psrstat()   : insufficient memory\n");
fflush(stderr);
  }
#endif

    } /* end if watting on sub */
   }
   else
    {
fprintf(stderr,"dsc_serv : dscd_psrstat()   : could not find %s in node dbase\n"
                  ,msg->fnname);
fprintf(stderr,"dsc_serv : dscd_psrstat()   : no dequeueing attempted\n");
fflush(stderr);
    }

XDONE:

   fflush(stdout);
   fflush(stderr);

#ifdef PSRSTAT
fprintf(stderr,"dsc_serv : dscd_psrstat()   : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: sendstat_loop()                                               */
/*                                                                           */
/* inputs:   msg - pointer to message buffer received                        */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  send current cpu and mem to node[loop_poss]                     */
/*                                                                           */
/*****************************************************************************/

int sendstat_loop()
{
 int rstat;
 int counter;
 int tmppos=0;
 DSCPSRSTAT msg;
 NODE_REC *np,*npp,*dscn_find();

#ifdef PSSENDL
fprintf(stderr,"dsc_serv : sendstat_loop()  : void\n");
fprintf(stderr,"dsc_serv : sendstat_loop()  : loop_poss %d\n",loop_poss);
fflush(stderr);
#endif

 msg.msg_type = msg.sanity = htonl(MSG_PSRSTAT);
 strcpy(msg.dsc_password , password );
 msg.msg_size = htonl(sizeof(MSG_PSRSTAT));
 msg.fnode = my_addr;
 strcpy( msg.fnname, my_node);
 
 counter = 0;
 if ( loop_poss > dbtop ) loop_poss = 0;

 np = (NODE_REC *) (EOF);

 tmppos = loop_poss;
 while ( ( np == (NODE_REC *) (EOF) ) && ( counter < dbtop ) )
 {
   np = dscn_find( 3, loop_poss, 0 );
   counter++;
   loop_poss++;
   if ( loop_poss > dbtop ) loop_poss = 0;
 }
 loop_poss = tmppos;

 if ( np != (NODE_REC *) (EOF) ) 
 {

#ifdef PSSENDL
fprintf(stderr,"dsc_serv : sendstat_loop()  : sending cpu %d mem %d\n",cpu,mem);
fprintf(stderr,"dsc_serv : sendstat_loop()  : for %s ",my_node);
fflush(stderr);
#endif

 msg.cpu = cpu;
 msg.mem = mem;
 
 rstat = dsc_dg_send( &msg, (int) sizeof(DSCPSRSTAT), sockdg,
                      np->portg, np->naddr);

#ifdef PSSENDL
fprintf(stderr,"dsc_serv : sendstat_loop()  : to %s \n",np->nname);
fflush(stderr);
#endif

 }

#ifdef PSSENDL
  else
   {
fprintf(stderr,"dsc_serv : sendstat_loop()  : no active to send status\n");
fflush(stderr);
   }
#endif

 loop_poss++;

XDONE:
 
#ifdef PSSENDL
fprintf(stderr,"dsc_serv : sendstat_loop()  : returning rstat %d\n",rstat);
fflush(stderr);
#endif
 
 return(rstat);

}
