/*****************************************************************************/
/*                                                                           */
/*                                dsc_serv.c                                 */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"                      /* dsc_serv external variables       */

int 		my_pid;		        /* this process's PID                */
char 		my_node[32];		/* local node name                   */
unsigned long   my_addr;                /* local host address                */
unsigned short  my_portg;               /* datagram port number              */
char 		msg[MAXMSG];		/* general message buffer            */
int             sockdg;                 /* datagram descriptor               */
int             portnm;                 /* int datagram port number          */
char            *cportnm;               /* char datagram port number         */
char            *password;              /* DSC_PASSWORD                      */
char            *dscpath;               /* DSC_PATH in dsc_setup.csh         */
int  		act_prob;		/* # active problems                 */
int		blk_prob;		/* # blocked problems                */
int             cpu;                    /* current CPU usage                 */
int             mem;                    /* current MEM usage                 */
int             cpurate;                /* cpu rating                        */
int             memrate;                /* mem rating                        */
int             numproc;                /* number of processors              */
int             fsystem;                /* file system number                */

DSCSOLV         solvmsg;
DSCKILL         killmsg;
int             waiting_on_sub=0;
char            pwd[100];               /* expanded path information         */ 

#define LOOP_WAIT 25
#define LOOP_WAITT 30
int             loop_pos;               /* loop position for checkin         */
int             loop_poss;              /* loop position for sendstat        */
int             dbtop;                  /* top of node database array        */
int             pspid;                  /* pid of the dsc_ps deamon          */
int             pssched;                /* scheduler flag                    */
int             loop_flg;               /* outstanding problem check flag    */
int             cpytry=0;               /* coroutine file copy flag          */

struct rusage rusage;                   /* shared w/ subproc signal          */
union wait statusp;                     /* in dsc_misc.c                     */

/*****************************************************************************/
/*                                                                           */
/* main : dsc_serv                                                           */
/*                                                                           */
/* inputs:  argv[1] - local node name in the form nodename_dsc_serv.log      */
/*          argv[2] - datagram port number                                   */ 
/*                                                                           */
/* outputs: log messages                                                     */
/*                                                                           */
/* purpose: redirect stderr & stdout to argv[1]                              */
/*          set the password                                                 */
/*          set the portnumber                                               */
/*          create inet datagram socket                                      */
/*          kill start a new log file                                        */
/*          write dsc_setup.csh file                                         */
/*          start accepting datagrams                                        */
/*                                                                           */
/*****************************************************************************/

main( argc, argv )
int argc;
char **argv;
{
static   int msgsock;
static   char buf[MAXMSG];
static   DSCCPYCTRL *msg;
static   int rval;                     /* return value                       */
static   fd_set fdset;                 /* fd mask                            */
static   struct timeval to;            /* max interval to wait for selection */
static   struct timeval too;           /* max interval to wait for selection */
static   struct sockaddr_in server;    /* socket information                 */
static   int on;
static   int i;
static   int length;                   /* size of sockaddr_in                */
static   int mask_normal;              /* normal signal mask                 */
static   struct linger lo;             /* linger option                      */
static   int pid;                      /* pid of child processes             */

   FILE *fp;                           /* kill started argv[1] flie pointer  */
   FILE *dscsetup;                     /* dsc_setup.csh file pointer         */
   void subproc_body();                /* handle terminated child process    */
   int dscd_halt();                    /* orderly server halt                */ 
   int subproc_signal();               /* call by UNIX asynchronously        */
   char *getenv();                     /* get value of environment variables */
   char *homedir;                      /* home directory                     */
   char *setupname[DSC_FILEZ];         /* full path/name of the setup file   */ 
   struct tm *tmp;                     /* structure returned by  localtime   */
   time_t tt;                          /* value = secs since epoch           */
   char *dsc_getpass();                /* get password from dsc_setup.csh    */
   char *dsc_getpath();                /* get DSC_PATH from dsc_setup.csh    */

   /*                          */
   /* start mainline code      */
   /* redirect stderr & stdout */
   /*                          */
   if (argc > 1) {
     fp = fopen(argv[1], "a+");
     dup2(fileno(fp),fileno(stdout));
     dup2(fileno(fp),fileno(stderr));
     }

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : argv[1] %s\n",argv[1]);
fprintf(stderr,"dsc_serv : main             : argv[2] %s\n",argv[2]);
#endif

   /*                                                        */
   /* set the external password variable to the DSC_PASSWORD */
   /*                                                        */
   password = dsc_getpass();

   /*                                                   */
   /* set the external dscpath variable to the DSC_PATH */
   /*                                                   */

   dscpath = dsc_getpath();

   /*                                                   */
   /* set the datagram portnumber from the command line */
   /*                                                   */
   portnm = atoi(argv[2]);

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : password %s\n",password);
fprintf(stderr,"dsc_serv : main             : portnumber %d\n",portnm);
#endif 

   cpytry = 0;
   msg = (DSCCPYCTRL *)buf;
   my_pid = getpid();
   /* sprintf( buf, "LD_OPTIONS=%s/dscpr.o", pwd);*/
   sprintf(buf,"");
   fflush( stderr );

   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"dsc_serv : main             : getting cwd\n");
     fprintf(stderr,"dsc_serv : main             : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }

   /*                */
   /* setup solvmsg  */
   /*                */
   strcpy(solvmsg.dsc_password , password );
   solvmsg.msg_type = solvmsg.sanity = htonl( MSG_SOLVE );
   solvmsg.msg_size = htonl( sizeof( DSCSOLV ));

   /*               */
   /* setup killmsg */
   /*               */
   strcpy(killmsg.dsc_password , password );
   killmsg.msg_type = solvmsg.sanity = htonl( MSG_KILL );
   killmsg.msg_size = htonl( sizeof( DSCKILL ));

   dscp_init();       /* problem database init */
   dscn_init();       /* network init */

   /*                             */
   /* create INET DATAGRAM socket */
   /*                             */
   sockdg = socket( AF_INET, SOCK_DGRAM, 0);
   if (sockdg < 0) {
     perror( "\topening datagram socket");
fprintf(stderr,"dsc_serv : main             : opening datagram socket\n");
fprintf(stderr,"dsc_serv : main             : forced halt!\n");
     exit(1);
   }

   /*                      */
   /* make socket reusable */
   /*                      */
   setsockopt( sockdg, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
   lo.l_onoff = TRUE;
   lo.l_linger = 120;           /* wait for remote node to accept msg */
   setsockopt( sockdg, SOL_SOCKET, SO_LINGER, &lo, sizeof(lo));   

   /*                             */
   /* name socket using wildcards */
   /*                             */
   server.sin_family = AF_INET;
   server.sin_addr.s_addr = INADDR_ANY;
   server.sin_port = my_portg;
   if (bind( sockdg, &server, sizeof(server))){
     perror("\ttrying to start DSC server [again]");
fprintf(stderr,"dsc_serv : main             : trying to start DSC server [again]\n");
fprintf(stderr,"dsc_serv : main             : forced halt!\n");
     exit(2);
   }

   /*                               */
   /* find out assigned port number */
   /*                               */
   length = sizeof( server );
   if (getsockname(sockdg, &server, &length)){
     perror( "\tgeting socket name");
fprintf(stderr,"dsc_serv : main             : geting socket name\n");
fprintf(stderr,"dsc_serv : main             : forced halt!\n");
     exit(3);
   }

   /*                                        */
   /* if we got this far, then startup is ok */
   /* so kill start a new log file           */
   /*                                        */
   if (argc > 1) {
     fclose( fp);
     unlink( argv[1] );
     fp = fopen(argv[1], "w+");
     dup2(fileno(fp),fileno(stdout));
     dup2(fileno(fp),fileno(stderr));
     }

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : argv[1] %s\n",argv[1]);
fprintf(stderr,"dsc_serv : main             : argv[2] %s\n",argv[2]);
#endif


/*                                                                           */
/* the runtime socket option was implemented by telling the server what port */
/* number to monitor for datagrams via the command line. Once this           */
/* information is recieved and verified both the server log file and the     */
/* dsc_setup.csh files are modified/created to include the portnumber.       */
/*                                                                           */

 /*                                          */
 /* Determine curent time for the Time stamp */
 /*                                          */
 time(&tt);
 tmp = localtime(&tt);

   fprintf( stderr, "Distributed Symbolic Computation environment\n");
   fprintf( stderr, "\t(c) 1990, 1991, 1992, 1993, 1994\n");
   fprintf( stderr, "\t(c) Kurt W. Schmitz, Angel L. Diaz, Erich Kaltofen\n");
   fprintf( stderr, "time stamp = %s",asctime(tmp));
   fprintf( stderr, "dsc_serv pid = %d\n", my_pid);
   fprintf( stderr, "dsc_serv port number = %d\n",portnm);

   /*                                                        */
   /* if we fail from here on out, must do orderly shutdown  */
   /* this call is specific to  the  SunOS  operating system */

#ifdef sun
   on_exit( dscd_halt, 0);   
#endif

   signal( SIGPIPE, SIG_IGN );
   signal( SIGQUIT, dscd_halt);
   signal( SIGTERM, dscd_halt);
   signal( SIGCHLD, subproc_signal);
   mask_normal = sigblock( sigmask(SIGPIPE));

   dscn_chkin( sockdg );
   nice( nice1 );        /* in DEAMON mode, we will run a long time */

   /*                            */
   /* start up the DSC PS deamon */
   /*                            */
   dsc_psstartup();

 /* Due to the run time socket number enhancement the file "dsc_setup.csh" */
 /* must be modified to include the proper DSC_PORT number used for the    */
 /* interprocess comminication. This number is placed in the file only     */
 /* upoun successfull startup of the server program. However this varibale */ 
 /* can not be read with the getenv command since the putenv command only  */
 /* changes environment varibales in the calling programs own environment. */
 /* Thus all queries to the DSC_PORT number (by dscpr_sub ...) will be     */
 /* read from the dsc_setup.csh file not the environment. This however does*/
 /* not change the use of the always fixed DSC_PATH variable. The          */
 /* dsc setup file is found in $HOME/dsc_setup.csh.                        */

 
   /*                                */
   /* set up dsc_setup.csh file name */
   /*                                */
   homedir = getenv("HOME");
   sprintf(setupname,"%s/",homedir);
   strcat(setupname,DSC_SETUP_NAME);

   dscsetup = fopen(setupname,"w");
   if ( dscsetup == NULL )
    {
fprintf(stderr,"dsc_serv : main             : can not open %s\n",setupname);
fprintf(stderr,"dsc_serv : main             : errno %d\n",errno);
fprintf(stderr,"dsc_serv : main             : fatal error!\n");
     exit(4);
    }

   /*                                          */
   /* Determine curent time for the Time stamp */
   /*                                          */ 
   time(&tt);
   tmp = localtime(&tt);
 
   fprintf(dscsetup,"# dsc_setup.csh file (sourced by .cshrc) \n"); 
   fprintf(dscsetup,"# Time Stamp : %s",asctime(tmp));
   fprintf(dscsetup,"# DSC_PORT %d \n",portnm);
   fprintf(dscsetup,"# DSC_PASSWORD %s \n",password);
   fprintf(dscsetup,"setenv HOME %s \n",getenv("HOME"));
   fprintf(dscsetup,"setenv DSC_PATH %s \n",dscpath);
   fprintf(dscsetup,"setenv PWD $cwd \n");

   fclose(dscsetup);

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : # dsc_setup.csh file (sourced by .cshrc) \n");
fprintf(stderr,"dsc_serv : main             : # Time Stamp : %s",asctime(tmp));
fprintf(stderr,"dsc_serv : main             : # DSC_PORT %d \n",portnm);
fprintf(stderr,"dsc_serv : main             : # DSC_PASSWORD %s \n",password);
fprintf(stderr,"dsc_serv : main             : setenv HOME %s \n",getenv("HOME"));
fprintf(stderr,"dsc_serv : main             : setenv DSC_PATH %s \n",dscpath);
fprintf(stderr,"dsc_serv : main             : setenv PWD $cwd \n");
fflush(stderr);
#endif

   /*                           */
   /* start accepting datagrams */
   /*                           */
   do {

    /*                                                         */
    /* check about new subproc terminations while we were away */ 
    /*                                                         */
     do {
      pid = wait3( &statusp, WNOHANG, &rusage );
      if (pid > 0) subproc_body( pid );
     } while (pid > 0);

    /*                             */
    /* block waiting for a message */
    /*                             */
     FD_ZERO( &fdset );
     FD_SET( sockdg, &fdset );
     to.tv_sec = LOOP_WAIT;             /* or timeout for checking loop */
     too.tv_sec = LOOP_WAITT;

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : checking descriptor set\n");
fprintf(stderr,"dsc_serv : main             : loop_flg %d\n",loop_flg);
#endif

     do {
       errno = 0;
      /****************************************************
       * if problems are outstanding, send out checking   *
       * messages from time to time, to make sure things  *
       * are still alive...                               *
       ****************************************************/
       if (loop_flg)
	 rval = select( sockdg+1, &fdset, 0, 0, &to);   /* short wait */
       else
	 rval = select( sockdg+1, &fdset, 0, 0, &too);  /* long wait  */
     } while( errno == EINTR );

     if (rval < 0)
      {
        perror( "select");
fprintf(stderr,"dsc_serv : main             : error with select\n");
fprintf(stderr,"dsc_serv : main             : errno %d\n",errno);
      }
     /*                                             */
     /* when the timer expires, continue checking   */
     /* viability of remote nodes.                  */
     /* if one dies, restart problem somewhere else */
     /*                                             */
     else if (rval == 0) { 

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : time limit expired\n");
fprintf(stderr,"dsc_serv : main             : continuing chkin_loop()\n");
#endif

       sigblock( sigmask( SIGCHLD ) );
       chkin_loop();
       i = sigsetmask( mask_normal );
       }
     else {
       rval = dsc_read( sockdg, buf, MAXMSG);
       if (rval == 0)
        {
	 perror( "reading datagram message");
fprintf(stderr,"dsc_serv : main             : err reading datagram message\n");
fprintf(stderr,"dsc_serv : main             : errno %d\n",errno);
        }
       else {

#ifdef DSMAIN
fprintf(stderr,"dsc_serv : main             : msg received and read\n");
#endif

         sigblock( sigmask( SIGCHLD ) );
         dsc_do_msg( buf, NULL);
         i = sigsetmask( mask_normal );
	 }
       }/* end if msg received */
   } while(TRUE);

} /* end main */

