/*****************************************************************************/
/*                                                                           */
/*                                 dsc_solv.c                                */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_solve( msg )                                             */
/*                                                                           */
/* inputs:    msg - pointer to message buffer                                */
/*                                                                           */
/* outputs:  (database update)                                               */
/*           (message to blocked parent (if any))                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  update problem database                                         */
/*           copy solution files from remote nodes                           */
/*           check for blocked parent and all subproblems complete           */
/*           send message to blocked parent if necessary                     */
/*                                                                           */
/*****************************************************************************/

int dscd_solve( msg )
DSCSOLV *msg;
{
   int rstat = DSC_SUCCESS;       /* dscd_solve() return status              */
   DSCDB *prob;                   /* problem data base entry                 */
   DSCDB *dscp_find();            /* find problem in prob dbase function     */
   NODE_REC *np;                  /* node record                             */
   NODE_REC *dscn_find();         /* find node in node dbase function        */

#ifdef DSDSOLVE
fprintf(stderr,"dsc_serv : dscd_solve()     : DSCSOLV *msg\n");
fprintf(stderr,"dsc_serv : dscd_solve()     : msg->src_name\n",
               msg->src_name);
fflush(stderr);
#endif

   /*                              */
   /*   FIND PROBLEM IN DATABASE   */
   /*                              */

   prob = dscp_find( 1, msg->prob_name);

   if (prob == (DSCDB *)EOF) {
fprintf( stderr, "dsc_serv : dscd_solve()     : unknown problem name %s\n", 
               msg->prob_name);
fprintf( stderr, "dsc_serv : dscd_solve()     : msg->src_name\n",
               msg->src_name);
fflush(stderr);
      rstat = DSC_FAILURE;
      goto XDONE;
      }

   /*                                    */
   /* if problem was executed remotely,  */
   /*  copy solution file back here      */
   /*                                    */
   if (prob->enode_addr != my_addr) {

      char name[DSC_NAMEZ], *cptr;

#ifdef DSDSOLVE
fprintf(stderr,"dsc_serv : dscd_solve()     : problem was executed remotely!\n"
);
fprintf(stderr,"dsc_serv : dscd_solve()     : must copy solution file back\n");
fflush(stderr);
#endif

      np = dscn_find( 0, prob->enode_addr,0 );

      np->act_prob = ntohl(msg->act_prob);
      np->blk_prob = ntohl(msg->blk_prob);
      np->err_cnt = 0;

      cptr = strrchr( prob->src_name, '/') + 1;
      if (cptr == (char *)1) cptr = prob->src_name;
      sprintf( name, "%s.d", cptr);      /* retrieve the solution file */

      if (dsc_cpyreq( 0, name, np) != DSC_SUCCESS)
        prob->compl_stat |= DSCERR_P_COMM;
      else prob->copy_flg = TRUE;
      }

#ifdef DSDSOLVE
fprintf(stderr,"dsc_serv : dscd_solve()     : updating dbase\n");
fflush(stderr);
#endif

   /*                     */
   /*   UPDATE DATABASE   */
   /*                     */

   if (prob->enode_addr == my_addr) act_prob--;
   dscp_compl( prob, dscp_find( 4, prob->parent_idx), 
       prob->pnode_addr, prob->prob_name, ntohl(msg->status) );

 XDONE:

#ifdef DSDSOLVE
fprintf(stderr,"dsc_serv : dscd_solve()     : returning rstat %d\n",rstat);
fflush(stderr);
#endif

   return( rstat );

}
