/*****************************************************************************/
/*                                                                           */
/*                                 dsc_stat.c                                */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*      dscd_stat()   - send problem status messages back to requestor       */
/*      dscs_pstat()  - send problem database to requestor                   */
/*      dscs_nstat()  - send node database to requestor                      */
/*      dscs_sstat()  - send server status to requestor                      */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscd_stat( msg, s)                                            */
/*                                                                           */
/* inputs:    msg - pointer to message buffer                                */
/*            s - stream socket to return data on                            */
/*                                                                           */
/* outputs:  (message back to requestor)                                     */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  send problem status messages back to requestor                  */
/*           requestor must be on connectionless socket                      */
/*                                                                           */
/*****************************************************************************/

int dscd_stat( msg, s )
DSCQSTA  *msg;
int s;
{

#ifdef DSDSTAT
fprintf(stderr,"dsc_serv : dscd_stat()      : DSCQSTA  *msg\n");
fprintf(stderr,"dsc_serv : dscd_stat()      : msg->flag %d\n",ntohl(msg->flag));
fprintf(stderr,"dsc_serv : dscd_stat()      : int s %d\n",s);
fflush(stderr);
#endif

   switch (ntohl(msg->flag)) {
   case MSG_PSTAT:
     dscs_pstat(msg, s);
     break;
   case MSG_NSTAT:
     dscs_nstat(msg, s);
     break;
   case MSG_SSTAT:
     dscs_sstat(msg, s);
     break;
   }

#ifdef DSDSTAT
fprintf(stderr,"dsc_serv : dscd_stat()      : blocking read waiting for socket termination\n");
fflush(stderr);
#endif

  /*                                              */
  /* blocking read waiting for socket termination */
  /*                                              */
/*
   read( s, msg,  1 );
*/
   close( s );

#ifdef DSDSTAT
fprintf(stderr,"dsc_serv : dscd_stat()      : returning void\n");
fflush(stderr);
#endif

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscs_pstat( msg, s)                                           */
/*                                                                           */
/* inputs:    msg - message buffer to use to send back data                  */
/*            s - socket to return data on                                   */
/*                                                                           */
/* outputs:  (messages to requesting problem)                                */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  send problem database to requestor                              */
/*                                                                           */
/*****************************************************************************/

int dscs_pstat(msg, s)
DSCPSTA *msg;
int s;
{
   int i;                         /* loop counter                            */
   int rstat;                     /* dscs_pstat() return status              */
   DSCDB *pp;                     /* problem data base entry                 */
   DSCDB *dscp_find();            /* find problem in prob dbase function     */
   NODE_REC *np;                  /* node record                             */
   NODE_REC *dscn_find();         /* find node in node dbase function        */

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_pstat()     : DSCPSTA *msg\n");
fprintf(stderr,"dsc_serv : dscs_pstat()     : msg->msg_type %d\n",
               ntohl(msg->msg_type));
fprintf(stderr,"dsc_serv : dscs_pstat()     : int s %d\n",s);
fflush(stderr);
#endif

   msg->sanity = msg->msg_type = htonl(MSG_PSTAT);
   strcpy( msg->dsc_password , password );
   msg->msg_size = htonl(sizeof( DSCPSTA ));
   msg->more = 'Y';

   /*                                  */
   /* LOOP FOR ALL MESSAGE IN DATABASE */
   /*                                  */

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_pstat()     : looping for all in prob dbase\n");
fflush(stderr);
#endif

   i = 0;
   do {
      pp = dscp_find( 2, &i);
      if (pp == (DSCDB *) EOF) continue;
      /*              */
      /* SEND MESSAGE */
      /*              */
      memcpy( (char *)&msg->prob, (char *)pp, sizeof( DSCDB ));

      np = dscn_find( 0, pp->enode_addr,0);

      if (np == (NODE_REC *)EOF)
	strcpy( msg->enname, my_node);
      else
	strcpy( msg->enname, np->nname);
      rstat = write( s, (char *)msg, sizeof(DSCPSTA));
    } while( pp != (DSCDB *) EOF); /* end for all problem records */

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_pstat()     : sending final message \n");
fflush(stderr);
#endif

   /*                    */
   /* SEND FINAL MESSAGE */
   /*                    */
   msg->more = 'N';
   rstat = write( s, (char *)msg, sizeof(DSCPSTA )); 

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_pstat()     : returning rstat %d\n",rstat);
fflush(stderr);
#endif
 
   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscs_nstat( msg, s)                                           */
/*                                                                           */
/* inputs:    msg - message buffer to use to send back data                  */
/*            s - socket to return data on                                   */
/*                                                                           */
/* outputs:  (messages to requesting problem)                                */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  send node database to requestor                                 */
/*                                                                           */
/*****************************************************************************/

int dscs_nstat(msg, s)
DSCNSTA *msg;
int s;
{
   int i;                         /* loop counter                            */
   int rstat;                     /* dscs_nstat() return status              */
   NODE_REC *np;                  /* node record                             */
   NODE_REC *dscn_find();         /* find node in node dbase function        */

#ifdef DSDNSTAT
fprintf(stderr,"dsc_serv : dscs_nstat()     : DSCNSTA *msg\n");
fprintf(stderr,"dsc_serv : dscs_nstat()     : msg->msg_type %d\n",
               ntohl(msg->msg_type));
fprintf(stderr,"dsc_serv : dscs_nstat()     : int s %d\n",s);
fflush(stderr);
#endif

   msg->sanity = msg->msg_type = htonl(MSG_NSTAT);
   strcpy( msg->dsc_password , password );
   msg->msg_size = htonl(sizeof( DSCNSTA ));
   msg->more = 'Y';

   /*                                  */
   /* LOOP FOR ALL MESSAGE IN DATABASE */
   /*                                  */

#ifdef DSDNSTAT
fprintf(stderr,"dsc_serv : dscs_nstat()     : looping for all in node dbase\n");
fflush(stderr);
#endif

   i = 0;
   do { 

      np=dscn_find( 2, &i,0);

      if (np == (NODE_REC *)(EOF)) continue;
      /*              */
      /* SEND MESSAGE */
      /*              */
      msg->node.index = htonl(np->index);
      msg->node.portg = np->portg;  /* network order already */
      msg->node.naddr = np->naddr;  /* network order already */
      strcpy( msg->node.nname, np->nname);
      msg->node.act_prob = htonl(np->act_prob);
      msg->node.blk_prob = htonl(np->blk_prob);
      msg->node.err_cnt = htonl(np->err_cnt);
      msg->node.fp = np->fp; /* network order does not matter here */
      msg->node.cpu = htonl(np->cpu) ;
      msg->node.mem = htonl(np->mem);
      msg->node.memrating = htonl(np->memrating);
      msg->node.cpurating = htonl(np->cpurating);
      msg->node.numproc = htonl(np->numproc);
      msg->node.fsystem = htonl(np->fsystem);
      rstat = write( s, (char *)msg, sizeof(DSCNSTA));
   } while (np != (NODE_REC *) EOF);  /* end for all problem records */

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_nstat()     : sending final message \n");
fflush(stderr);
#endif

   /*                    */
   /* SEND FINAL MESSAGE */
   /*                    */
   msg->more = 'N';
   rstat = write( s, (char *)msg, sizeof(DSCNSTA )); 

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_nstat()     : returning rstat %d\n",rstat);
fflush(stderr);
#endif
 
   return( rstat );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: dscs_sstat()                                                  */
/*                                                                           */
/* inputs:    msg - message buffer to use to send back data                  */
/*            s - socket to return data on                                   */
/*                                                                           */
/* outputs:  (messages to requesting problem)                                */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  send server status to requestor                                 */
/*                                                                           */
/*****************************************************************************/

int dscs_sstat(msg, s)
DSCSSTA *msg;
int s;
{

#ifdef DSDSSTAT
fprintf(stderr,"dsc_serv : dscs_sstat()     : DSCSSTA *msg\n");
fprintf(stderr,"dsc_serv : dscs_sstat()     : msg->msg_type %d\n",
               ntohl(msg->msg_type));
fprintf(stderr,"dsc_serv : dscs_sstat()     : int s %d\n",s);
fflush(stderr);
#endif

   msg->sanity = msg->msg_type = htonl(MSG_SSTAT);
   strcpy( msg->dsc_password , password );
   msg->msg_size = htonl(sizeof( DSCSSTA ));
   strcpy( msg->nname, my_node );
   msg->naddr = my_addr;              /* already network format */
   msg->act_prob = htonl( act_prob );
   msg->blk_prob = htonl( blk_prob );
   msg->waiting  = htonl( waiting_on_sub );
   msg->cpu = htonl( cpu );
   msg->mem = htonl( mem );
   msg->cpurating = htonl( cpurate );
   msg->memrating = htonl( memrate );
   msg->numproc = htonl( numproc );
   msg->fsystem = htonl( fsystem );

   /*                    */
   /* SEND FINAL MESSAGE */
   /*                    */

#ifdef DSDSSTAT
fprintf(stderr,"dsc_serv : dscs_sstat()     : sending final message \n");
fflush(stderr);
#endif

   write( s, (char *)msg, sizeof(DSCSSTA )); 

#ifdef DSDPSTAT
fprintf(stderr,"dsc_serv : dscs_nstat()     : returning  DSC_SUCCESS \n");
fflush(stderr);
#endif 

   return( DSC_SUCCESS );
}
