/*****************************************************************************/
/*                                                                           */
/*                                dscctrl.c                                  */
/*                                                                           */
/* Change History:                                                           */
/* ~~~~~~~~~~~~~~~~~                                                         */
/* (AAP) - Added C and Ci types for C++ applications.                        */
/* (AAP) - Added C++ status output.                                          */
/*                                                                           */
/*****************************************************************************/

#include "dscpr.c"        /* include server external variables               */

/*****************************************************************************/
/*                                                                           */
/* functions:                                                                */
/*      main()         - display data base and call appropriate function     */
/*      ctrl_init()    - setup sockets                                       */
/*      ctrl_start()   - start a subproblem                                  */
/*      ctrl_pstat()   - problem database status                             */
/*      ctrl_nstat()   - node database status                                */
/*      ctrl_sstat()   - server status                                       */
/*      ctrl_halt()    - orderly shutdown of local server                    */
/*      ctrl_kill()    - kill specific subproblem                            */
/*      ctrl_remote()  - start a DSC Server on single remote node            */
/*      ctrl_allup()   - start all DSC RP's configured                       */
/*      ctrl_alldn()   - stop all DSC RP's                                   */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* subroutine: main( argc, argv)                                             */
/*                                                                           */
/* inputs:   argc - number of command line arguments                         */
/*           argv - command line arguments                                   */
/*                                                                           */
/*           dscctrl nodename portnumber                                     */
/*                                                                           */
/* outputs:  (start problems, kill problems, ...)                            */
/*                                                                           */
/* return:   0 = DSC_SUCESS                                                  */
/*                                                                           */
/* purpose:  serv as an interface between the user and the DSC environment   */
/*                                                                           */
/*****************************************************************************/

int main( argc, argv)
int argc;
char **argv;
{
   int status; 
   int    ans;
   char   *getenv();

#ifdef DCTRLMAIN
fprintf(stderr,"dscctrl  : main             : int argc %d\n",argc);
fprintf(stderr,"dscctrl  : main             : char **argv %d\n");
for(ans=0;ans<argc;ans++)
fprintf(stderr,"dscctrl  : main             : argv[%d] %s\n",ans,argv[ans]);
ans=0;
fflush(stderr);
#endif

   fprintf( stderr, "Distributed Symbolic Computation environment\n");
   fprintf( stderr, "\t(c) 1990, 1991, 1992, 1993, 1994\n");
   fprintf( stderr, "\t(c) Kurt W. Schmitz, Angel L. Diaz, Erich Kaltofen\n");

   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"dscctrl  : main             : getting cwd\n");
     fprintf(stderr,"dscctrl  : main             : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }

   printf( "\tcurrent working directory  is %s\n", pwd);

#ifdef DCTRLMAIN
fprintf(stderr,"dscctrl  : main             : establishing network setup\n");
fflush(stderr);
#endif

  /*                         */
  /* establish network setup */
  /*                         */
  if ( ctrl_init(argc, argv) 		!= 1) goto XDONE;

#ifdef DCTRLMAIN
fprintf(stderr,"dscctrl  : main             : network setup acomplished!\n");
fflush(stderr);
#endif

  /*           */
  /* main loop */
  /*           */
   do {
      /*                 */
      /*  DISPLAY MENU   */
      /*                 */
      printf( "\n******* DSC Control Menu *******\n");
      printf( "\t1:     Start new problem\n");
      printf( "\t2:     Display Problem Database\n");
      printf( "\t3:     Display Node Database\n");
      printf( "\t4:     Display Server Statistics\n");
      printf( "\t5:     HALT Server on this node\n");
      printf( "\t6:     Kill a specific subproblem\n");
      printf( "\t7:     Start single remote Server\n");
      printf( "\t8:     HALT all configured Servers\n");
      printf( "\t9:     Start all configured Servers\n");
      
      printf( "\t99:    Exit this menu\n");
      printf( "\n\toption: ");

      /*                 */
      /*  GET SELECTION  */
      /*                 */
      scanf( "%2d", &ans);

      /*                   */
      /*  DISPATCH ACTION  */
      /*                   */
      switch ((int) ans) {
         case 1:
              status = ctrl_start( );
              break;
         case 2:
              status = ctrl_pstat();
             break;
         case 3:
              status = ctrl_nstat();
              break;
         case 4:
              status = ctrl_sstat();
              break;
         case 5:
              status = ctrl_halt();
              break;
         case 6:
              status = ctrl_kill();
              break;
         case 7:
              status = ctrl_remote();
              break;
         case 8:
              status = ctrl_alldn();
              break;
         case 9:
              status = ctrl_allup();
              break;
         case 99:
              status = dscpr_end( );
              printf( "\n\nExiting.\n");
              break;
         default:
              status = 0;
              printf( "\nInvalid selection, try again.\n");
              break;
         }		        		/* end case statement */
      if (status != DSC_SUCCESS) goto XDONE;	/* error check */
   } while (ans != 99);

XDONE:

#ifdef DCTRLMAIN
fprintf(stderr,"dscctrl  : main             : exit status %d\n",status);
fflush(stderr);
#endif

   exit(status);

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_init( argc, argv)                                        */
/*                                                                           */
/* inputs:   argc, argv - from command line                                  */
/*           argv[1]    - might contain target node name                     */
/*           argv[2]    - might contain port number                          */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCESS                                                  */
/*           n = status                                                      */
/*                                                                           */
/* purpose:  get listen socket and datagram socket                           */
/*           build address for listen socket and server                      */
/*                                                                           */
/*****************************************************************************/

int ctrl_init( argc, argv)
int argc;
char **argv;
{
   int status=1;
   struct hostent *hp, *gethostbyname();
 
#ifdef DCTRLINIT
fprintf(stderr,"dscctrl  : ctrl_init()      : int argc %d\n",argc);
fprintf(stderr,"dscctrl  : ctrl_init()      : char **argv %d\n");
for(status=0;status<argc;status++)
fprintf(stderr,"dscctrl  : ctrl_init()      : argv[%d] %s\n",status,
                argv[status]);
status=1;
fflush(stderr);
#endif

   /*                       */
   /* start executable code */
   /*                       */
   mypid = (-2);    /* must be -2, DSCRP can tell main problems this way */
   overide = FALSE; /* port number overide option                        */

   if (argc > 2) {
      overide = TRUE;
      cportnm = argv[2];
      if ( dscpr_isnum(cportnm) == FALSE ) {
fprintf(stderr,"dscctrl  : ctrl_init()      : invalid port number %s!\n",
        cportnm);
         status = dscpr_end();
         exit(status);
         }
      portnm = atoi(cportnm);
      }  

#ifdef DCTRLINIT
fprintf(stderr,"dscctrl  : ctrl_init()      : setting up assuming local server\n");
fflush(stderr);
#endif

  dscpr_netpnm();       /* this sets up everything assuming local server */

  fprintf(stderr,"\tusing port number %d \n",portnm);

  /*                             */
  /* get local process info here */
  /*                             */
   if (argc > 1) {
      hp = gethostbyname(argv[1]);
      if (hp == 0) 
       {
fprintf( stderr,
"dscctrl  : ctrl_init()      : %s not a valid node name, ignored!\n",argv[1]);
fflush(stderr);
       }
      else  
       {

         bcopy( hp->h_addr, &saddr.sin_addr, hp->h_length);

#ifdef DCTRLINIT
fprintf(stderr,"dscctrl  : ctrl_init()      : %s %lu.%lu.%lu.%lu\n",
          hp->h_name,
          ((unsigned long) (*(long *)hp->h_addr)&0xff000000) >> 24,
          ((unsigned long) (*(long *)hp->h_addr)&0x00ff0000) >> 16,
          ((unsigned long) (*(long *)hp->h_addr)&0x0000ff00) >>  8,
          ((unsigned long) (*(long *)hp->h_addr)&0x000000ff));
fflush(stderr);
#endif

       }
      } /* end if node name specified */

#ifdef DCTRLINIT
fprintf(stderr,"dscctrl  : ctrl_init()      : returning status %d\n",status);
fflush(stderr);
#endif

   return( status);
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_start( )                                                 */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   1 = success                                                     */
/*           n = size of message sent/received but did not match             */
/*                                                                           */
/* purpose:  start a subproblem                                              */
/*                                                                           */
/*****************************************************************************/

int ctrl_start(  )
{
   int status=DSC_SUCCESS;      /* return status of ctrl_start()             */
   int estcpu;                  /* estimated cpu usgae of main problem       */
   int estmem;                  /* estimated mem usage of main problem       */
   char estcpus[8],estmems[9];
   char probname[32+1];         /* problem name                              */
   char probtype[8];            /* problem type                              */
   char build_s[DSC_CSHZ+1];    /* build command                             */
   char exec_s[DSC_CSHZ+1];     /* execute command                           */
   char file_s[DSC_FILEZ+1];    /* source file name (no extention)           */

#ifdef DCTRLSTART
fprintf(stderr,"dscctrl  : ctrl_start()     : void\n");
fflush(stderr);
#endif

   gets( probname);           /* get last newline */

   /*                    */
   /*  GET PROBLEM NAME  */
   /*                    */
   printf( "Enter New Problem name: ");
   gets(probname);

   /*                        */
   /*  GET Source FILE NAME  */
   /*                        */
   printf( "Enter Source File Name (no extention): ");
   gets(file_s);

   /*                        */
   /*  GET source file type  */
   /*                        */

printf("\n*****************************************************************************\n");
printf("* Type * Description                                                        *\n");
printf("*****************************************************************************\n");
printf("* c    * C source code                                                      *\n");
printf("* C    * C++ source code                                                    *\n");
printf("* l    * Lisp source code                                                   *\n");
printf("* lc   * Lisp source code with remote node compilation                      *\n");
printf("* lo   * Lisp object code                                                   *\n");
printf("* p    * Maple source code                                                  *\n");
printf("* m    * Mathematica source code (Not fully implemented)                    *\n");
printf("* i    * Input file                                                         *\n");
printf("* ci   * C source code with an input file                                   *\n");
printf("* Ci   * C++ source code with an input file                                 *\n");
printf("* mi   * Mathematics source code with an input file (Not fully implemented) * \n");
printf("* li   * Lisp source code with an input file                                *\n");
printf("* lci  * Lisp source code with remote node compilation with an input file   * \n");
printf("* loi  * Lisp object code with an input file                                *\n");
printf("* pi   * Maple source code with an input file                               *\n");
printf("*****************************************************************************\n\n");
printf("Enter file type (c/C/l/lc/lo/p/m/i/ci/Ci/mi/li/lci/loi/pi) [c]: ");
  
   gets(probtype);
   if (strlen(probtype)==0) probtype[0] = DSC_C;  /* default problem type */

   /*                           */
   /*  GET BUILD instructions   */
   /*                           */

   if ( (probtype[0] != 'l' ) && (probtype[0] != 'm') && (probtype[0] != 'p') ) 
     {
     printf( "do you have special build instructions ? ");
     gets( build_s);
     if ((build_s[0]== 'y')|| (build_s[0]=='Y'))  {
       printf( "Enter Build instructions: ");
       gets( build_s);
     }
     else
        build_s[0] = 0;
    }
   else if (probtype[0]=='l') 
      strcpy(build_s,"dsc_lisp.build");
   else if (probtype[0]=='m')
      strcpy(build_s,"dsc_math.build");
   else if (probtype[0]=='p')
      strcpy(build_s,"dsc_maple.build");

   /*                             */
   /*  GET Execute instructions   */
   /*                             */
   printf( "do you have special execute instructions ? ");
   gets( exec_s);
   if ((exec_s[0] == 'y')||(exec_s[0] == 'Y'))  {
     printf( "Enter execute instructions: ");
     gets( exec_s);
     }
   else
     exec_s[0] = 0;

   printf( "Enter estimated CPU usage 1)HIGH 2)MED 3)LOW [HIGH]: ");
   gets( estcpus ); estcpu = atoi(estcpus);
   if ( estcpu == 1 ) estcpu = HIGH_CPU;
   else
   if ( estcpu == 2 ) estcpu = MED_CPU;
   else
   if ( estcpu == 3 ) estcpu = LOW_CPU;
   else estcpu = HIGH_CPU;
      
   printf( "Enter estimated MEM usage (megs) [5]: ");
   gets( estmems ); estmem = atoi(estmems);
   if ( estmem == 0 ) estmem = 5;

#ifdef DCTRLSTART
fprintf(stderr,"dscctrl  : ctrl_start()     : dispatching new problem\n");
fprintf(stderr,"dscctrl  : ctrl_start()     : probname %s\n",probname);
fprintf(stderr,"dscctrl  : ctrl_start()     : file_s %s\n",file_s);
fprintf(stderr,"dscctrl  : ctrl_start()     : probtype %s\n",probtype);
fprintf(stderr,"dscctrl  : ctrl_start()     : build_s %s\n",build_s);
fprintf(stderr,"dscctrl  : ctrl_start()     : exec_s %s\n",exec_s);
fprintf(stderr,"dscctrl  : ctrl_start()     : estcpu %d\n",estcpu);
fprintf(stderr,"dscctrl  : ctrl_start()     : estmem %d\n",estmem);
fflush(stderr);
#endif

   /*                        */
   /*  DISPATCH New Problem  */
   /*                        */
   status = ctrl_dispatch( probname, file_s, probtype, build_s, exec_s,
                           0, estcpu, estmem);

#ifdef DCTRLSTART
fprintf(stderr,"dscctrl  : ctrl_start()     : returning status %d\n",status);
fflush(stderr);
#endif

   return(status);
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_pstat()                                                  */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose: display problem database status                                  */
/*                                                                           */
/*****************************************************************************/

int ctrl_pstat( )
{
   DSCQSTA	msg;      /* status request message                          */
   DSCPSTA      msg2;     /* problem status message                          */
   int          status;   
   int          on;       
   char         ans[12];  /* user input read string                          */

#ifdef DCTRLPSTAT
fprintf(stderr,"dscctrl  : ctrl_start()     : void\n");
fflush(stderr);
#endif

   /*                         */
   /*  build Status Req. Msg  */
   /*                         */
   msg.msg_type = msg.sanity = htonl(MSG_QSTAT);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl(sizeof( DSCQSTA ));
   msg.flag  = htonl(MSG_PSTAT);
   memcpy( &msg.addr, &paddr, sizeof(paddr));

#ifdef DCTRLPSTAT
fprintf(stderr,"dscctrl  : ctrl_start()     : sending message to local RP \n");
fflush(stderr);
#endif

   /*                                */
   /*  SEND MESSAGE TO LOCAL DSC RP  */
   /*                                */
   do {
     errno = 0;
     status = sendto( sockdg, &msg, sizeof(DSCQSTA),
		     0, &saddr, sizeof(saddr));
     if (errno == ENOBUFS) sleep(2);
   } while( (errno==EINTR) || (errno==ENOBUFS));

   gets( ans );                        /* eat up trailing line feed */
   sock = accept( sockl, 0, 0);
   on = 1;
   setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));

   printf( "\n P R O B L E M   D A T A B A S E\n");

#ifdef DCTRLPSTAT
fprintf(stderr,"dscctrl  : ctrl_start()     : looping on responses\n");
fflush(stderr);
#endif

   /*                         */
   /*  loop on responses      */
   /*                         */
   do {
      status = dsc_read( sock, (char *)&msg2, sizeof(DSCPSTA));
      if (status != ntohl(msg2.msg_size)) break;
      status = DSC_SUCCESS;		/* set success status */
      if (msg2.more != 'Y') break;
      if ((ans[0]|0x20) == 'q') continue;

      /*                         */
      /*  display each record    */
      /*                         */
      if (ntohl(msg2.prob.depth) == (-1)) continue;
      printf( "\n problem name: %s\tindex %3d\n"
             ,msg2.prob.prob_name,ntohl(msg2.prob.index));
     printf(
      " problem  PID: %6ld\tnode : %s %lu.%lu.%lu.%lu \tCPU %2d\n",
	     ntohl(msg2.prob.prob_PID), msg2.enname, 
	     (msg2.prob.enode_addr & 0xff000000) >> 24,
	     (msg2.prob.enode_addr & 0x00ff0000) >> 16,
	     (msg2.prob.enode_addr & 0x0000ff00) >>  8,
              msg2.prob.enode_addr & 0x000000ff,
	     ntohl(msg2.prob.cpu_type));

     printf( " parent   PID: %6ld\tnaddr: %lu.%lu.%lu.%lu\tindex %3d\n",
	     ntohl(msg2.prob.parent_PID),
	     (msg2.prob.pnode_addr&0xff000000) >> 24,
	     (msg2.prob.pnode_addr&0x00ff0000) >> 16,
	     (msg2.prob.pnode_addr&0x0000ff00) >>  8,
	     msg2.prob.pnode_addr&0x000000ff,
	     ntohl(msg2.prob.parent_idx));
      printf( " source  name: %s\n", msg2.prob.src_name);
      printf( "            c: %1d  C++: %1d  lisp: %1d  math: %1d maple: %1d obj: %1d  input:%1d\n", 
            msg2.prob.ctyp,
            msg2.prob.Ctyp,
            msg2.prob.ltyp,
            msg2.prob.mtyp,
            msg2.prob.ptyp,
            msg2.prob.otyp,
            msg2.prob.ityp);
      printf( " build command: %s\n", msg2.prob.src_build);
      printf( " exec command: %s\n", msg2.prob.src_exec);
      printf( " blocked sock: %3d\t block condition %ld\n",
	     ntohl(msg2.prob.block_sok),
	     ntohl(msg2.prob.block_sub));
      printf( " completion  : %3d\tstatus: %08x build: %3d\n",
	     ntohl(msg2.prob.compl_flg),
	     ntohl(msg2.prob.compl_stat),
	     ntohl(msg2.prob.build_flg));
      printf( " peer link   : %3d\tchild list: %4d\t # subprob: %3d\n",
	     ntohl(msg2.prob.peer_next),
	     ntohl(msg2.prob.child_next),
	     ntohl(msg2.prob.num_sub));
      /* 0 awake ; 1 sleeping */
      printf(" state       : %3d\n",ntohl(msg2.prob.state));
      if ( ntohl(msg2.prob.cpurating) == HIGH_CPU ) 
       printf(" estimated cpu use : HIGH_CPU \t");
      else
      if ( ntohl(msg2.prob.cpurating) == MED_CPU )
       printf(" estimated cpu use : MED_CPU \t");
      else
      if ( ntohl(msg2.prob.cpurating) == LOW_CPU )
       printf(" estimated cpu use : LOW_CPU \t");
      else
       printf(" estimated cpu use : %d (UNKOWN VALUE)\n"
              ,ntohl(msg2.prob.cpurating));

      printf(" estimated mem need (megs) : %d\n",ntohl(msg2.prob.memrating));

     
      printf( "\nMORE ");
      gets( ans );
   } while (msg2.more == 'Y');

   printf( "======== end of database ! ==========\n");

   /*                                */
   /*  CLOSE THE SOCKET              */
   /*                                */
   close( sock );
   sock = -1;

#ifdef DCTRLPSTAT
fprintf(stderr,"dscctrl  : ctrl_start()     : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_nstat()                                                  */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose: display node database status                                     */
/*                                                                           */
/*****************************************************************************/

int ctrl_nstat( )
{
   DSCQSTA	msg;      /* status request message                          */
   DSCNSTA      msg2;     /* node status message                             */
   int          status;
   int          on, cnt=0;
   char         ans[12];  /* user input read string                          */

#ifdef DCTRLNSTAT
fprintf(stderr,"dscctrl  : ctrl_nstat()     : void\n");
fflush(stderr);
#endif

   /*                         */
   /*  build Status Req. Msg  */
   /*                         */
   msg.msg_type = msg.sanity = htonl(MSG_QSTAT);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl(sizeof( DSCQSTA ));
   memcpy( &msg.addr, &paddr, sizeof(paddr));
   msg.flag = htonl(MSG_NSTAT);

#ifdef DCTRLNSTAT
fprintf(stderr,"dscctrl  : ctrl_nstat()     : sending message to local RP \n");
fflush(stderr);
#endif

   /*                                */
   /*  SEND MESSAGE TO LOCAL DSC RP  */
   /*                                */
   do {
     errno = 0;
     status = sendto( sockdg, &msg, sizeof(DSCQSTA),
		     0, &saddr, sizeof(saddr));
     if (errno == ENOBUFS) sleep(2);
   } while( (errno==EINTR) || (errno==ENOBUFS));


   gets( ans );                        /* eat up trailing line feed */
   sock = accept( sockl, 0, 0);
   on = 1;
   setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));

   printf( "\n N O D E    D A T A B A S E\n");

#ifdef DCTRLNSTAT
fprintf(stderr,"dscctrl  : ctrl_nstat()     : looping on responses\n");
fflush(stderr);
#endif

   /*                         */
   /*  loop on responses      */
   /*                         */
   do {
      status = dsc_read( sock, (char *)&msg2, sizeof(DSCNSTA));
      if (status != ntohl(msg2.msg_size)) break;
      status = DSC_SUCCESS;		/* set success status */
      if (msg2.more != 'Y') break;
      if ((ans[0]|0x20) == 'q') continue;

      /*                         */
      /*  display each record    */
      /*                         */
      if (ntohl(msg2.node.index) == (-1)) continue;
      printf( "\n       node name: %s\n", msg2.node.nname);
      printf( "           index: %lu\n", ntohl(msg2.node.index));
      if (ntohl(msg2.node.act_prob) < 0)
	printf( "\t**** node out of service ****\n");
      else {
        printf( "active problems      : %ld\n", ntohl(msg2.node.act_prob));
        printf( "blocked problems     : %ld\n", ntohl(msg2.node.blk_prob));
        printf( "cpu load indicator   : %ld\n", ntohl(msg2.node.cpu));
        printf( "mem usage indicator  : %ld\n", ntohl(msg2.node.mem));     
        printf( "cpu mips rating      : %ld\n", ntohl(msg2.node.cpurating));
        printf( "mem capacity         : %ld\n", ntohl(msg2.node.memrating));
        printf( "number of processors : %ld\n", ntohl(msg2.node.numproc));
        printf( "file system number   : %ld\n", ntohl(msg2.node.fsystem));
        }
      
      if (cnt >= 3) {
         printf( "\nMORE ");
         gets( ans );
	 cnt = 0;
       }
      else cnt++;
   } while (msg2.more == 'Y');


   printf( "\n end of database ! \n");

   /*                                */
   /*  CLOSE THE SOCKET              */
   /*                                */
   close( sock );
   sock = -1;

#ifdef DCTRLNSTAT
fprintf(stderr,"dscctrl  : ctrl_nstat()     : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_sstat()                                                  */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  ask for server statistics, print them out when msg received.    */
/*                                                                           */
/*****************************************************************************/

int ctrl_sstat( )
{
   DSCQSTA	msg;      /* status request message                          */
   DSCSSTA      msg2;     /* server status message                           */
   int          status;
   int          on;

#ifdef DCTRLSSTAT
fprintf(stderr,"dscctrl  : ctrl_sstat()     : void\n");
fflush(stderr);
#endif

   /*                         */
   /*  build Status Req. Msg  */
   /*                         */
   msg.msg_type = msg.sanity = htonl(MSG_QSTAT);
   strcpy(msg.dsc_password , password ); 
   msg.msg_size = htonl(sizeof( DSCQSTA ));
   msg.flag =  htonl(MSG_SSTAT);
   memcpy( &msg.addr, &paddr, sizeof(paddr));

#ifdef DCTRLSSTAT
fprintf(stderr,"dscctrl  : ctrl_sstat()     : sending message to local RP \n");
fflush(stderr);
#endif

   /*                                */
   /*  SEND MESSAGE TO LOCAL DSC RP  */
   /*                                */
   do {
     errno = 0;
     status = sendto( sockdg, &msg, sizeof(DSCQSTA),
		     0, &saddr, sizeof(saddr));
     if (errno == ENOBUFS) sleep(2);
   } while( (errno==EINTR) || (errno==ENOBUFS));
   sock = accept( sockl, 0, 0);
   on = 1;
   setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));

   printf( "\n S E R V E R   S T A T I S T I C S\n");

#ifdef DCTRLSSTAT
fprintf(stderr,"dscctrl  : ctrl_sstat()     : display responses\n");
fflush(stderr);
#endif

      /*                         */
      /*  display responses      */
      /*                         */
      status = dsc_read( sock, (char *)&msg2, sizeof(DSCSSTA));
      if (status == ntohl(msg2.msg_size)) {
         status = DSC_SUCCESS;		/* set success status */

        /*                         */
        /*  display server status  */
        /*                         */
         printf( "\nnode name          : %s\n", msg2.nname);
         printf( "active problems      : %ld\n", ntohl(msg2.act_prob));
         printf( "blocked problems     : %ld\n", ntohl(msg2.blk_prob));
         printf( "queued problems      : %ld\n", ntohl(msg2.waiting));
         printf( "cpu load indicaotr   : %ld\n", ntohl(msg2.cpu));  
         printf( "mem usage indicator  : %ld\n", ntohl(msg2.mem));
         printf( "cpu mips ratting     : %ld\n", ntohl(msg2.cpurating));
         printf( "mem capacity         : %ld\n", ntohl(msg2.memrating));
         printf( "number of processors : %ld\n", ntohl(msg2.numproc));
         printf( "file system number   : %ld\n", ntohl(msg2.fsystem));
        }

      /*                                */
      /*  CLOSE THE SOCKET              */
      /*                                */
   close( sock );
   sock = -1;

#ifdef DCTRLSSTAT
fprintf(stderr,"dscctrl  : ctrl_sstat()     : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_halt()                                                   */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*           n = error number                                                */
/*                                                                           */
/* purpose:  halt local DSC RP                                               */
/*                                                                           */
/*****************************************************************************/

int ctrl_halt( )
{

   DSCHALT msg;   /* server halt message                                     */
  
#ifdef DCTRLHALT
fprintf(stderr,"dscctrl  : ctrl_halt()      : void\n");
fflush(stderr);
#endif

   /*                         */
   /*  Build HALT message     */
   /*                         */
   msg.msg_type = msg.sanity = htonl(MSG_HALT);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl( sizeof( DSCHALT ));
   msg.from_PID = htonl(mypid);

#ifdef DCTRLHALT
fprintf(stderr,"dscctrl  : ctrl_halt()      : sending message to local RP \n");
fflush(stderr);
#endif

   /*                                */
   /*  SEND MESSAGE TO LOCAL DSC RP  */
   /*                                */
   do {
     errno = 0;
     sendto( sockdg, &msg, sizeof(DSCHALT),
	    0, &saddr, sizeof(saddr));
     if (errno == ENOBUFS) sleep(2);
   } while( (errno==EINTR) || (errno==ENOBUFS));
   if (errno != 0) 
    {
      perror("sending to resident process");
fprintf(stderr,"dscctrl  : ctrl_halt()      : error sending halt to DSC RP\n");
fprintf(stderr,"dscctrl  : ctrl_halt()      : errno %d\n",errno);
fflush(stderr);
    }

#ifdef DCTRLHALT
fprintf(stderr,"dscctrl  : ctrl_halt()      : returning errno %d\n",errno);
fflush(stderr);
#endif

   return( errno );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_kill()                                                   */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*           n = error number                                                */
/*                                                                           */
/* purpose:  get (sub)problem index                                          */
/*           kill a specified (sub)problem                                   */
/*                                                                           */
/*****************************************************************************/

int ctrl_kill( )
{
   long i;         /* index of problem to kill                               */
   DSCKILL msg;    /* kill problem message                                   */
  
#ifdef DCTRLKILL
fprintf(stderr,"dscctrl  : ctrl_kill()      : void\n");
fflush(stderr);
#endif
 
   /*                         */
   /*  Build KILL message     */
   /*                         */
   msg.msg_type = msg.sanity = htonl(MSG_KILL);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl( sizeof( DSCKILL ));
   msg.from_PID = htonl(-1);                    /* this identifies DSCCTRL */

   printf( "\n\tEnter index of problem to kill:");
   scanf( "%d", &i);
   msg.index = htonl(i);

#ifdef DCTRLKILL
fprintf(stderr,"dscctrl  : ctrl_kill()      : sending message to local RP \n");
fflush(stderr);
#endif

   /*                                */
   /*  SEND MESSAGE TO LOCAL DSC RP  */
   /*                                */
   do {
     errno = 0;
     sendto( sockdg, &msg, sizeof(DSCKILL), 0, &saddr, sizeof(saddr));
     if (errno == ENOBUFS) sleep(2);
   } while( (errno==EINTR) || (errno==ENOBUFS));
   if (errno != 0) 
    {
      perror("sending to resident process");
fprintf(stderr,"dscctrl  : ctrl_kill()      : error sending kill to DSC RP\n");
fprintf(stderr,"dscctrl  : ctrl_kill()      : errno %d\n",errno);
fflush(stderr);
    }

#ifdef DCTRLKILL
fprintf(stderr,"dscctrl  : ctrl_kill()      : returning errno %d\n",errno);
fflush(stderr);
#endif

   return( errno );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_remote()                                                 */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  start a DSC Server on remote node                               */
/*                                                                           */
/*****************************************************************************/

int ctrl_remote( )
{
   char node_name[64];      /* full node name                                */
   char temp[64];           /* stripped version of node name                 */
   char *cptr;
   char *strchr();
   int pid;
   union wait statusp;
   static  char *argv[10];  /* rsh command                                   */

#ifdef DCTRLREMOTE
fprintf(stderr,"dscctrl  : ctrl_remote()    : void\n");
fflush(stderr);
#endif

   printf( "\n\tRemote node name: ");
   scanf( "%s", node_name );

#ifdef DCTRLREMOTE
fprintf(stderr,"dscctrl  : ctrl_remote()    : forking remote shell\n");
fflush(stderr);
#endif

   pid = fork();
   if (pid == 0) {
     strcpy( temp, node_name);
     cptr = strchr( temp, '.');
     if (cptr != 0) *cptr = 0;

     /* Convert portnm into character form */
     dscpr_itoa(portnm,cportnm);

     argv[0] = "rsh";
     argv[1] = node_name;
     argv[2] = "csh";
     argv[3] = "dsc_serv.csh";
     argv[4] = node_name; 
     argv[5] = cportnm;
     argv[6] = 0;

     pid = execvp( "rsh", argv);
     perror( "failure in execvp");
     fprintf(stderr,"dscctrl  : ctrl_remote()    : failure in rsh execvp\n");
     fprintf(stderr,"dscctrl  : ctrl_remote()    : errno %d\n",errno);
     fflush(stderr);
     exit(errno);
   }
   else {
     printf( "starting DSC Server on %s\n", node_name);
     wait(&statusp);
     if (statusp.w_retcode)
       printf( "remote startup failed\n");
     else if (statusp.w_status)
       printf( "remote startup failed\n");
     else if (statusp.w_coredump)
       printf( "remote startup failed (coredump)\n");
   }

#ifdef DCTRLREMOTE
fprintf(stderr,"dscctrl  : ctrl_remote()    : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_allup()                                                  */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  open $DSC_PATH/dscrp_net.lis and start all servers in the list  */
/*                                                                           */
/*****************************************************************************/

int ctrl_allup() 
{ 
   int pid;
   FILE *fp;                /* file pointer to DSCRP_NETFILE                 */
   char *cptr;
   char buffer[125];        /* line in DSCRP_NETFILE                         */
   char temp[125];          /* stripped version of node name                 */
   union wait statusp;
   static char *argv[10];

#ifdef DCTRLALLUP
fprintf(stderr,"dscctrl  : ctrl_allup()     : void\n");
fflush(stderr);
#endif

   /*                     */
   /* start mainline code */
   /*                     */
   fp = fopen( DSCRP_NETFILE, "r");
   if (fp == NULL) {
       perror( DSCRP_NETFILE );
fprintf(stderr,"dscctrl  : ctrl_allup()     : can not open %s\n",DSCRP_NETFILE);
fprintf(stderr,"dscctrl  : ctrl_allup()     : errno %d\n",errno);
fflush(stderr);
       goto XDONE;
      }   /* end if config file not found */

   /*               */
   /* for all nodes */
   /*               */

#ifdef DCTRLALLUP
fprintf(stderr,"dscctrl  : ctrl_allup()     : starting all nodes in %s\n",
        DSCRP_NETFILE);
fflush(stderr);
#endif

   while (!feof(fp)) {
     cptr = &buffer[0];
     buffer[0] = '\0';

     fgets(buffer, DSC_NODEZ, fp);
     if (strlen( buffer) == 0) break;
     if (*cptr == '*') continue ;            /* skip comment lines */

     cptr = strchr( buffer, '\n');
     if (cptr != 0)
       *cptr = 0;
     cptr = strchr( buffer, '|');
     if (cptr != 0)
       *cptr = 0;
     if (buffer[0] == '\0') continue;        /* empty record */


     pid = fork();
     if (pid == 0) {
       strcpy( temp, buffer);
       /* Next line strips the string */
       /* cptr = strchr( temp, '.'); */
       if (cptr != 0) *cptr = 0;
       /* Convert portnm into character form */
       dscpr_itoa(portnm,cportnm);

       if (strcmp( buffer, mynode) == 0) {
          argv[0] = buffer;
          argv[1] = "csh";
          argv[2] = "dsc_serv.csh";
          argv[3] = temp; 
          argv[4] = cportnm;
          argv[5] = 0;
       }
       else {
          argv[0] = "rsh";
          argv[1] = buffer;
          argv[2] = "csh";
          argv[3] = "dsc_serv.csh";
          argv[4] = temp; 
          argv[5] = cportnm;
          argv[6] = 0;
	}
       pid = execvp( "rsh", argv);
       perror( "failure in execvp");
fprintf(stderr,"dscctrl  : ctrl_allup()     : failure in rsh execvp\n");
fprintf(stderr,"dscctrl  : ctrl_allup()     : errno %d\n",errno);
fflush(stderr);
       exit(errno);
     }
     else {
       printf( "starting DSC Server on %s\n", buffer);
       wait(&statusp);
       if (statusp.w_retcode)
         printf( "remote startup failed\n");
       else if (statusp.w_status)
         printf( "remote startup failed\n");
       else if (statusp.w_coredump)
         printf( "remote startup failed (coredump)\n");
     }/* else if parent process */
   }/* while more records in file */

XDONE:

#ifdef DCTRLALLUP
fprintf(stderr,"dscctrl  : ctrl_allup()     : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: ctrl_alldn()                                                  */
/*                                                                           */
/* inputs:   none                                                            */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = success                                                     */
/*                                                                           */
/* purpose:  open $DSC_PATH/dscrp_net.lis and halt all servers in the list   */
/*                                                                           */
/*****************************************************************************/

int ctrl_alldn() 
{ 
   FILE *fp;               /* file pointer to DSCRP_NETFILE                 */
   char *cptr;       
   char buffer[125];       /* line in DSCRP_NETFILE                         */
   char *strchr();
   struct hostent *hp, *gethostbyname();
   struct sockaddr_in raddr;
   DSCHALT msg;            /* server halt message                           */

#ifdef DCTRLALLDN
fprintf(stderr,"dscctrl  : ctrl_alldn()     : void\n");
fflush(stderr);
#endif

   /*                     */
   /* start mainline code */
   /*                     */
   raddr.sin_family = AF_INET;
   raddr.sin_port = htons(portnm);

   msg.msg_type = msg.sanity = htonl(MSG_HALT);
   strcpy(msg.dsc_password , password );
   msg.msg_size = htonl( sizeof( DSCHALT ));
   msg.from_PID = htonl(mypid);

   fp = fopen( DSCRP_NETFILE, "r");
   if (fp == NULL) {
       perror( DSCRP_NETFILE );
fprintf(stderr,"dscctrl  : ctrl_alldn()     : can not open %s\n",DSCRP_NETFILE);
fprintf(stderr,"dscctrl  : ctrl_alldn()     : errno %d\n",errno);
fflush(stderr);
       goto XDONE;
      }   /* end if config file not found */

   /*               */
   /* for all nodes */
   /*               */

#ifdef DCTRLALLDN
fprintf(stderr,"dscctrl  : ctrl_alldn()     : sending halt to all nodes in %s\n"
              ,DSCRP_NETFILE);
fflush(stderr);
#endif

   while (!feof(fp)) {
     cptr = &buffer[0];
     buffer[0] = '\0';

     fgets(buffer, DSC_NODEZ, fp);
     if (strlen( buffer) == 0) break;
     if (*cptr == '*') continue ;            /* skip comment lines */

     cptr = strchr( buffer, '\n');
     if (cptr != 0)
       *cptr = 0;
     cptr = strchr( buffer, '|');
     if (cptr != 0)
       *cptr = 0;
     if (buffer[0] == '\0') continue;        /* empty record */


     hp = gethostbyname(buffer);
     if (hp == 0) 
       fprintf( stderr,
          "configured node %s is not known\n", buffer);
     else  bcopy( hp->h_addr, &raddr.sin_addr, hp->h_length);

     do {
       errno = 0;
       sendto( sockdg, &msg, sizeof(DSCHALT), 0,
	      raddr, sizeof(raddr));
       if (errno == ENOBUFS) sleep(2);
     } while((errno ==EINTR) || (errno==ENOBUFS));
     if (errno != 0)
       perror( "sending HALT to remote DSC server");
     
   }/* while more records in file */

 XDONE:

#ifdef DCTRLALLDN
fprintf(stderr,"dscctrl  : ctrl_alldn()     : returning DSC_SUCCESS\n");
fflush(stderr);
#endif

   return( DSC_SUCCESS );
}
