/*****************************************************************************/
/*                                                                           */
/*                                dscdmsg.c                                  */
/*                                                                           */
/*****************************************************************************/

#include "dscrp.h"                      /* dsc_serv external variables       */

/*****************************************************************************/
/*                                                                           */
/* subroutine: dsc_do_msg( msg )                                             */
/*                                                                           */
/* inputs:   msg - message to be processed                                   */
/*                                                                           */
/* outputs:  none                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  dispatch proper function to handle this message                 */
/*                                                                           */
/*****************************************************************************/

int dsc_do_msg( msg )
DSCSTRT *msg;
{
   int dsc_chkpass();       /* string compare passwords      */
   int rstat=DSC_SUCCESS;   /* status of dsc_do_msg          */
   long mtype;              /* message type                  */
   int  s;                  /* stream socket                 */

   DSCKIN  *msgcin;         /* Node checkin message          */
   DSCSTRT *msgsrt;         /* Start subproblem message      */
   DSCSOLV *msgsol;         /* (Sub)Problem solution message */
   DSCHALT *msghalt;        /* Halt a (sub)problem message   */
   DSCQSTA *msgstat;        /* Request status message        */
   DSCPRBLK *msgpbk;        /* Problem blocking message      */
   DSCCPYCTRL *msgcpy;      /* Copy request/ack message      */
   DSCKILL *msgkill;        /* Problem terminating message   */
   DSCSLP *msgslp;          /* Subprobelm sleeping message   */
   DSCAWK *msgawk;          /* Awaken subproblem message     */
   DSCPSSTAT *msgpsstat;    /* DSC PS status message         */
   DSCPSRSTAT *msgpsrstat;  /* Node status message           */

   mtype = ntohl( msg->msg_type ); 

#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : DSCSTRT *msg\n");
fprintf(stderr,"dsc_serv : dsc_do_msg       : msg type %ld\n",mtype);
#endif

   switch( mtype ) {

      /*                               */
      /*     Node checkin message      */
      /*                               */
      case MSG_CHECKIN:

#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * CHECKIN \n");
fflush( stderr );
#endif

           msgcin = (DSCKIN *)msg;
           if ( dsc_chkpass( password, msgcin->dsc_password) == DSC_SUCCESS )
            rstat = dscd_chkin( msg );
           break;

      /*                               */
      /* Local DSC PS status message   */
      /*                               */
      case MSG_PSSTAT:

#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * PSSTAT (local)\n");
fflush( stderr );
#endif
           msgpsstat = (DSCPSSTAT *)msg;
           rstat = dscd_psstat( msg );
           break;

      /*                               */
      /* Remote DSC PS status message  */
      /*                               */
      case MSG_PSRSTAT:

#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * PSRSTAT \n");
fflush( stderr );
#endif

           msgpsrstat = (DSCPSRSTAT *)msg;
           rstat = dscd_psrstat( msg );
           break;

      /*                               */
      /*   Start subproblem message    */
      /*                               */
      case MSG_START:

#ifdef DSDMSG 
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * START (sub)PROBLEM \n");
fflush( stderr );
#endif

	   /* !!!!! if start is from remote node, dont get socket !!!!! */
           msgsrt = (DSCSTRT *)msg;
           if ( dsc_chkpass( password, msgsrt->dsc_password) == DSC_SUCCESS )
            {
             if (msg->fnode == my_addr) 
               {
                s = dscn_sock(msg);
#ifdef DSDMSG 
fprintf(stderr,"dsc_serv : dsc_do_msg       : local start message\n");
fprintf(stderr,"dsc_serv : dsc_do_msg       : socket %d\n",s);
#endif
               }
              else 
               {
                s = EOF;
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : remote start message\n");
fprintf(stderr,"dsc_serv : dsc_do_msg       : socket %d\n",s);
#endif
               } 
              rstat = dscd_start( msg, s );
            }
           break;

      /*                                      */
      /*   Put subproblem to sleep message    */
      /*                                      */
      case MSG_SLEEPING:

#ifdef  DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * SLEEPING SUBPROBLEM \n");
fflush( stderr );
#endif
 
           msgslp = (DSCSLP *)msg;
           if ( dsc_chkpass( password, msgslp->dsc_password) == DSC_SUCCESS )
            {
             rstat = dscd_sleep( msg );
            }
           break;

      /*                                      */
      /*   Awaken slept subproblem message    */
      /*                                      */
      case MSG_AWAKEN:

#ifdef  DSDMSG 
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * AWAKEN SUBPROBLEM \n");
fflush( stderr );
#endif

	   /* !!!!! if start is from remote node, dont get socket !!!!! */
           msgawk = (DSCAWK *)msg;
           if ( dsc_chkpass( password, msgawk->dsc_password) == DSC_SUCCESS )
            {
             if (msg->fnode == my_addr) 
              {
                s = dscn_sock(msg);
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : local awaken message\n");
fprintf(stderr,"dsc_serv : dsc_do_msg       : socket %d\n",s);
#endif
              }
              else 
              {
                s = EOF;
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : remote awaken message\n");
fprintf(stderr,"dsc_serv : dsc_do_msg       : socket %d\n",s);
#endif
              }
              rstat = dscd_awaken( msg, s );
            }
          break;
            
      /*                               */
      /* (Sub)Problem solution message */
      /*                               */
      case MSG_SOLVE:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * SOLUTION has arrived \n");
fflush( stderr );
#endif
           msgsol = (DSCSOLV *)msg;
           if ( dsc_chkpass( password, msgsol->dsc_password) == DSC_SUCCESS )
            rstat = dscd_solve( msg );
           break;

      /*                               */
      /* Halt a (sub)problem message   */
      /*                               */
      case MSG_HALT:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * HALT all problems on this node \n");
fflush( stderr );
#endif

           msghalt = (DSCHALT *)msg;
           if ( dsc_chkpass( password, msghalt->dsc_password) == DSC_SUCCESS )
            rstat = dscd_halt();
           break;

      /*                               */
      /*    Request status message     */
      /*                               */
      case MSG_QSTAT:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * STATUS Inquiry \n");
fflush( stderr );
#endif
           msgstat =(DSCQSTA *)msg;
           if ( dsc_chkpass( password, msgstat->dsc_password) == DSC_SUCCESS )
            {
             if ((s = dscn_sock(msg)) != (-1))
                rstat = dscd_stat( msg, s );
            }
           break;

      /*                               */
      /*   Problem blocking message    */
      /*                               */
      case MSG_PRBLOCK:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * PROBLEM blocking for subproblems to complete \n");
fflush( stderr );
#endif
           msgpbk = (DSCPRBLK *)msg;
           if ( dsc_chkpass( password, msgpbk->dsc_password) == DSC_SUCCESS )
            {
              if ((s = dscn_sock(msg)) != (-1))
                rstat = dscd_block( msg, s );
            }
           break;

      /*                               */
      /*   Copy request/ack message    */
      /*                               */
      case MSG_COPYCTRL:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * COPY FILE to/from remote node \n");
fflush( stderr );
#endif
           msgcpy = (DSCCPYCTRL *)msg;
           if ( dsc_chkpass( password, msgcpy->dsc_password) == DSC_SUCCESS )
            {
             if ((s = dscn_sock(msg)) != (-1))
                rstat = dscd_cpyctrl( msg, s );
            }
           break;

      /*                               */
      /* Problem terminating message   */
      /*                               */
      case MSG_KILL:
#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * KILL a (sub)problem \n");
fflush( stderr );
#endif
           msgkill = (DSCKILL *)msg;
           if ( dsc_chkpass( password, msgkill->dsc_password) == DSC_SUCCESS )
	    dscd_kill( msg );
           break;

      /*                               */
      /*           Default             */
      /*                               */
      default:
fprintf(stderr,"dsc_serv : dsc_do_msg       : * MSG * RP_PROCMSG - unknown message type %d\n", mtype);

      }			/* end case statement */

#ifdef DSDMSG
fprintf(stderr,"dsc_serv : dsc_do_msg       : returning rstat %d\n",rstat);
#endif 

   return( rstat );
}
