/*****************************************************************************/
/*                                                                           */
/*                             cpycoe_driver.c                              */
/*                                                                           */
/*****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/socket.h>
#include <sys/time.h>
#include "dscpr.c"

/*****************************************************************************/
/*                                                                           */
/*                                 definitions                               */
/*                                                                           */
/*****************************************************************************/

#define MATRIX_X 10
#define MATRIX_Y 5
#define MULT 7
#define MATRIX_FILE "sub_matrix_%d"
#define SOURCE_FILE "subsrc.c"
#define DEBUG 1

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:   calling sequence cpycode_driver cpycode_driver.d                */
/*                                                                           */
/* outputs:  subproblems, data files                                         */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  demonstrate copying subproblems ans the use of dscpr_sub and    */
/*           dscpr_wait for matrix multiplication                            */
/*                                                                           */
/*****************************************************************************/

main ( argc, argv)
int argc;
char **argv;
 {

   int addend;
   int x,y,i,idx[MATRIX_Y];
   char fidc[32];           /* source program file id                       */
   char fidd[32];           /* data file file id                            */
   char fid[32];            /* file id                                      */
   char exec[132];          /* subproblem exec command                      */
   char build[132];         /* subproblem build command                     */
   char PATH[200];          /* real path w/ matrixfile name w/o extension   */
   char temp[200];
   FILE *fp, *cfp;
   int  rstat;
   long seed;

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : starting argc %d\n",argc);
for(x=0;x<argc;x++)
fprintf(stderr,"gcode    : main             : argv[%d] %s\n",x,argv[x]);
fflush(stderr);
#endif

   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"gcode    : main             : getting cwd\n");
     fprintf(stderr,"gcode    : main             : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }

  strcat(PATH,"/sub_matrix_%d");

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : path %s\n",PATH);
fflush(stderr);
#endif

  /*                                */
  /* seed a random number generator */
  /*                                */
  time( &seed );
  srand( (int) seed );

  if ( rstat == 2 ) dscdbg_start();     /* if debug mode */
  addend = rand() % 0x3f;      /* add to every element of matrix */

   /*                                      */
   /*    generate each row of the matrix   */
   /*                                      */
   for (y=0; y<MATRIX_Y; y++) {

   /*                               */
   /* create subproblem source file */
   /*                               */
   strcpy(build,"cc -o ");
   strcat(build,MATRIX_FILE);
   sprintf(build,build,y);
   strcat(build," ");
   strcat(build,MATRIX_FILE);
   sprintf(build,build,y);
   strcat(build,".c");

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : build %s\n",build);
fflush(stderr);
#endif


   /*             */
   /* create EXEC */
   /*             */
   strcpy(exec,MATRIX_FILE);
   sprintf(exec,exec,y);
   strcat(exec," ");
   strcat(exec,MATRIX_FILE);
   strcat(exec,".d");
   sprintf(exec,exec,y);

   sprintf( fid, MATRIX_FILE, y);     /* build source file name */
   strcpy( fidc, fid);
   strcat( fidc, ".c");
   strcpy( fidd, fid);
   strcat( fidd, ".i");

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : exec %s\n",exec);
fflush(stderr);
#endif

   /*                                      */
   /* create data file for this subproblem */
   /*                                      */
   fp = fopen( fidd, "w");
   if (fp == NULL) {
      perror( fidd );
      _exit(errno);
    }          /* end if file cannot be created */
    fprintf( fp, "%d\n", addend);   /* first line is addend */
    for (x=0; x<(MATRIX_X-1); x++) /* rest is column from matrix*/
      fprintf( fp, "%d\n", rand()%0x3f);
    fclose ( fp );

    /*                                         */
    /* make a copy copy subproblem source file */
    /*   useing the new name                   */
    fp = fopen( SOURCE_FILE, "r");
    if (fp == NULL) {
        perror(SOURCE_FILE);
        _exit(errno);
     }          /* end if file cannot be created */
     cfp = fopen( fidc, "w");
     if (fp == NULL) {
         perror(fidc);
         _exit(errno);
      }          /* end if file cannot be created */
     while( !feof(fp)) {
       fgets( temp, 132, fp);
       if (strlen(temp) > 0) fputs( temp, cfp );
     }  /* while more records to copy */
     fclose( fp );
     fclose( cfp );

   /*            */
   /* Create FID */
   /*            */
   sprintf( fid, PATH, y);

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : fid %s\n",fid);
fflush(stderr);
#endif

   /*                  */
   /* Start subproblem */
   /*                  */
   idx[y] = dscpr_sub( fid, "c", build, exec, LOW_CPU, 5);

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : index idx[%d] %d\n",y,idx[y]);
fflush(stderr);
#endif

    }                             /* end for all subproblems */

    /*                           */
    /* wait for first subproblem */
    /*                           */
    fp = fopen( argv[1], "w");
    rstat = dscpr_wait( -1 );

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : reading all subproblem data\n");
fflush(stderr);
#endif

    /*                                                         */
    /* read all subproblem data files and spit the results out */
    /*                                                         */
    fp = fopen( argv[1], "w");   /* output file for main problem */
    fprintf( fp, "--                                    --");
    for( y=0; y<MATRIX_Y; y++) {
     fprintf( fp, "\n|");
     sprintf( temp, "sub_matrix_%d.d", y);
     cfp = fopen( temp, "r");
      while (!feof(cfp)) {
       temp[0] = '\0';
       fgets( temp, 13, cfp);
       if (strlen(temp)==0) break;
       i = atoi(temp);
       fprintf( fp, "%3.3d,", i);
      } /* while more elements in subproblem file */
      fprintf( fp, "|");
     }  /* while more subproblem files */
     fprintf( fp, "\n--                                    --\n");
     fclose( fp );

#ifdef DEBUG
fprintf(stderr,"cpyco    : main             : done! status %08x\n", rstat);
fflush(stderr);
#endif


     sleep(5);      /* slight pause at end for debugging only */
     exit(0);
}
