/*****************************************************************************/
/*                                                                           */
/*                               gencode_driver.c                            */
/*                                                                           */
/*****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/socket.h>
#include <sys/time.h>
#include "dscpr.c"

/*****************************************************************************/
/*                                                                           */
/*                                 definitions                               */
/*                                                                           */
/*****************************************************************************/

#define MATRIX_X 50
#define MATRIX_Y 7
#define MULT 7
#define MATRIX_FILE "sub_matrix_%d"
#define DEBUG 1

#define RD (rand()%0x3f)
#define X0(s) fputs( s, fp)
#define X1(s,a) fprintf(  fp, s, a)
#define X10(s,a0,a1,a2,a3,a4,a5,a6,a7,a8,a9) \
fprintf( fp, s, a0,a1,a2,a3,a4,a5,a6,a7,a8,a9)

/*****************************************************************************/
/*                                                                           */
/*                           module level variables                          */
/*                                                                           */
/*****************************************************************************/

static int row[ MATRIX_X ];

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:   calling sequence gencode_driver gencode_driver.d                */
/*                                                                           */
/* outputs:  subproblems, data files                                         */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*                                                                           */
/* purpose:  demonstrat the use of dscpr_sub and dscpr_wait for matrix       */
/*           multiplication                                                  */
/*                                                                           */
/*****************************************************************************/

main ( argc, argv)
int argc;
char **argv;
 {

   int x,y, idx[MATRIX_Y];
   char fidc[32];           /* source program file id                       */
   char fidd[32];           /* data file file id                            */
   char fid[32];            /* file id                                      */
   char exec[132];          /* subproblem exec command                      */
   char build[132];         /* subproblem build command                     */
   char PATH[200];          /* real path w/ matrixfile name w/o extension   */
   FILE *fp;
   int  rstat;
   long seed;
   long subidx;             /* subproblem index to DSC server               */
   char ans[80];            /* answer file                                  */

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : starting argc %d\n",argc);
for(x=0;x<argc;x++)
fprintf(stderr,"gcode    : main             : argv[%d] %s\n",x,argv[x]);
fflush(stderr);
#endif

   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"gcode    : main             : getting cwd\n");
     fprintf(stderr,"gcode    : main             : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }

  strcat(PATH,"/sub_matrix_%d");

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : path %s\n",PATH);
fflush(stderr);
#endif

  /*                                */
  /* seed a random number generator */
  /*                                */
  time( &seed );
  srand( (int) seed );

  if ( rstat == 2 ) dscdbg_start();     /* if debug mode */

   /*                                      */
   /*    generate each row of the matrix   */
   /*                                      */
   for (y=0; y<MATRIX_Y; y++) {

   /*                               */
   /* create subproblem source file */
   /*                               */
   strcpy(build,"cc -o ");
   strcat(build,MATRIX_FILE);
   sprintf(build,build,y);
   strcat(build," ");
   strcat(build,MATRIX_FILE);
   sprintf(build,build,y);
   strcat(build,".c");

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : build %s\n",build);
fflush(stderr);
#endif

   /*             */
   /* create EXEC */
   /*             */
   strcpy(exec,MATRIX_FILE);
   sprintf(exec,exec,y);
   sprintf( fid, MATRIX_FILE, y);     /* build source file name */
   strcpy( fidc, fid);
   strcat( fidc, ".c");
   strcpy( fidd, fid);
   strcat( fidd, ".d");
   fp = fopen( fidc, "w");
   if (fp == NULL) {
     fprintf( stderr, "unable to open output matrix file %s\n",
              fidc);
     _exit(0);
   }                  /* end if file cannot be created */

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : exec %s\n",exec);
fflush(stderr);
#endif

   /*                          */
   /* populate subproblem file */
   /*                          */
   /*     X0( "#include \042dscpr.c\042\n" ); */
   X0( "#include <ctype.h>\n");
   X0( "#include <stdio.h>\n");
   X0( "#include <string.h>\n");
   X0( "#include <errno.h>\n");
   X0( "#include <sys/types.h>\n");
   X0( "#include <sys/signal.h>\n");
   X0( "#include <sys/socket.h>\n");
   X0( "#include <sys/time.h>\n");
   X1( "static int row[%d] = {\n", MATRIX_X );
   for (x=0; x<(MATRIX_X-1); x+=10)
   X10( "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,\n",
   RD,RD,RD,RD,RD,RD,RD,RD,RD,RD);

   X0( "};");
   X0( "main()\n" );
   X0( "{int i;int *rptr; FILE *sfp;\n");
   X0( "\t rptr = &row[0];\n");
   if ( y == 1)  X0( "\t sleep(15);\n");
   else X0("\t sleep(45);\n");

   X1( "\t sfp=fopen(\042%s\042,\042w\042);", fidd);
   X1( "\t for (i=0; i<%d; rptr++,i++) {\n", MATRIX_X);
   X1( "\t\t (*rptr) += %d;\n", MULT);
   X0( "\t\t fprintf( sfp, \042%d\\n\042, *rptr );\n");
   X0( "\t\t } \n");
   X0( "\t fclose( sfp );\n");
   X0( "\t exit(0);\n");
   X0( "}\n");
   fclose ( fp );                /* close problem source file */

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : populated subproblem file\n");
fflush(stderr);
#endif

   /*            */
   /* Create FID */
   /*            */
   sprintf( fid, PATH, y);

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : fid %s\n",fid);
fflush(stderr);
#endif

   /*                  */
   /* Start subproblem */
   /*                  */
   idx[y] = dscpr_sub( fid, "c", build, exec, LOW_CPU, 5);

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : index idx[%d] %d\n",y,idx[y]);
fflush(stderr);
#endif

    }                             /* end for all subproblems */

    /*                           */
    /* wait for first subproblem */
    /*                           */
    fp = fopen( argv[1], "w");
    rstat = dscpr_wait( -1 );

    fprintf( fp, "gencode_driver done, status is %08x\n", rstat);

#ifdef DEBUG
fprintf(stderr,"gcode    : main             : done! status %08x\n", rstat);
fflush(stderr);
#endif

    fclose( fp );
    sleep(5);
    exit(0);

}
