##############################################################################
#                                                                            #
#                                   det.mpl                                  #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
#                               debugging hooks                              #
#                                                                            #
##############################################################################

DET_SP_DEBUG := 'DET_SP_DEBUG';
DET_SP_DEBUG := 1;

with(linalg);

##############################################################################
#                                                                            #
# subroutine: ddet                                                           #
#                                                                            #
# inputs:  one_input  - input file for subproblem                            #
#          one_output - output file for subproblem                           #
#                                                                            #
# outputs: determinant of image                                              #
#                                                                            #
# return:  none                                                              #
#                                                                            #
# purpose: maple procedure to compute the determinant of the integer matrix  #
#                                                                            #
##############################################################################

ddet := proc(one_input,one_output)
 local commandfile,  # dsc_maple DSC command file
       messagepid,   # dsc_maple_message process id file
       dscmaplepid,  # dsc_maple process id file 
       answerfile,   # subtask solution file
       msgstr,       # temporary string
       PWD,          # real path of the current working directory
       ddet,         # determinant
       TDDET;        # temporary determinant

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : one_input `.one_input);
   print(`det     : ddet             : one_output `.one_output);
 fi;

 #                                  #
 # Read in DSC Maple User Functions #
 #                                  #
 read(`dsc_maple.mpl`);

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : starting dsc_maple deamon`);
 fi;

 #                        #
 # Start dsc_maple deamon #
 #                        #
 commandfile := cat(one_input,`.cmd`);
 messagepid  := cat(one_input,`.mpid`);
 dscmaplepid := cat(one_input,`.ppid`);
 answerfile  := cat(one_input,`.ans`);
 msgstr := cat(`dsc_maple `,commandfile,` `,messagepid,` `,dscmaplepid,` &`);
 print(msgstr);
 system(msgstr);
 system(`sleep 2`);

 #                           #
 # Get real path information #
 #                           #
 PWD := dscpr_pwd(commandfile,messagepid,dscmaplepid,answerfile);

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : `.PWD);
  fi;

 #                       #
 # Kill dsc_maple_deamon #
 #                       #
 dscpr_maple_kill(commandfile,messagepid,dscmaplepid,PWD);
 system(`sleep 2`);

 #              #
 # Remove files #
 #              #
 msgstr := `rm -f `;
 msgstr := cat(msgstr,commandfile);
 system(msgstr);
 msgstr := `rm -f `;
 msgstr := cat(msgstr,messagepid);
 system(msgstr);
 msgstr := `rm -f `;
 msgstr := cat(msgstr,dscmaplepid);
 system(msgstr);
 msgstr := `rm -f `;
 msgstr := cat(msgstr,answerfile);
 system(msgstr);

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : reading in matrix and modulo`);
 fi;

 #                           #
 # Read in matrix and modulo #
 #                           # 
 msgstr := PWD;
 msgstr := cat(msgstr,`/`,one_input);
 read(msgstr);

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : computing determinand A modulo tmod`);
 fi;

 #                                          #
 # compute the determinant of A modulo tmod #
 #                                          #
 TDDET := Det(A) mod tmod;

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : determinant`.TDDET);
 fi;

 ddet := TDDET;

 save ddet , one_output;

 if DET_SP_DEBUG = 1 then
   print(`det     : ddet             : done!`);
 fi;

end;

ddet(DSC_IFILE,DSC_OFILE);
quit;
