##############################################################################
#                                                                            #
#                                   serdet.mpl                               #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
#                               debugging hooks                              #
#                                                                            #
##############################################################################

DET_S_DEBUG := 'DET_S_DEBUG';
DET_S_DEBUG := 1;

with(linalg);

##############################################################################
#                                                                            #
# subroutine: sdet                                                           #
#                                                                            #
# inputs:  A     - square matrix                                             #
#          iprod - starting point for prime moduli                           #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# return:  determinant of matrix A                                           #
#                                                                            #
# purpose: maple procedure to compute the determinant of the integer matrix  #
#          A by homomorphic imaging and Chinese remaindering                 #
#                                                                            #
##############################################################################

sdet := proc(A,iprod)
 local ubound,      # the upper bound for the product of the moduli
       prod,        # the product of the moduli
       difference,  # difference of result of the chrem and product
       moduli,      # the array of moduli
       nm,          # number of moduli needed
       residue,     # the array of determinant residues
       u;           # the determinant
       inc;         # the next starting point for the moduli search

 ubound := evalf(2*hadamard(A));

 #                    #
 # Compute the Moduli #
 #                    #

 if DET_S_DEBUG = 1 then
  print(`det_s.mpl: sdet             : computing moduli starting at `.iprod);
 fi;
  
 prod := 1; nm:=0;
 inc :=iprod; 

 while prod < ubound do
  nm := nm+1;
  moduli[nm] := nextprime(inc);
  inc := moduli[nm]+1;
  prod := prod * moduli[nm];
  
 if DET_S_DEBUG = 1 then
  print(`det_s.mpl: sdet             : moduli[`.nm.`] `);
  print(moduli[nm]);
 fi;

 od;

 if DET_S_DEBUG = 1 then
  print(`det_s.mpl: sdet             : Solving `.nm.` sub tasks`);
 fi;
 
 #                #
 # Solve subtasks #
 #                #
 for i from 1 to nm do

  if DET_S_DEBUG = 1 then
   print(`det_s.mpl: sdet             : Solving subtask `.i);
  fi;

  residue[i] :=Det(A) mod moduli[i]; 
 od;

 #                                          #
 # get determinant from residues and moduli #
 #                                          # 
 u := chrem(convert(residue,list),convert(moduli,list));

 PROD := prod;
 difference := u-prod; 

 if DET_S_DEBUG = 1 then
  print(`det_s.mpl: sdet             : product         `.prod);
  print(`det_s.mpl: sdet             : chrem           `.u);
  print(`det_s.mpl: sdet             : chrem - product `.difference);
 fi;

  if DET_S_DEBUG = 1 then
   print(`det_s.mpl: sdet             : returning u `.u);
  fi;

 RETURN(u);

end;

#read(`IM_1_10_10`);
#sdet(A,2000);
#read(`IM_1_10_10`);
#sdet(A,3000);
read(`IM_1_50_50`);
sdet(A,3000);
#read(`IM_1_100_100`);
#sdet(A,3000);
#read(`IM_1_100_100`);
#sdet(A,700000000);
#read(`IM_1_200_200`);
#sdet(A,1000000000000000000000);
#read(`serdet.mpl`);
