/*****************************************************************************/
/*                                                                           */
/*                          one_level_co_driver.C                            */
/*                                                                           */
/*****************************************************************************/

#include "bif.C"                   // DSC BIF user library

#define SF         "one_level_co"  // subproblem prefix
#define MAXRESUME  10              // number of coroutine resume invokations
#define NUMPROBS   2               // number of one_level_co#.C subproblems
#define PERCENTAGE 100             // BIF successful completion percentage
#define RETRY      5               // # of times for BIF to resubmit problems
#define DEBUG      1               // debugging hook

/*****************************************************************************/
/*                                                                           */
/* subroutine: main ()                                                       */
/*                                                                           */
/* inputs:     calling sequence one_level_co_driver one_level_co_driver.d    */
/*                                                                           */
/* outputs:    one_level_co_driver.d                                         */
/*                                                                           */
/* return:     none                                                          */
/*                                                                           */
/* purpose: This program can be used as a template for using BIF's to run    */
/*          coroutines within the DSC environment.  This module is the       */
/*          driver program which creates a bag of incomplete futures using   */
/*          a homogeneous BIF constructor and then designating the BIF as    */
/*          a bag of coroutines with the MakeCoroutines() function.          */
/*                                                                           */
/*          The RunTasks() function is used to submit the coroutines to DSC  */
/*          the first time, after which calls to GetDataFile() and to        */
/*          ResumeTasks() are alternately made to read results and to awaken */
/*          sleeping coroutines, until each coroutine has been run MAXRESUME */
/*          times.                                                           */
/*                                                                           */
/*          Once the final results are written to one_level_co_driver.d, all */
/*          sleeping coroutines are killed with a call to Kill() and the BIF */
/*          is emptied with a call to Empty().  An empty BIF still exists    */
/*          and can be refilled with new tasks.  In order to destroy a BIF,  */
/*          a call to its destructor must be made.  (This will occur when    */
/*          main() exits.)                                                   */
/*                                                                           */
/*****************************************************************************/

main (void)
{
 int number;             // number read from (one_level#.d) file
 int count = 0;
 int sum=0;              // final data file sum
 int indexlist[132];     // list of the DSC indecies of each subproblem
 char fileprf[132];      // one_level# file name w/o path
 char fid[132];          // subproblem file id
 char SOURCE_FILE[132];  // full path to source file w/o extention
 char exec[132];         // subproblem exec command
 char build[132];        // subproblem build command
 char scrfile[132];      // local problem scratch file
 char pwd[100];          // current working directory
 char *cptr;
 FILE *scr;              // pointer to local problem scratch file
 FILE *fp;               // one_level#.d file pointer

 void quit (FILE *);                         // exit
 void writefiles (int, char *, int, FILE *); // create source and input files

  //
  // Get absolute path to source file.
  //
  strcpy (SOURCE_FILE, dscpr_getabs (SF));
//                     ~~~~~~~~~~~~~~~~~
  strcpy(scrfile,"one_level_co_driver.scr");

#ifdef DEBUG
cerr << "driver   : main()           : opening " << scrfile << endl;
cerr << flush;
#endif

  //
  // Open scratch file.
  //
  if ((scr = fopen(scrfile, "w+")) == NULL) {
     cerr << "driver   : main()           : can not open " << scrfile << endl;
     cerr << "driver   : main()           : errno " << errno << endl;
     cerr << flush;
     quit (scr);
    }

#ifdef DEBUG
fprintf(scr,"driver   : main()           : starting\n");
fprintf(scr,"driver   : main()           : creating %d input and source files\n"
             ,NUMPROBS);
fflush(scr);
#endif

  //
  // Create input files and subproblems.
  //
  writefiles (NUMPROBS, SOURCE_FILE, 2, scr);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : Placing tasks in BIF\n");
fflush(scr);
#endif

  //
  // Create a homogeneous BIF.
  //
  BIF *one_level = new BIF (NUMPROBS, PERCENTAGE, SF, scr, LOW_CPU, 5);
//    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  //
  // Designate BIF to be a bag of coroutines.
  //
   one_level -> MakeCoroutines ();
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef DEBUG
fprintf(scr,"driver   : main()           : Running tasks\n");
fflush(scr);
#endif

  //
  // Call RunTasks to run tasks in BIF.
  //
   one_level -> RunTasks (RETRY);
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef DEBUG
fprintf(scr,"driver   : main()           : Tasks completed\n");
fprintf(scr,"driver   : main()           : Looping through coroutines\n");
fflush(scr);
#endif

//
// Loop through coroutine sequence.
//
for (count=0; count < MAXRESUME ; count++ ) {   

    int y = 0;
    sleep (3);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : reading data files\n");
fflush(scr);
#endif
    //
    // Read subtask data files.
    //
    while ((strcmp ((strcpy (fid, one_level -> GetDataFile ())), "nil")) != 0){
//                                ~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef DEBUG
fprintf (scr, "driver   : main()           : reading %s\n", fid);
fflush (scr);
#endif

     if ((fp = fopen (fid, "r")) == NULL) {
         cerr << "driver   : main()           : can not open " << fid << endl;
         cerr << "driver   : main()           : errno " << errno << endl;
         cerr << flush;
         quit (scr);
     }
     fscanf (fp, "%d", &number);

#ifdef DEBUG
fprintf (scr," driver   : main()           : read in %d\n", number);
fflush (scr);
#endif

     fclose (fp);
     
     //
     // If MAXRESUME iterations haven't yet completed,
     //   repopulate data files with new values.
     //
     if (count+1 < MAXRESUME) {
        //
        // Repopulate input files.
        //
        cptr = strrchr (fid, '.');
        strcpy (cptr, ".i");

#ifdef DEBUG
fprintf (scr, "driver   : main()           : writing %s\n", fid);
fflush (scr);
#endif

        if ((fp = fopen (fid, "w")) == NULL) {
          cerr << "driver   : main()           : can not open " << fid << endl;
          cerr << "driver   : main()           : errno " << errno << endl;
          cerr << flush;
        }
        fprintf (fp, "%d %d", number*2, number*2);
        fclose(fp);

#ifdef DEBUG
fprintf (scr,
         "driver   : main()           : wrote to input %d %d to input file\n",
         number*2, number*2);
fflush(scr);
#endif
     }
     else sum = number * 2;     // Calculate final answer.

   } // end "Read subtask data files."

   //
   // If MAXRESUME iterations haven't yet completed,
   //   awaken sleeping coroutines.
   //
   if (count+1 < MAXRESUME) {
       //
       // Call ResumeTasks() to resume sleeping coroutines.
       //
       int rval = one_level -> ResumeTasks (RETRY);
//                ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef DEBUG
fprintf (scr, "driver   : main()           : resume round %d \n", count);
fprintf (scr, "driver   : main()           : Resume rval %d \n", rval);
fflush (scr);
#endif
   }

} // end "Loop through coroutine sequence."

   //
   // Populate final data file.
   //
#ifdef DEBUG
fprintf(scr,"driver   : main()           : populating final data file with\n");
fprintf(scr,"driver   : main()           :      sum of last two data files\n");
fprintf(scr,"driver   : main()           : writing %d to data file\n",sum);
fflush(scr);
#endif

    if ((fp = fopen ("one_level_co_driver.d", "w")) == NULL) {
         cerr << "driver   : main()           : cannot open data file" << endl;
         cerr << "driver   : main()           : errno " << errno << endl;
         cerr << flush;
         quit (scr);
    }
    fprintf (fp, "%d", sum);
    fclose (fp);

    //
    // Kill all tasks in BIF.
    //
    one_level -> Kill (ALL);
//  ~~~~~~~~~~~~~~~~~~~~~~~

    //
    // Empty BIF.
    //
    one_level -> Empty ();
//  ~~~~~~~~~~~~~~~~~~~~~

    quit (scr);
}

/*****************************************************************************/
/* subroutine: quit (FILE *scr)                                              */
/*                                                                           */
/* inputs:     scr - pointer to problem scratch file                         */
/*                                                                           */
/* outputs:    none                                                          */
/*                                                                           */
/* return:     status                                                        */
/*                                                                           */
/* purpose:    exit                                                          */
/*****************************************************************************/
void quit (FILE *scr)
{
#ifdef DEBUG
fprintf(scr, "driver   : main()           : exiting!\n");
#endif
     cerr << flush;
     fflush (scr);
     fclose (scr);

     exit (0);
}

/*****************************************************************************/
/*                                                                           */
/* subroutine: writefiles(numfiles, SOURCE_FILE , number, scr)               */
/*                                                                           */
/* inputs:    numfiles    - number of file pairs to create                   */
/*            SOURCE_FILE - c source file w/o extension                      */
/*            number      - number to populate input file                    */
/*            scr         - pointer to scratch file                          */
/*                                                                           */
/* outputs:  input and source files                                          */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  create numfiles source and input file pairs                     */
/*                                                                           */
/*****************************************************************************/

void writefiles (int numfiles, char *SOURCE_FILE, int number, FILE *scr)
{
  int y;
  char fid[64];
  char temp[80];
  char inputfile[132];
  char readfile[132];
  char writefile[132];
  FILE *fp;
  FILE *readptr;
  FILE *writeptr;

  strcpy(readfile,SOURCE_FILE);
  strcat(readfile,".C");

  /*                        */
  /* Set the nth input file */
  /*                        */

  for (y = 0 ; y < numfiles ; y++)
  {
    sprintf(fid,"%s%d",SOURCE_FILE,y);
    strcat(fid,".i");

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : opening %s\n",fid);
fflush(scr);
#endif

   fp = fopen(fid,"w");
   if (fp == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",fid);
      fprintf(scr,"driver   : writefile        : errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : putting in %d %d\n",
           number,number);
fflush(scr);
#endif
    fprintf(fp,"%d %d",number,number);

    fclose(fp);
  }

  /*                         */
  /* Set the nth source file */
  /*                         */

  for (y=0; y < numfiles; y++)
  {
  strcpy(writefile,SOURCE_FILE);
  strcat(writefile,"%d.C");
  sprintf(writefile,writefile,y);

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : reading from %s\n",readfile);
fprintf(scr,"driver   : writefile        : writing to %s\n",writefile);
fflush(scr);
#endif

  readptr = fopen(readfile,"r");
   if (readptr == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",readfile);
      fprintf(scr,"driver   : writefile        :errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

  writeptr = fopen(writefile, "w");
   if (writeptr == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",writefile);
      fprintf(scr,"driver   : writefile        : errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

  /*                  */
  /* copy source code */
  /*                  */
  while( !feof(readptr)) {
          temp[0] = '\0';
          fgets( temp, 80,readptr);
          if (strlen(temp) > 0) fputs( temp,writeptr);
        }
  fclose(readptr);
  fclose(writeptr);
 
  }

}

