/*****************************************************************************/
/*                                                                           */
/*                              one_level_co.c                               */
/*                                                                           */
/*****************************************************************************/

#include "dscpr.c"             /* DSC C user library                         */
#define DEBUG 1                /* debugging hook                             */

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:   calling sequence one_level_co one_level_co.d                    */
/*                                                                           */
/* outputs:  data files                                                      */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  this program can be used as a template for using co routines    */
/*           with in the DSC environment. This module is the subproblem      */
/*           wich reads its input files and creates new data files and enters*/
/*           a sleep state using the dscpr_cowait function. Once an awake    */
/*           signal is recieved from the driver program via a dscpr_coresume */
/*           call the program will contiue executuion after the dscpr_cowait */
/*           call. This cowait is enclosed in an infinite loop so that the   */
/*           subporoblem can be reused many times until a dscpr_kill command */
/*           is issued from the driver program.                              */
/*                                                                           */
/*****************************************************************************/

int main(argc,argv)
int argc;
char **argv;

{

 int x;                  /* first number in input file                       */
 int y;                  /* second number in input file                      */
 int sum=0;              /* sum of numbers in input file                     */
 int count=0;          
 char temp[132];
 char *cptr;
 FILE *fp;               /* input file pointer                               */
 FILE *outfp;            /* output (data) file pointer                       */
 FILE *scr;              /* pointer to local problem scratch file            */
 FILE *fopen();          /* opens the file name, and associate a stream      */

  /*                          */
  /* declare coroutine status */
  /*                          */
  dscpr_cosetup();

  strcpy(temp,argv[1]);
  cptr = strrchr(temp,'.');
  strcpy(cptr,".scr");

  scr = fopen(temp,"w+");
  if (scr == NULL )
   {
fprintf(stderr,"one_level: main()           : can not open %s\n",temp);
fprintf(stderr,"one_level: main()           : errno %d\n",errno);
fflush(stderr);
exit(errno);
   }

  while( TRUE )
  {

  count++;

#ifdef DEBUG
fprintf(scr,"one_level: main()           : opening %s count %d\n"
           ,argv[1],count);
fflush(scr);
#endif
 
  outfp = fopen(argv[1],"w");
  if (outfp == NULL)
   {
fprintf(scr,"one_level: main()           : can not open %s\n",argv[1]);
fprintf(scr,"one_level: main()           : errno %d\n",errno);
fflush(stderr);
exit(errno);
   }

  strcpy(temp,argv[1]);
  cptr = strrchr(temp,'.');
  strcpy(cptr,".i");

 /*                 */
 /* read input file */
 /*                 */

#ifdef DEBUG
fprintf(scr,"one_level: main()           : reading input file %s\n",temp);
fflush(scr);
#endif

  fp = fopen(temp,"r");
  if (fp == NULL )
   {
fprintf(scr,"one_level: main()           : can not open %s\n",temp);
fprintf(scr,"one_level: main()           : errno %d\n",errno);
fflush(stderr);
exit(errno);
   }

  fscanf(fp,"%d %d",&x,&y);

#ifdef DEBUG
fprintf(scr,"one_level: main()           : read in %d %d\n",x,y);
fflush(scr);
#endif

  fclose(fp);

  sum = x+y;
  
  /*                      */
  /* write to output file */
  /*                      */

#ifdef DEBUG
fprintf(scr,"one_level: main()           : writing %d\n",sum);
fflush(scr);
#endif

  
  fprintf(outfp,"%d\n",sum);
  fclose(outfp);
 
  /*             */ 
  /* go to sleep */
  /*             */

  sleep(2);
  /* 0 - do not copy a data file back */
  /* 1 - do copy a data file back     */
  dscpr_cowait(1);
  fflush(scr);

}

}
