/*****************************************************************************/
/*                                                                           */
/*                          one_level_co_driver.c                            */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>             /* standard I/O                               */
#include <string.h>            /* string functions                           */
#include "dscpr.c"             /* DSC C user library                         */
#define SF "one_level_co"      /* subproblem prefix                          */
#define MAXRESUME 10           /* number of coroutine resume invokations     */
#define NUMPROBS 2             /* number of one_level_co#.c subproblems      */
#define DEBUG 1                /* debugging hook                             */

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:    calling sequence one_level_co_driver one_level_co_driver.d     */
/*                                                                           */
/* outputs:  None                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose: this program can be used as a template for using co routines     */
/*          within the DSC environment. This module is the driver program    */
/*          which first spawns subproblems by creating the code and input    */
/*          files for the latter and uses the dscpr_sub command for the      */
/*          remote procedure call. The program will then wait on the         */
/*          completion of all of its subporoblems. The call to dscpr_wait    */
/*          with the -1 flag will block this problem until either all of its */
/*          subproblems terminate (via an exit or error) or all of its       */
/*          subproblems enter a sleep state. The driver program assumes      */
/*          that its subproblems are coroutines so after it is unblocked     */
/*          it reads its subproblems data files and creats new input files   */
/*          for its sleeping subproblems. Once this has been accomplished    */
/*          the driver program will send a remote proceudre call to start    */
/*          up the coroutine (sleeping  subproblem) via the dscpr_coresume   */
/*          function which takes as its argument the index to the subproblem */
/*          (this index is returned from the dscpr_sub call). This process   */
/*          of creating input files and sending RPC's will continue for      */
/*          MAXRESUME iterations. Once MAXRESUME is reached the subproblems  */
/*          are killed via the dscpr_kill command and  the driver program    */
/*          exits.                                                           */
/*                                                                           */
/*****************************************************************************/

main ( argc, argv)
int argc;
char **argv;
{
 int y;                  /* subproblem loop counter                          */
 int i;                  /* DSC C user interface return values               */
 int number;             /* number read from (one_level#.d) file             */
 int count = 0;
 int sum=0;              /* final data file sum                              */
 int indexlist[132];     /* list of the DSC indecies of each subproblem      */
 char fileprf[132];      /* one_level# file name w/o path                    */
 char fid[132];          /* subproblem file id                               */
 char SOURCE_FILE[132];  /* full path to source file w/o extention           */
 char exec[132];         /* subproblem exec command                          */
 char build[132];        /* subproblem build command                         */
 char scrfile[132];      /* local problem scratch file                       */
 char pwd[100];          /* current working directory                        */
 FILE *scr;              /* pointer to local problem scratch file            */
 FILE *fp;               /* one_level#.d file pointer                        */
 FILE *fopen();          /* opens the file name, and associate a stream      */
 void writefiles();      /* create source and input files                    */

  if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"driver   : main()           : getting cwd\n");
     fprintf(stderr,"driver   : main()           : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }

  strcpy(SOURCE_FILE,pwd);
  strcat(SOURCE_FILE,"/");
  strcat(SOURCE_FILE,SF);

  strcpy(scrfile,"one_level_co_driver.scr");

#ifdef DEBUG
fprintf(stderr,"driver   : main()           : opening %s\n",scrfile);
fflush(stderr);
#endif


  scr = fopen(scrfile, "w+");
  if ( scr == NULL )
    {
     fprintf(stderr,"driver   : main()           : can not open %s\n",scrfile);
     fprintf(stderr,"driver   : main()           : errno %d\n",errno);
     fflush(stderr);
     exit(errno);
    }

#ifdef DEBUG
fprintf(scr,"driver   : main()           : starting\n");
fprintf(scr,"driver   : main()           : creating %d input and source files\n"
             ,NUMPROBS);
fflush(scr);
#endif

  /*                                              */
  /* initial creation of input files and subprobs */
  /*                                              */
  writefiles(NUMPROBS,SOURCE_FILE,2,scr);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : entering distribution loop\n");
fflush(scr);
#endif

     for (y=0; y<NUMPROBS; y++)
     {
        /*            */
        /* create fid */
        /*            */
        strcpy(fid,SOURCE_FILE);
        strcat(fid,"%d");
        sprintf( fid,fid , y);

        /*              */
        /* create build */
        /*              */
        strcpy(build,"cc -o ");
        strcat(build,SF);
        strcat(build,"%d");
        sprintf(build,build,y);
        strcat(build," ");
        strcat(build,SF);
        strcat(build,"%d");
        sprintf(build,build,y);
        strcat(build,".c");

        /*             */
        /* Create exec */
        /*             */
        strcpy(exec,SF);
        strcat(exec,"%d");
        sprintf(exec,exec,y);
        strcat(exec," ");
        strcat(exec,SF);
        strcat(exec,"%d.d");
        sprintf(exec,exec,y);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : fid %s\n",fid);
fprintf(scr,"driver   : main()           : build %s\n",build);
fprintf(scr,"driver   : main()           : exec %s\n",exec);
fflush(scr);
#endif

        /*                      */
        /* start the subproblem */
        /*                      */

        i = dscpr_sub( fid, "ci", build , exec, LOW_CPU, 5);  
        if ( i < 0 )
         {
          fprintf(scr,"driver   : main()           : error distributing\n");
          fprintf(scr,"driver   : main()           : rval %d\n",i);
          goto XDONE;
         }

        fprintf(scr,"driver   : main()           : spawned index %d\n",i);
        fflush(scr);
        indexlist[y] = i;
        }			

        /*                             */
        /* Awaken sleeping subprobelms */
        /*                             */

for (count=0; count < MAXRESUME ; count++ )
 {   

     fprintf(scr,"driver   : main()           : wait on all \n");

     /*                                 */
     /* wait on subproblems to complete */
     /*                                 */

     i = dscpr_wait(-1);
     if ( i< 0 )
      {
       fprintf(scr,"driver   : main()           : error in wait \n");
       fprintf(scr,"driver   : main()           : rval %d\n",i);
       goto XDONE;
      }

     sleep(3);

     /*                        */
     /* read the nth data file */
     /*                        */
     fprintf(scr,"driver   : main()           : reading data files\n");

     strcpy(fileprf,SOURCE_FILE);
     strcat(fileprf,"%d");

  for (y=0; y < NUMPROBS; y++)
  {
    sprintf(fid,fileprf,y);
    strcat(fid,".d");

#ifdef DEBUG
fprintf(scr,"driver   : main()           : reading %s\n",fid);
#endif

    fp = fopen(fid,"r");
    if (fp == NULL) 
     {
      fprintf(scr,"driver   : main()           : can not open %s\n",fid);
      fprintf(scr,"driver   : main()           : errno %d\n",errno);
      fflush(scr);
      goto XDONE;
     }

    fscanf(fp,"%d",&number);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : read in %d\n",number);
fflush(scr);
#endif

    fclose(fp);
     
    sprintf(fid,fileprf,y);
    strcat(fid,".i");

#ifdef DEBUG
fprintf(scr,"driver   : main()           : writing %s\n",fid);
#endif

    fp = fopen(fid,"w");
    if (fp == NULL) 
     {
      fprintf(scr,"driver   : main()           : can not open %s\n",fid);
      fprintf(scr,"driver   : main()           : errno %d\n",errno);
      fflush(scr);
     }
    fprintf(fp,"%d %d",number*2,number*2);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : wrote to input %d %d to input file\n"
           ,number*2,number*2);
fflush(scr);
#endif

    fclose(fp);
  }

   fprintf(scr,"driver   : main()           : resume round %d \n",count);

   if ( count+1 < MAXRESUME )
   { 
   for (y=0; y < NUMPROBS; y++)
    {
     fprintf(scr,"driver   : main()           : sending resume to index %d "
                ,indexlist[y]);
     /*                                                                 */
     /*  "NULL" will not send an .i file                                */
     /*  "" will send the name related to the index with an .i appended */
     /*  Any other valid string can be used to be sent for an .i file   */
     /*                                                                 */
     i = dscpr_coresume(indexlist[y],"");

     fprintf(scr,"driver   : main()           : coresume returned with %d \n"
                ,i);
    }
    }


} /* End for i = 1 to count */ 

   /*                          */
   /* populate final data file */
   /*                          */

#ifdef DEBUG
fprintf(scr,"driver   : main()           : populating final data file\n");
fflush(scr);
#endif

     /*                        */
     /* read the nth data file */
     /*                        */
     fprintf(scr,"driver   : main()           : reading data files\n");

     strcpy(fileprf,SOURCE_FILE);
     strcat(fileprf,"%d");

  for (y=0; y < NUMPROBS; y++)
  {
    sprintf(fid,fileprf,y);
    strcat(fid,".d");

#ifdef DEBUG
fprintf(scr,"driver   : main()           : reading %s\n",fid);
#endif

    fp = fopen(fid,"r");
    if (fp == NULL)
     {
      fprintf(scr,"driver   : main()           : can not open %s\n",fid);
      fprintf(scr,"driver   : main()           : errno %d\n",errno);
      fflush(scr);
      goto XDONE;
     }

    fscanf(fp,"%d",&number);

#ifdef DEBUG
fprintf(scr,"driver   : main()           : read in %d\n",number);
fflush(scr);
#endif

    fclose(fp);

    sum = sum + number;
   }
     fp = fopen("one_level_co_driver.d","w");
     if (fp == NULL)
     {
fprintf(scr,"driver   : main()           : can not open one_level_co_driver.d\n"
       );
fprintf(scr,"driver   : main()           : errno %d\n",errno);
fflush(scr);
goto XDONE;
     }
     fprintf(fp,"%d",sum);
     fclose(fp);

   /*                              */
   /* send kill subproblem message */
   /*                              */
   for (y=0; y < NUMPROBS; y++)
    {
     fprintf(scr,"driver   : main()           : sending kill to index %d "
                ,indexlist[y]);
     fflush(scr);

     i = dscpr_kill(indexlist[y]);

     fprintf(scr,"driver   : main()           : dscpr_kill returned with %d \n"
            ,i);
    }

XDONE:

#ifdef DEBUG
fprintf(scr,"driver   : main()           : exiting!\n");
#endif

     fflush(scr);
     fflush(stderr);
     fclose(scr);


     exit(0);

}

/*****************************************************************************/
/*                                                                           */
/* subroutine: writefiles(numfiles, SOURCE_FILE , number, scr)               */
/*                                                                           */
/* inputs:    numfiles    - number of file pairs to create                   */
/*            SOURCE_FILE - c source file w/o extension                      */
/*            number      - number to populate input file                    */
/*            scr         - pointer to scratch file                          */
/*                                                                           */
/* outputs:  input and source files                                          */
/*                                                                           */
/* return:   none                                                            */
/*                                                                           */
/* purpose:  create numfiles source and input file pairs                     */
/*                                                                           */
/*****************************************************************************/

void writefiles(numfiles, SOURCE_FILE , number, scr)
int numfiles;
char *SOURCE_FILE;
int number;
FILE *scr;
{
  int y;
  char fid[64];
  char temp[80];
  char inputfile[132];
  char readfile[132];
  char writefile[132];
  FILE *fp;
  FILE *fopen();
  FILE *readptr;
  FILE *writeptr;

  strcpy(readfile,SOURCE_FILE);
  strcat(readfile,".c");

  /*                        */
  /* Set the nth input file */
  /*                        */

  for (y=0; y < numfiles; y++)
  {
    sprintf(fid,"%s%d",SOURCE_FILE,y);
    strcat(fid,".i");

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : opening %s\n",fid);
fflush(scr);
#endif

   fp = fopen(fid,"w");
   if (fp == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",fid);
      fprintf(scr,"driver   : writefile        : errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : putting in %d %d\n",
           number,number);
fflush(scr);
#endif
    fprintf(fp,"%d %d",number,number);

    fclose(fp);
  }

  /*                         */
  /* Set the nth source file */
  /*                         */

  for (y=0; y < numfiles; y++)
  {
  strcpy(writefile,SOURCE_FILE);
  strcat(writefile,"%d.c");
  sprintf(writefile,writefile,y);

#ifdef DEBUG
fprintf(scr,"driver   : writefile        : reading from %s\n",readfile);
fprintf(scr,"driver   : writefile        : writing to %s\n",writefile);
fflush(scr);
#endif

  readptr = fopen(readfile,"r");
   if (readptr == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",readfile);
      fprintf(scr,"driver   : writefile        :errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

  writeptr = fopen(writefile, "w");
   if (writeptr == NULL)
     {
      fprintf(scr,"driver   : writefile        : can not open %s\n",writefile);
      fprintf(scr,"driver   : writefile        : errno %d\n",errno);
      fflush(scr);
      fclose(scr);
      fclose(fp);
      exit(errno);
     }

  /*                  */
  /* copy source code */
  /*                  */
  while( !feof(readptr)) {
          temp[0] = '\0';
          fgets( temp, 80,readptr);
          if (strlen(temp) > 0) fputs( temp,writeptr);
        }
  fclose(readptr);
  fclose(writeptr);
 
  }

}

