/*****************************************************************************/
/*                                                                           */
/*                                 sub.c                                     */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>             /* standard I/O                               */
#include <string.h>            /* string functions                           */
#include "dscpr.c"             /* DSC C user library                         */
#define NUMPROBS 2             /* number of subsub#.c subproblems            */ 
#define SOURCE_FILE "subsub.c" /* subproblem prefix                          */ 
#define DEBUG 1                /* debugging hook                             */

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:    calling sequence sub sub.d                                     */
/*                                                                           */
/* outputs:  None                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose:  this is the source code spawned by subdriver.c                  */
/*                                                                           */
/*****************************************************************************/

int main(argc,argv)
int argc;
char **argv;
{
 int y;                  /* subproblem loop counter                          */
 int i;                  /* dscpr_sub return value                           */
 int sum;                /* sum of data files (subsub.d)                     */
 int x;                  /* temporary variable                               */
 int rvalue;             /* read value from input file                       */
 char fidc[200];         /* C source program file id                         */
 char fidd[200];         /* data file id                                     */
 char fidi[200];         /* input file id                                    */
 char fid[200];          /* file id                                          */
 char temp[80];          /* temporary memory space                           */
 char exec[80];          /* subproblem execute                               */
 char build[80];         /* sobproblem build                                 */
 char pwd[100];          /* current working directory                        */
 char mnode[DSC_NODEZ];  /* local node name                                  */
 char argprefix[80];     /* current problem prefix name                      */
 char *cptr;             /* character pointer                                */
 int ap;                 /* counter                                          */
 FILE *dotdfp;           /* data file pointer                                */
 FILE *dotifp;           /* input file pointer                               */
 FILE *dotcwfp;          /* new C source file pointer                        */
 FILE *dotcrfp;          /* origional C file pointer                         */
 FILE *scrfp;            /* local subproblem scratch file                    */ 
 FILE *datafp;           /* local subproblem  data file                      */

  datafp = fopen(argv[1],"w");
  if ( datafp == NULL )
    {
      fprintf(stderr,"sub      : main()           : can not open %s\n",argv[1]);
      fprintf(stderr,"sub      : main()           : errno %d\n",errno);
      fflush(stderr);
      exit(errno);
    }

  cptr=argv[1];ap=0;
  while ( *cptr != '.' ) argprefix[ap++]=*cptr++;
  argprefix[ap] = '\0';

  strcat(argprefix,".scr");
  scrfp = fopen(argprefix,"w");
  if ( scrfp == NULL ) 
   {
     fprintf(stderr,"sub      : main()           : can not open%s\n",argprefix);
     fprintf(stderr,"sub      : main()           : errno %d\n",errno);
     fflush(stderr); 
     exit(errno);
   }

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : starting\n");
fprintf(scrfp,"sub      : main()           : argv[0] %s\n",argv[0]);
fprintf(scrfp,"sub      : main()           : argv[1] %s\n",argv[1]); 
fflush(scrfp);
#endif

  gethostname(mnode, DSC_NODEZ);
   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"sub      : main()           : getting cwd\n");
     fprintf(stderr,"sub      : main()           : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }


#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : node %s\n",mnode);
fprintf(scrfp,"sub      : main()           : cwd %s\n",pwd);
fflush(scrfp);
#endif

  cptr=argv[1];ap=0;
  while ( *cptr != '.' ) argprefix[ap++]=*cptr++;
  argprefix[ap] = '\0';

  sprintf( fidi,"%s.i",argprefix);
 
  dotifp = fopen(fidi,"r");
  if ( dotifp == NULL )
         {
fprintf(scrfp,"sub      : main()           : can not open %s\n",fidi);
fprintf(scrfp,"sub      : main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
         }
   fscanf(dotifp,"%d",&rvalue);
   fprintf(scrfp,"sub      : main()           : read in from %s %d\n",
           fidi,rvalue);
   fclose(dotifp);
 
   rvalue += rvalue;

  for (y=0; y<NUMPROBS; y++) 
  { 

        /*                                                   */
        /* create subproblem source c  files and input files */
        /* in DSC_PATH/subdir. Note that they will go into   */
        /* the DSC_PATH at run time.                         */
        /*                                                   */

        sprintf( fid, "%s/%ssubsub%d",pwd,argprefix, y);	
        strcpy( fidc, fid);                    
        strcat( fidc, ".c");
        strcpy( fidi, fid);                    
        strcat( fidi, ".i"); 

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : fid %s\n",fid);
fprintf(scrfp,"sub      : main()           : fidc %s\n",fidc);
fprintf(scrfp,"sub      : main()           : fidi %s\n",fidi);
fprintf(scrfp,"sub      : main()           : creating .i file\n");
fflush(scrfp);
#endif

        /*                 */
        /* create .i files */
        /*                 */

        dotifp = fopen(fidi,"w");
        if ( dotifp == NULL )
         {
fprintf(scrfp,"sub      : main()           : can not open %s\n",fidi);
fprintf(scrfp,"sub      : main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
         }
        fprintf(dotifp,"%d\n",rvalue);
        fclose(dotifp);

        dotcrfp = fopen( SOURCE_FILE, "r");
        if (dotcrfp == NULL) {
fprintf(scrfp,"sub      : main()           : can not open %s\n",SOURCE_FILE);
fprintf(scrfp,"sub      : main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
        }

        dotcwfp = fopen( fidc, "w");

        if (dotcwfp == NULL) {
fprintf(scrfp,"sub      : main()           : can not open %s\n",fidc);
fprintf(scrfp,"sub      : main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
        }

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : creating source file\n");
fflush(scrfp);
#endif

        while( !feof(dotcrfp)) {
          temp[0] = '\0';
          fgets( temp, 80, dotcrfp);
          if (strlen(temp) > 0) fputs( temp, dotcwfp );
        }  /* while more records to copy */

        fclose( dotcrfp );
        fclose( dotcwfp );

sprintf( exec, "%ssubsub%d %ssubsub%d.d",argprefix, y,argprefix, y);
sprintf( build,"cc -o %ssubsub%d %ssubsub%d.c",argprefix, y,argprefix, y);

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           :  exec %s\n",exec);
fprintf(scrfp,"sub      : main()           :  build %s\n",build);
fflush(scrfp);
#endif

        i = dscpr_sub( fid, "ci", build, exec, LOW_CPU, 5); 

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : index %d\n",i);
fflush(scrfp);
#endif

  }		

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : waiting on all subproblems\n");
fflush(scrfp);
#endif

       dscpr_wait(-1);

  sum = 0;

  for (y=0; y<NUMPROBS; y++) 
  {

        sprintf( fid, "%s/%ssubsub%d",pwd, argprefix,y); 
        strcpy( fidd, fid);                  /* build data file */
        strcat( fidd, ".d");

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : reading  ... %s\n",fidd);
fflush(scrfp);
#endif

        dotdfp = fopen(fidd,"r");
        if (dotdfp == NULL )
         {
fprintf(scrfp,"sub      : main()           : can not open %s\n",fidd);
fprintf(scrfp,"sub      : main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
         }

        fscanf(dotdfp,"%d",&x);

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : value %d\n",x);
fflush(scrfp);
#endif

        sum += x; x=0;

        fclose(dotdfp);
  }

#ifdef DEBUG
fprintf(scrfp,"sub      : main()           : sum %d \n",sum);
fflush(scrfp);
#endif

 fprintf(datafp,"%d\n",sum);
 fflush(datafp);
 fclose(datafp);

 fprintf(scrfp,"sub      : main()           : exiting!\n");
 fflush(scrfp);
 fclose(scrfp);

 exit(0);
}
