/*****************************************************************************/
/*                                                                           */
/*                               subdriver.c                                 */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>             /* standard I/O                               */
#include <string.h>            /* string functions                           */
#include "dscpr.c"             /* DSC C user library                         */
#define NUMPROBS 3             /* number of subsub#.c subproblems            */
#define SOURCE_FILE "sub.c"    /* subproblem prefix                          */
#define DEBUG 1                /* debugging hook                             */

/*****************************************************************************/
/*                                                                           */
/* subroutine: main(argc,argv)                                               */
/*                                                                           */
/* inputs:    calling sequence subdriver subdriver.d                         */
/*                                                                           */
/* outputs:  None                                                            */
/*                                                                           */
/* return:   0 = DSC_SUCCESS                                                 */
/*           n = error mask                                                  */
/*                                                                           */
/* purpose: test program that demonstrates the proper way to manipulate the  */
/*           paths for subproblems distributing subproblems.                 */
/*                                                                           */
/*                                                                           */
/* File description ...                                                      */
/*                                                                           */
/* subdriver ... make copies of sub.c to DSC_PATH                            */
/*                                       named sub1.c ... subn.c             */
/*                                                                           */
/*               make sub.i in           DSC_PATH                            */
/*                                       named sub1.i ... subn.i             */
/*                                                                           */
/*                                       FID = DSC_PATH/sub#                 */
/*                                       (.d get copied to FID)              */ 
/*                                       EXEC = DSC_PATH/sub#                */
/*                                                                           */
/*               after wait read .d files DSC_PATH/sub#.d                    */
/*               write result in DSC_PATH/subdriver.d                        */
/*                                                                           */
/*                                                                           */
/* sub#.c ....... make copies of subsub.c to DSC_PATH                        */
/*                                           named subsub1.c ... subsubn.c   */
/*                                                                           */
/*                make subsub.i in           DSC_PATH                        */
/*                                           named subsub1.i ... subsubn.c   */
/*                                           FID = DSC_PATH/subsub#          */
/*                                           (.d gets copied to FID)         */
/*                                           EXEC = DSC_PATH/subsub#         */
/*                                                                           */
/*                after wait read .d files  DSC_PATH/subsub#.d               */
/*                write results to DSC_PATH/sub#.d                           */
/*                                                                           */
/*                                                                           */
/* subsub.c .... read the two numbers in DSC_PATH/subsub#.i add them         */
/*               and write the sum to DSC_PATH/subsub#.d                     */
/*                                                                           */
/*****************************************************************************/

int main ( argc, argv)
int argc;
char **argv;
{
 int y;                  /* subproblem loop counter                          */
 int i;                  /* dscpr_sub return value                           */
 int sum;                /* sum of data files (subsub.d)                     */
 int x;                  /* temporary variable                               */
 char fidc[200];         /* C source program file id                         */
 char fidd[200];         /* data file id                                     */
 char fidi[200];         /* input file id                                    */
 char fid[200];          /* file id                                          */
 char temp[80];          /* temporary memory space                           */
 char exec[80];          /* subproblem execute                               */
 char build[80];         /* sobproblem build                                 */
 char pwd[100];          /* current working directory                        */
 char mnode[DSC_NODEZ];  /* local node name                                  */
 char argprefix[80];     /* current problem prefix name                      */
 char *cptr;             /* character pointer                                */
 int ap;                 /* counter                                          */
 FILE *dotdfp;           /* data file pointer                                */
 FILE *dotifp;           /* input file pointer                               */
 FILE *dotcwfp;          /* new C source file pointer                        */
 FILE *dotcrfp;          /* origional C file pointer                         */
 FILE *scrfp;            /* local subproblem scratch file                    */
 FILE *datafp;           /* local subproblem  data file                      */

  datafp = fopen(argv[1],"w"); 
  if ( datafp == NULL ) 
    {
      fprintf(stderr,"subdriver: main()           : can not open %s\n",argv[1]);
      fprintf(stderr,"subdriver: main()           : errno %d\n",errno);
      fflush(stderr);
      exit(errno);
    }

  cptr=argv[1];ap=0;
  while ( *cptr != '.' ) argprefix[ap++]=*cptr++;
  argprefix[ap] = '\0';

  strcat(argprefix,".scr");
  scrfp = fopen(argprefix,"w");  
  if ( scrfp == NULL )
   {
     fprintf(stderr,"subdriver: main()           : can not open%s\n",argprefix);
     fprintf(stderr,"subdriver: main()           : errno %d\n",errno);
     fflush(stderr);
     exit(errno);
   }


#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : starting\n");
fflush(scrfp);
#endif

  gethostname(mnode, DSC_NODEZ);
   if ( getcwd(pwd,sizeof(pwd)) == NULL )
    {
     fprintf(stderr,"subdriver: main()           : getting cwd\n");
     fprintf(stderr,"subdriver: main()           : errno %d\n",errno);
     fflush( stderr );
     exit(5);
    }


#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : node %s\n",mnode);
fprintf(scrfp,"subdriver: main()           : cwd %s\n",pwd);
fflush(scrfp);
#endif

  for (y=0; y<NUMPROBS; y++) 
  { 

        /*                                                   */
        /* create subproblem source c  files and input files */
        /* in DSC_PATH/subdir. Note that they will go into   */
        /* the DSC_PATH at run time.                         */
        /*                                                   */

        sprintf( fid, "%s/sub%d",pwd,y);   /* build file id */
        strcpy( fidc, fid);                 /* build C file name */
        strcat( fidc, ".c");
        strcpy( fidi, fid);                 /* build .i file name */
        strcat( fidi, ".i");

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : fid %s\n",fid);
fprintf(scrfp,"subdriver: main()           : fidc %s\n",fidc);
fprintf(scrfp,"subdriver: main()           : fidi %s\n",fidi);
fprintf(scrfp,"subdriver: main()           : creating .i file\n");
fflush(scrfp);
#endif

        /*                 */
        /* create .i files */
        /*                 */

        dotifp = fopen(fidi,"w");
        if ( dotifp == NULL )
         {
fprintf(scrfp,"subdriver: main()           : can not open %s\n",fidi);
fprintf(scrfp,"subdriver: main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
         }

        fprintf(dotifp,"100\n");         
        fclose(dotifp);


        dotcrfp = fopen( SOURCE_FILE, "r");

        if (dotcrfp == NULL) {
fprintf(scrfp,"subdriver: main()           : can not open %s\n",SOURCE_FILE);
fprintf(scrfp,"subdriver: main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
        }

        dotcwfp = fopen( fidc, "w");

        if (dotcwfp == NULL) {
fprintf(scrfp,"subdriver: main()           : can not open %s\n",fidc);
fprintf(scrfp,"subdriver: main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
        }

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : creating source file\n");
fflush(scrfp);
#endif

        while( !feof(dotcrfp)) {
          temp[0] = '\0';
          fgets( temp, 80, dotcrfp);
          if (strlen(temp) > 0) fputs( temp, dotcwfp );
        }  /* while more records to copy */

        fclose( dotcrfp );
        fclose( dotcwfp );

        sprintf(exec,"sub%d sub%d.d", y, y);
        sprintf(build,"cc -o sub%d sub%d.c", y, y);

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : exec %s\n",exec);
fprintf(scrfp,"subdriver: main()           : build %s\n",build);
fflush(scrfp);
#endif

        i = dscpr_sub( fid, "ci", build, exec, LOW_CPU,5); 

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : index %d\n",i);
fflush(scrfp);
#endif

        }

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : waiting on all subproblems\n");
fflush(scrfp);
#endif

  dscpr_wait(-1);
  sum = 0; x=0;
  sleep(20);


  /*                                                         */
  /* read from sub data files, sum, and write sum to .d file */
  /*                                                         */

  for (y=0,sum=0; y<NUMPROBS; y++) 
  {
        sprintf( fid, "%s/sub%d",pwd, y);  
        strcpy( fidd, fid);        
        strcat( fidd, ".d");

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : reading  ... %s\n",fidd);
fflush(scrfp);
#endif

        dotdfp = fopen(fidd,"r");
        if (dotdfp == NULL )
         {
fprintf(scrfp,"subdriver: main()           : can not open %s\n",fidd);
fprintf(scrfp,"subdriver: main()           : errno %d\n",errno);
fflush(scrfp);
exit(errno);
         }

        fscanf(dotdfp,"%d",&x);

#ifdef DEBUG
fprintf(scrfp,"subdriver: main()           : value %d\n",x);
fflush(scrfp);
#endif

        sum +=  x;x=0;

        fclose(dotdfp);
  }

 fprintf(datafp,"%d\n",sum);
 fflush(datafp);
 fclose(datafp);

 fprintf(scrfp,"subdriver: main()           : exiting!\n");
 fflush(scrfp);
 fclose(scrfp);

 exit(0);
}
