;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                             one_level.lsp                              ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(progn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;; Invocation                                                             ;;;
;;;    dsc_lisp.int lispprog outfile infile                                ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                          debugging hooks                               ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq odebug nil)
(setq odebug 1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                             globals                                    ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; in               - input file pointer  (si:argv 2)
;;; out              - output file pointer (si:argv 1)
;;; odebug           - one level debugging hook

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                          interface preface                             ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;                      ;;;
;;; get output file name ;;;
;;;                      ;;;
(defun getout () 
  (cond ( (or ( = (si:argc) 2)
              ( = (si:argc) 3)) 
          (string-downcase  (si:argv 1)) 
        )
        (T NIL)
  )
)


;;;                     ;;;
;;; get input file name ;;;
;;;                     ;;;
(defun getin ()
  (cond ( ( = (si:argc) 3) 
          (string-downcase (si:argv 2))
        ) 
        ( (not (>= (si:argc) 2)) 'USAGE-ERROR 
        )
        (T NIL)
  ) 
)

;;;                                             ;;;
;;; set output filename and pointer if possible ;;;
;;;                                             ;;;
(setq outfilename (getout))
(cond ( (not (null outfilename))
         (setq out (open outfilename :direction
                                     :output
                                     :if-exists :overwrite
                                     :if-does-not-exist :create)
         )
       )
       ( T (print "usage : dsc_lisp.* lisp.prog out.file [in.file]"))
)


;;;                                            ;;;
;;; set input filename and pointer if possible ;;;
;;;                                            ;;;
(setq infilename (getin))
(cond (  (null infilename) NIL)
      (   (not (equal infilename 'USAGE-ERROR))
         (setq in (open infilename :direction
                                   :input)
         )
      )
      ( T (print "usage : dsc_lisp.* lisp.prog out.file [in.file]"))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                        end of interface preface                        ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;; At this point load in your DSC LISP interface functions, and source    ;;;
;;; code. Then execute them from this file.                                ;;;
;;;                                                                        ;;;
;;;  Remember: The output file pointer is in ..... in                      ;;;
;;;            The input file pointer is in  ..... out                     ;;;
;;;                                                                        ;;;
;;; Do not reset the universal-error-handler if you are using linked       ;;;
;;; function calls (ie dscpr_sub ... )                                     ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                           YOUR PROGRAM GOES HERE                       ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(setq x (read in))
(setq y (read in))

  (cond ( (= odebug 1)
          (format t "one.l    : main             : inputs ~a ~a ~%" x y)
        )
  )

(print (+ x y ) out)

  (cond ( (= odebug 1)
          (format t "one.l    : main             : done ~%")
        )
  )

) ;End of progn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;             NO MORE CODE AFTER THE END OF PROGN PARENTHESIS            ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

