;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                          one_level_driver.lsp                          ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(progn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;; Invocation                                                             ;;;
;;;    dsc_lisp.int lispprog outfile infile                                ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                          debugging hooks                               ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq oddebug nil)
(setq oddebug 1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                             globals                                    ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; in               - input file pointer  (si:argv 2)
;;; out              - output file pointer (si:argv 1)
;;; oddebug          - one level driver debugging hook 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                          interface preface                             ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;                      ;;;
;;; get output file name ;;;
;;;                      ;;;
(defun getout () 
  (cond ( (or ( = (si:argc) 2)
              ( = (si:argc) 3)) 
          (string-downcase  (si:argv 1)) 
        )
        (T NIL)
  )
)

;;;                     ;;;
;;; get input file name ;;;
;;;                     ;;;
(defun getin ()
 (cond ( ( = (si:argc) 3) 
         (string-downcase (si:argv 2))
       ) 
       ( (not (>= (si:argc) 2)) 'USAGE-ERROR 
       )
      (T NIL)
  ) 
)

;;;                                             ;;;
;;; set output filename and pointer if possible ;;;
;;;                                             ;;;
(setq outfilename (getout))
(cond ( (not (null outfilename))
         (setq out (open outfilename :direction
                                     :output
                                     :if-exists :overwrite
                                     :if-does-not-exist :create)
         )
       )
       ( T (print "usage : dsc_lisp.* lisp.prog out.file [in.file]"))
)

;;;                                            ;;;
;;; set input filename and pointer if possible ;;;
;;;                                            ;;;
(setq infilename (getin))
(cond (  (null infilename) NIL)
      (   (not (equal infilename 'USAGE-ERROR))
         (setq in (open infilename :direction
                                   :input)
         )
      )
      ( T (print "usage : dsc_lisp.* lisp.prog out.file [in.file]"))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                        end of interface preface                        ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;; At this point load in your DSC LISP interface functions, and source    ;;;
;;; code. Then execute them from this file.                                ;;;
;;;                                                                        ;;;
;;;  Remember: The output file pointer is in ..... in                      ;;;
;;;            The input file pointer is in  ..... out                     ;;;
;;;                                                                        ;;;
;;; Do not reset the universal-error-handler if you are using linked       ;;;
;;; function calls (ie dscpr_sub ... )                                     ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                           YOUR PROGRAM GOES HERE                       ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(load "loop.o")
(load "dsc_lisp.lsp") 
(load "dsc_lisp_library.lsp")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;; subroutine: one_level_driver                                           ;;;
;;;                                                                        ;;;
;;; inputs:  x - first integer in one_level_driver.i                       ;;;
;;;          y - second integer in one_level_driver.i                      ;;;
;;;                                                                        ;;;
;;; outputs: source code files, data, input                                ;;;
;;;                                                                        ;;;
;;; returns: none                                                          ;;;
;;;                                                                        ;;;
;;; purpose: distribute lisp soubproblems which sum two numbers,           ;;;
;;;          compute the sum of the sums calcualted by the sub tasks       ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun one_level_driver (x y)
 (let (   
      (tmp 0)        ;;; temporary varibale
      (r 0)          ;;; sum of subproblem data files
      (name "one")   ;;; source file prefix
      (tname "")     ;;; source file with extentions (i,lsp)
      (n 5)          ;;; number of subproblems
      (index -1)     ;;; DSC Lisp function return values
      )

   (cond ( (= oddebug 1)
           (format t "driver.l : main             : x ~a~%" x)
           (format t "driver.l : main             : y ~a~%" y)
           (format t "driver.l : main             : creating .i files~%")
         )
   )

  ;;;                       ;;;
  ;;; Create "one#.i" files ;;;
  ;;;                       ;;; 
  (loop for i from 1 to  n  by 1
   do
    (setq tmp (list 1 i))       do
    (setq tname name)           do 
    (setq tname (dsc_strcat tname (dsc_int-str i))) do
    (setq tname (dsc_strcat tname ".i"))            do
    (setq tout (open tname :direction
                          :output
                          :if-exists :supersede
                          :if-does-not-exist :create
              )
    )                     do
    (print  x tout)       do
    (print  y tout)       do
    (close tout)
  ) ; End loop

   (cond ( (= oddebug 1)
           (format t "driver.l : main             : creating .lsp files~%")
         )
   )

  ;;;                         ;;;
  ;;; Create "one#.lsp" files ;;;
  ;;;                         ;;;
  (setq filelist '())
  (loop for i from 1 to n by 1
   do
   (setq tname name)                               do
   (setq tname (dsc_strcat tname (dsc_int-str i))) do
   (setq tname (dsc_strcat tname ".lsp"))          do
   (setq filelist (append filelist (list tname)))
  ) ; End loop
  (dsc_copysub "one_level.lsp" filelist)

  ;;;                   ;;; 
  ;;; start DSC debuger ;;;
  ;;;                   ;;;
  (setq index (dscdbg_start))

   (cond ( (= oddebug 1)
           (format t "driver.l : main             : debugging index ~a ~%" 
            index)
           (format t "driver.l : main             : distributing sub tasks~%")
         )
   )

  ;;;                      ;;;
  ;;; Distribute Additions ;;;
  ;;;                      ;;; 
  (loop for i from 1 to n by 1
   do
   (setq fid "/fs5/grads4/diaza/sparc/")         do
   (setq fid (dsc_strcat fid name))              do
   (setq fid (dsc_strcat fid (dsc_int-str i)))   do
   (setq ffid (dsc_strcat name (dsc_int-str i))) do
   (setq dotlisp (dsc_strcat ffid ".lsp "))      do
   (setq dotd (dsc_strcat ffid ".d "))           do
   (setq doti (dsc_strcat ffid ".i"))            do
   (setq exec "dsc_lisp.int ")                   do
   (setq exec (dsc_strcat exec dotlisp))         do
   (setq exec (dsc_strcat exec dotd))            do
   (setq exec (dsc_strcat exec doti))            do
   (setq index (dscpr_sub fid "li" "dsc_lisp.build" exec low_cpu 5))
   
   (cond ( (= oddebug 1)
           (format t "driver.l : main             : fid ~a ~%" fid)
           (format t "driver.l : main             : index ~a ~%" index) 
         )
   )
 
  ) ; End loop

  ;;;             ;;;
  ;;; Wait on all ;;;
  ;;;             ;;;
  (setq index (dscpr_wait -1))

   (cond ( (= oddebug 1)
           (format t "driver.l : main             : wait index ~a ~%"
            index)
           (format t "driver.l : main             : collecting data files~%")
         )
   )

  ;;;
  ;;; Collect data files
  ;;;
  (loop for i from 1 to n by 1
   do
   (setq fid name)                              do
   (setq fid (dsc_strcat fid (dsc_int-str i)))  do
   (setq fid (dsc_strcat fid ".d"))             do
   (setq tin (open fid :direction
                       :input
             )
   )                                            do
   (setq tmp (read tin))                        do
   (close tin)                                  do
   (setq r ( + tmp r))                          do
  ) ; End loop

  (cond ( (= oddebug 1)
          (format t "driver.l : main             : sum ~a ~%" r)
        )
  )

  (setq r r)
 ) ; End let
) ; End one_level_driver

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;                            automatic startup                           ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 ;;;                                                ;;;
 ;;; read input file with (read in)                 ;;;
 ;;; write to output file with (print VARIABLE out) ;;;
 ;;;                                                ;;; 
 (setq arg1 (read in))
 (setq arg2 (read in))

  (cond ( (= oddebug 1)
          (format t "driver.l : main             : inputs ~a ~a ~%" arg1 arg2)
        )
  )

 (setq vari (one_level_driver arg1 arg2))
 (print vari out)

) ;End of progn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                        ;;;
;;;             NO MORE CODE AFTER THE END OF PROGN PARENTHESIS            ;;;
;;;                                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
