##############################################################################
#                                                                            #
#                              one_level.mpl                                 #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
#                                   globals                                  #
#                                                                            #
##############################################################################

# DSC_IFILE - input file name
# DSC_OFILE - output file name

##############################################################################
#                                                                            #
#                               debugging hooks                              #
#                                                                            #
##############################################################################

DSC_M_ONE  := 'DSC_M_ONE';
DSC_M_ONE  := 1;

##############################################################################
#                                                                            #
# subroutine: one_level                                                      #
#                                                                            #
# inputs:  one_input - input file for subproblem                             #
#          one_output - output file for subproblem                           #
#                                                                            #
# outputs: data file                                                         #
#                                                                            #
# return:  none                                                              #
#                                                                            #
# purpose: read the input from one_input, perform a sum of the two integers  #
#          place the result in the one_output file.                          #
#                                                                            #
##############################################################################

one_level:= proc(one_input,one_output)
 local commandfile,  # dsc_maple commandfile name
       messagepid,   # dsc_maple_message pid file name
       dscmaplepid,  # dsc_maple pid file name
       answerfile,   # dsc_maple answer file name
       msgstr,       # system command string
       PWD;          # current working directory

 if DSC_M_ONE = 1 then
  printf(`one      : main             : one_input %s\n`,one_input);
  printf(`one      : main             : one_outpu %s\n`,one_output);
 fi;

 #                                  #
 # Read in DSC Maple User Functions #
 #                                  #
 read(`dsc_maple.mpl`);

 if DSC_M_ONE = 1 then
  printf(`one      : main             : starting dsc_maple deamon\n`);
 fi;

 #                        #
 # Start dsc_maple deamon #
 #                        #
 commandfile := cat(one_input,`.cmd`);
 messagepid  := cat(one_input,`.mpid`);
 dscmaplepid := cat(one_input,`.ppid`);
 answerfile  := cat(one_input,`.ans`);
 msgstr := cat(`dsc_maple `,commandfile,` `,messagepid,` `,dscmaplepid,` &`);
 system(msgstr);

 #                          #
 # Wait a while for startup #
 #                          #
 system(`sleep 2`);

 #                           #
 # Get real path information #
 #                           #
 PWD := dscpr_pwd(commandfile,messagepid,dscmaplepid,answerfile);

 if DSC_M_ONE = 1 then
  printf(`one      : main             : PWD %s\n`,PWD);
 fi;

 #                       #
 # Kill dsc_maple_deamon #
 #                       #
 dscpr_maple_kill(commandfile,messagepid,dscmaplepid,PWD);

 #                            #
 # Wait a while for shut down #
 #                            #
 system(`sleep 2`);

 #              #
 # Remove files #
 #              #
 msgstr := `rm -f `; 
 msgstr := cat(msgstr,commandfile); 
 system(msgstr);
 msgstr := `rm -f `; 
 msgstr := cat(msgstr,messagepid); 
 system(msgstr);
 msgstr := `rm -f `;  
 msgstr := cat(msgstr,dscmaplepid); 
 system(msgstr);
 msgstr := `rm -f `;  
 msgstr := cat(msgstr,answerfile); 
 system(msgstr);

 if DSC_M_ONE = 1 then
  printf(`one      : main             : reading and summing data file\n`);
 fi;
 
 #                             #
 # Read and sum from one_input #
 #                             #
 msgstr := PWD;
 msgstr := cat(msgstr,`/`,one_input);

 #                                       #
 # Read in I1 and I2 from one_input file #
 #                                       #
 read( msgstr );
 sum := I1 + I2;

 save sum, one_output; 

 if DSC_M_ONE = 1 then
  printf(`one      : main             : exiting!\n`);
 fi;

end;

one_level(DSC_IFILE,DSC_OFILE);
quit;
